@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,20 +16,19 @@ Animation class for importers and exporters.
 :date:   12.11.2019
 """
 
-from PySide2.QtGui import QPainterPath, QFont, QFontMetrics
-from PySide2.QtCore import Qt, Signal, Slot, QObject, QTimeLine, QRectF, QPointF, QLineF
-from PySide2.QtWidgets import QGraphicsPathItem
+from PySide6.QtGui import QPainterPath, QFont, QFontMetrics
+from PySide6.QtCore import Qt, Signal, Slot, QObject, QTimeLine, QRectF, QPointF, QLineF
+from PySide6.QtWidgets import QGraphicsPathItem
 from spinetoolbox.helpers import color_from_index
 
 
 class AnimationSignaller(QObject):
-
     animation_started = Signal()
     animation_stopped = Signal()
 
 
 class ImporterExporterAnimation:
-    def __init__(self, item, duration=1800, plane_count=4, point_size=10, loop_width=30, loop_aspect_ratio=3):
+    def __init__(self, item, duration=1800, plane_count=4, pixel_size=12, loop_width=30, loop_aspect_ratio=3):
         """Initializes animation stuff.
 
         Args:
@@ -42,7 +41,7 @@ class ImporterExporterAnimation:
         self._loop_width = loop_width
         self._loop_height = loop_width / loop_aspect_ratio
         self._font = QFont("Font Awesome 5 Free Solid")
-        self._font.setPointSize(point_size)
+        self._font.setPixelSize(pixel_size)
         self.time_line = QTimeLine()
         self.time_line.setDuration(duration)
         self.time_line.setLoopCount(0)  # loop forever
@@ -110,7 +109,7 @@ class ImporterExporterAnimation:
             ]
         for k, plane in enumerate(self._planes):
             plane.color = color_from_index(k, len(self._planes))
-        if new_state == QTimeLine.NotRunning:
+        if new_state == QTimeLine.State.NotRunning:
             for plane in self._planes:
                 plane.wipe_out()
             self._planes.clear()
@@ -118,7 +117,7 @@ class ImporterExporterAnimation:
     @Slot()
     def start(self):
         """Starts the animation."""
-        if self.time_line.state() == QTimeLine.Running:
+        if self.time_line.state() == QTimeLine.State.Running:
             return
         self.time_line.start()
 
