@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -39,27 +39,6 @@ class UpdateCancelOnErrorCommand(SpineToolboxCommand):
         self._project_item.set_cancel_on_error(self._undo_cancel_on_error)
 
 
-class UpdatePurgeBeforeWritingCommand(SpineToolboxCommand):
-    def __init__(self, project_item, purge_before_writing):
-        """Command to update purge before running setting.
-
-        Args:
-            project_item (ProjectItem): Item
-            purge_before_writing (bool): New setting
-        """
-        super().__init__()
-        self._project_item = project_item
-        self._redo_purge_before_writing = purge_before_writing
-        self._undo_purge_before_writing = not purge_before_writing
-        self.setText(f"change {project_item.name} purge before running setting")
-
-    def redo(self):
-        self._project_item.set_purge_before_writing(self._redo_purge_before_writing)
-
-    def undo(self):
-        self._project_item.set_purge_before_writing(self._undo_purge_before_writing)
-
-
 class UpdateOnConflictCommand(SpineToolboxCommand):
     def __init__(self, project_item, on_conflict):
         """Command to update Importer and Merger 'on conflict' setting.
@@ -144,48 +123,3 @@ class UpdateGroupIdCommand(SpineToolboxCommand):
 
     def undo(self):
         self._item.do_set_group_id(self._undo_group_id)
-
-
-class UpdateOutFileName(SpineToolboxCommand):
-    """Command to update exporter's output file name."""
-
-    def __init__(self, exporter, file_name, database_path):
-        """
-        Args:
-            exporter (ExporterBase): exporter
-            file_name (str): the output filename
-            database_path (str): the associated db path
-        """
-        super().__init__()
-        self.exporter = exporter
-        self.redo_file_name = file_name
-        self.undo_file_name = self.exporter.database(database_path).output_file_name
-        self.database_path = database_path
-        self.setText(f"change output file in {exporter.name}")
-
-    def redo(self):
-        self.exporter.set_out_file_name(self.redo_file_name, self.database_path)
-
-    def undo(self):
-        self.exporter.set_out_file_name(self.undo_file_name, self.database_path)
-
-
-class UpdateOutputTimeStampsFlag(SpineToolboxCommand):
-    """Command to set exporter's output directory time stamps flag."""
-
-    def __init__(self, exporter, value):
-        """
-        Args:
-            exporter (ExporterBase): exporter item
-            value (bool): flag's new value
-        """
-        super().__init__()
-        self.setText(f"toggle output time stamps setting of {exporter.name}")
-        self._exporter = exporter
-        self._value = value
-
-    def redo(self):
-        self._exporter.set_output_time_stamps_flag(self._value)
-
-    def undo(self):
-        self._exporter.set_output_time_stamps_flag(not self._value)
