@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -19,23 +19,31 @@ Module for data connection class.
 import os
 import shutil
 import logging
-from sqlalchemy import create_engine
-from PySide2.QtCore import Slot, Qt, QFileInfo, QModelIndex, QItemSelection
-from PySide2.QtGui import QStandardItem, QStandardItemModel, QBrush
-from PySide2.QtWidgets import QFileDialog, QGraphicsItem, QFileIconProvider, QInputDialog, QMessageBox
+from PySide6.QtCore import Slot, Qt, QFileInfo, QModelIndex, QItemSelection, QTimer
+from PySide6.QtGui import QStandardItem, QStandardItemModel, QBrush
+from PySide6.QtWidgets import QFileDialog, QGraphicsItem, QFileIconProvider, QInputDialog, QMessageBox
 from spine_engine.utils.serialization import deserialize_path, serialize_path
 from spinetoolbox.project_item.project_item import ProjectItem
-from spinetoolbox.helpers import open_url
+from spinetoolbox.widgets.custom_qwidgets import ToolBarWidget
+from spinetoolbox.helpers import open_url, same_path
 from spinetoolbox.config import INVALID_FILENAME_CHARS
 from .commands import AddDCReferencesCommand, RemoveDCReferencesCommand, MoveReferenceToData
 from .custom_file_system_watcher import CustomFileSystemWatcher
 from .executable_item import ExecutableItem
 from .item_info import ItemInfo
 from .output_resources import scan_for_resources
-from ..widgets import UrlSelector
+from ..database_validation import DatabaseConnectionValidator
+from ..widgets import UrlSelectorDialog
 from ..utils import split_url_credentials
 
 
+_DATA_FILE_PATH_ROLE = Qt.ItemDataRole.UserRole + 1
+_MISSING_ROLE = Qt.ItemDataRole.UserRole + 2
+
+
+_MISSING_ITEM_FOREGROUND = QBrush(Qt.red)
+
+
 class DataConnection(ProjectItem):
     def __init__(
         self, name, description, x, y, toolbox, project, file_references=None, db_references=None, db_credentials=None
@@ -61,19 +69,23 @@ class DataConnection(ProjectItem):
         if db_credentials is None:
             db_credentials = dict()
         self._toolbox = toolbox
-        self.reference_model = QStandardItemModel()  # References to files
+        self.reference_model = QStandardItemModel()  # References
         self.data_model = QStandardItemModel()  # Paths of project internal files. These are found in DC data directory
         self.file_system_watcher = None
         self.file_references = list(file_references)
         self.db_references = list(db_references)
         self.db_credentials = db_credentials
-        self._file_ref_root = None
-        self._db_ref_root = None
+        self._file_ref_root = QStandardItem("File paths")
+        self._file_ref_root.setFlags(self._file_ref_root.flags() & ~Qt.ItemIsEditable)
+        self._db_ref_root = QStandardItem("URLs")
+        self._db_ref_root.setFlags(self._file_ref_root.flags() & ~Qt.ItemIsEditable)
         self.file_refs_selected = False
         self.any_refs_selected = False
         self.any_data_selected = False
+        self.current_is_file_ref = False
         self.populate_reference_list()
         self.populate_data_list()
+        self._database_validator = DatabaseConnectionValidator()
 
     def set_up(self):
         super().set_up()
@@ -117,8 +129,6 @@ class DataConnection(ProjectItem):
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
         # pylint: disable=unnecessary-lambda
-        s[self._properties_ui.toolButton_plus_file.clicked] = self.show_add_file_references_dialog
-        s[self._properties_ui.toolButton_plus_url.clicked] = self.show_add_db_reference_dialog
         s[self._properties_ui.toolButton_minus.clicked] = self.remove_references
         s[self._properties_ui.toolButton_add.clicked] = self.copy_to_project
         s[self._properties_ui.treeView_dc_references.doubleClicked] = self.open_reference
@@ -126,10 +136,12 @@ class DataConnection(ProjectItem):
         s[self._properties_ui.treeView_dc_references.files_dropped] = self._add_file_references
         s[self._properties_ui.treeView_dc_data.files_dropped] = self.add_data_files
         s[self.get_icon().files_dropped_on_icon] = self.receive_files_dropped_on_icon
-        s[self._properties_ui.treeView_dc_references.del_key_pressed] = lambda: self.remove_references
+        s[self._properties_ui.treeView_dc_references.del_key_pressed] = lambda b=False: self.remove_references(b)
         s[self._properties_ui.treeView_dc_data.del_key_pressed] = self.remove_files
         s[self._properties_ui.treeView_dc_references.selectionModel().selectionChanged] = self._update_selection_state
         s[self._properties_ui.treeView_dc_data.selectionModel().selectionChanged] = self._update_selection_state
+        s[self._properties_ui.action_new_file_reference.triggered] = self.show_add_file_references_dialog
+        s[self._properties_ui.action_new_db_reference.triggered] = self.show_add_db_reference_dialog
         return s
 
     def restore_selections(self):
@@ -138,6 +150,18 @@ class DataConnection(ProjectItem):
         self._properties_ui.treeView_dc_data.setModel(self.data_model)
         self._do_update_selection_state()
         self._properties_ui.treeView_dc_references.expandAll()
+        file_ref_root_widget = ToolBarWidget("File paths")
+        file_ref_root_widget.tool_bar.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        file_ref_root_widget.tool_bar.addAction(self._properties_ui.action_new_file_reference)
+        self._properties_ui.treeView_dc_references.setIndexWidget(
+            self.reference_model.indexFromItem(self._file_ref_root), file_ref_root_widget
+        )
+        db_ref_root_widget = ToolBarWidget("URLs")
+        db_ref_root_widget.tool_bar.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        db_ref_root_widget.tool_bar.addAction(self._properties_ui.action_new_db_reference)
+        self._properties_ui.treeView_dc_references.setIndexWidget(
+            self.reference_model.indexFromItem(self._db_ref_root), db_ref_root_widget
+        )
 
     @Slot(QGraphicsItem, list)
     def receive_files_dropped_on_icon(self, icon, file_paths):
@@ -158,7 +182,7 @@ class DataConnection(ProjectItem):
                 self._logger.msg_error.emit("[OSError] Copying failed")
 
     @Slot(bool)
-    def show_add_file_references_dialog(self, checked=False):
+    def show_add_file_references_dialog(self, _=False):
         """Opens a file browser where user can select files to be added as references for this Data Connection."""
         answer = QFileDialog.getOpenFileNames(self._toolbox, "Add file references", self._project.project_dir, "*.*")
         file_paths = answer[0]
@@ -175,10 +199,11 @@ class DataConnection(ProjectItem):
         """
         repeated_paths = []
         new_paths = []
+        existing_references = (reference for reference in self.file_references if os.path.isfile(reference))
         for path in paths:
             if not os.path.isfile(path):
                 continue
-            if any(os.path.samefile(path, ref) for ref in self.file_references):
+            if any(os.path.samefile(path, ref) for ref in existing_references):
                 repeated_paths.append(path)
             else:
                 new_paths.append(path)
@@ -189,27 +214,32 @@ class DataConnection(ProjectItem):
             self._toolbox.undo_stack.push(AddDCReferencesCommand(self, new_paths, []))
 
     @Slot(bool)
-    def show_add_db_reference_dialog(self, checked=False):
+    def show_add_db_reference_dialog(self, _=False):
         """Opens a dialog where user can select a url to be added as reference for this Data Connection."""
-        selector = UrlSelector(self._toolbox, self._toolbox)
-        selector.exec_()
+        selector = UrlSelectorDialog(self._toolbox.qsettings(), self._toolbox, self._toolbox)
+        selector.exec()
         url = selector.url
         if not url:  # Cancel button clicked
             return
-        try:
-            engine = create_engine(url)
-            with engine.connect():
-                pass
-        except Exception as e:  # pylint: disable=broad-except
-            self._logger.msg_error.emit(f"Unable to connect to <b>{url}</b>: {e}")
-            return
         url, credentials = split_url_credentials(url)
         if url in self.db_references:
             self._logger.msg_warning.emit(f"Reference to database <b>{url}</b> already exists")
             return
+        self._database_validator.validate_url(
+            selector.dialect, url, self._log_database_reference_error, success_slot=None
+        )
         self.db_credentials[url] = credentials
         self._toolbox.undo_stack.push(AddDCReferencesCommand(self, [], [url]))
 
+    @Slot(str)
+    def _log_database_reference_error(self, error):
+        """Logs final database validation error messages.
+
+        Args:
+            error (str): message
+        """
+        self._logger.msg_error.emit(f"<b>{self.name}</b>: invalid database URL: {error}")
+
     def do_add_references(self, file_refs, db_refs):
         file_refs = [os.path.abspath(ref) for ref in file_refs]
         self.file_references += file_refs
@@ -221,7 +251,7 @@ class DataConnection(ProjectItem):
         self._resources_to_successors_changed()
 
     @Slot(bool)
-    def remove_references(self, checked=False):
+    def remove_references(self, _=False):
         """Pushes a remove references command to undo stack"""
         indexes = self._properties_ui.treeView_dc_references.selectedIndexes()
         if not indexes:  # Nothing selected
@@ -236,9 +266,9 @@ class DataConnection(ProjectItem):
             if not parent.isValid():
                 continue
             if parent == file_ref_root_index:
-                file_references.append(index.data(Qt.DisplayRole))
+                file_references.append(index.data(Qt.ItemDataRole.DisplayRole))
             elif parent == db_ref_root_index:
-                db_references.append(index.data(Qt.DisplayRole))
+                db_references.append(index.data(Qt.ItemDataRole.DisplayRole))
         self._toolbox.undo_stack.push(RemoveDCReferencesCommand(self, file_references, db_references))
         self._logger.msg.emit("Selected references removed")
 
@@ -259,7 +289,7 @@ class DataConnection(ProjectItem):
     def _remove_file_references(self, *refs):
         result = False
         for k in reversed(range(self._file_ref_root.rowCount())):
-            if any(_samepath(self._file_ref_root.child(k).text(), ref) for ref in refs):
+            if any(same_path(self._file_ref_root.child(k).text(), ref) for ref in refs):
                 self.file_references.pop(k)
                 self._file_ref_root.removeRow(k)
                 result = True
@@ -275,19 +305,10 @@ class DataConnection(ProjectItem):
                 result = True
         return result
 
-    def _rename_file_reference(self, old_path, new_path):
-        for k in range(self._file_ref_root.rowCount()):
-            item = self._file_ref_root.child(k)
-            if _samepath(item.text(), old_path):
-                item.setText(new_path)
-                self.file_references[k] = new_path
-                return True
-        return False
-
     def _remove_data_file(self, path):
         for k in reversed(range(self.data_model.rowCount())):
-            data_filepath = self.data_model.item(k).data(Qt.UserRole)
-            if _samepath(data_filepath, path):
+            data_filepath = self.data_model.item(k).data(_DATA_FILE_PATH_ROLE)
+            if same_path(data_filepath, path):
                 self.data_model.removeRow(k)
                 return True
         return False
@@ -295,20 +316,73 @@ class DataConnection(ProjectItem):
     def _rename_data_file(self, old_path, new_path):
         for k in range(self.data_model.rowCount()):
             item = self.data_model.item(k)
-            if _samepath(item.data(Qt.UserRole), old_path):
+            if same_path(item.data(_DATA_FILE_PATH_ROLE), old_path):
                 item.setText(os.path.basename(new_path))
-                item.setData(new_path, Qt.UserRole)
+                item.setData(new_path, _DATA_FILE_PATH_ROLE)
                 return True
         return False
 
+    def copy_local_data(self, item_dict):
+        """See base class."""
+        original_data_dir = item_dict.get("original_data_dir")  # (str) original dir of duplicated ProjectItem
+        duplicate_files = item_dict.get("duplicate_files")  # (bool) Flag indicating if linked files should be copied
+        if not original_data_dir and not duplicate_files:
+            return
+        if duplicate_files:
+            try:
+                os.makedirs(self.data_dir, exist_ok=True)  # Make data dir
+            except OSError as e:
+                self._logger.msg_error.emit(f"Creating directory {self.data_dir} failed. {e}.")
+                return
+            for f in os.listdir(original_data_dir):
+                src_file = os.path.join(original_data_dir, f)
+                if os.path.isfile(src_file):
+                    dst_file = os.path.join(self.data_dir, f)
+                    try:
+                        shutil.copy(src_file, dst_file)  # Copy file
+                    except OSError as e:
+                        self._logger.msg_error.emit(f"Copying file <b>{f}</b> to <b>{self.data_dir}</b> failed. [{e}]")
+                        continue
+                    self._logger.msg.emit(f"File <b>{f}</b> copied to <b>{self.data_dir}</b>")
+        self.populate_data_list()
+
     @Slot(str)
     def _handle_file_removed(self, path):
-        if self._remove_file_references(path) or self._remove_data_file(path):
+        """Marks file reference missing or removes data reference.
+
+        Args:
+            path (str): file path
+        """
+        resources_changed = self._try_to_mark_file_reference_missing(path) or self._remove_data_file(path)
+        if resources_changed:
             self._check_notifications()
             self._resources_to_successors_changed()
 
+    def _try_to_mark_file_reference_missing(self, path):
+        """Marks a file reference as missing if given path is in references.
+
+        Args:
+            path (str): file path
+
+        Returns:
+            bool: True if references was marked missing successfully, False otherwise
+        """
+        for row in range(self._file_ref_root.rowCount()):
+            item = self._file_ref_root.child(row)
+            if same_path(item.text(), path):
+                self._mark_as_missing(item)
+                return True
+        return False
+
     @Slot(str, str)
     def _handle_file_renamed(self, old_path, new_path):
+        """Marks file reference missing or renames data file.
+
+        Args:
+            old_path (str): original path
+            new_path (str): renamed path
+        """
+
         def replace_new_path(paths):
             for i, path in enumerate(paths):
                 if path == new_path:
@@ -316,10 +390,12 @@ class DataConnection(ProjectItem):
                     return True
             return False
 
-        renamed = self._rename_file_reference(old_path, new_path)
-        if not renamed:
-            renamed = self._rename_data_file(old_path, new_path)
-        if not renamed:
+        if self._try_to_mark_file_reference_missing(old_path):
+            self._refresh_file_reference_delayed(old_path)
+            self._check_notifications()
+            self._resources_to_successors_changed()
+            return
+        if not self._rename_data_file(old_path, new_path):
             return
         self._check_notifications()
         file_refs = list(self.file_references)
@@ -334,15 +410,46 @@ class DataConnection(ProjectItem):
         )
         self._resources_to_successors_replaced(old_resources, new_resources)
 
+    def _refresh_file_reference_delayed(self, path):
+        """Checks if given file exists after certain delay.
+
+        Args:
+            path (str): file path
+        """
+
+        # Some software saves files by renaming them and then creating
+        # a new file with the same name for safety reasons.
+        # So, sometimes a file goes "missing" for a moment and then comes back.
+        # We'll try to refresh the file reference once in this case.
+        def refresh():
+            if not os.path.exists(path):
+                return
+            fixed_references = []
+            for row in range(self._file_ref_root.rowCount()):
+                item = self._file_ref_root.child(row, 0)
+                if not same_path(path, item.text()):
+                    continue
+                fixed_references.append(path)
+                item.clearData()
+                item.setFlags(~Qt.ItemIsEditable)
+                item.setData(path, Qt.ItemDataRole.DisplayRole)
+            if not fixed_references:
+                return
+            self.file_system_watcher.add_persistent_file_paths(ref for ref in fixed_references)
+            self._check_notifications()
+            self._resources_to_successors_changed()
+
+        QTimer.singleShot(1000, refresh)
+
     @Slot(str)
     def _handle_file_added(self, path):
-        if _samepath(os.path.dirname(path), self.data_dir):
+        if same_path(os.path.dirname(path), self.data_dir):
             self._append_data_files_to_model(path)
             self._check_notifications()
             self._resources_to_successors_changed()
 
     @Slot(bool)
-    def copy_to_project(self, checked=False):
+    def copy_to_project(self, _=False):
         """Copy selected file references to this Data Connection's data directory."""
         selected_indexes = self._properties_ui.treeView_dc_references.selectedIndexes()
         if not selected_indexes:
@@ -367,6 +474,28 @@ class DataConnection(ProjectItem):
             except OSError:
                 self._logger.msg_error.emit("[OSError] Copying failed")
 
+    def refresh_references(self):
+        """Checks if missing file references have somehow come back to life."""
+        selected_indexes = self._properties_ui.treeView_dc_references.selectedIndexes()
+        fixed_references = []
+        if not selected_indexes:
+            return
+        for index in selected_indexes:
+            if self.reference_model.itemFromIndex(index.parent()) is not self._file_ref_root:
+                continue
+            item = self.reference_model.itemFromIndex(index)
+            path = item.data(Qt.ItemDataRole.DisplayRole)
+            if item.data(_MISSING_ROLE) and os.path.exists(path):
+                fixed_references.append(path)
+                item.clearData()
+                item.setFlags(~Qt.ItemIsEditable)
+                item.setData(path, Qt.ItemDataRole.DisplayRole)
+        if not fixed_references:
+            return
+        self.file_system_watcher.add_persistent_file_paths(ref for ref in fixed_references)
+        self._check_notifications()
+        self._resources_to_successors_changed()
+
     @Slot(QModelIndex)
     def open_reference(self, index):
         """Open reference in default program."""
@@ -392,7 +521,7 @@ class DataConnection(ProjectItem):
         if not index.isValid():
             logging.error("Index not valid")
             return
-        data_file = index.data(Qt.UserRole)
+        data_file = index.data(_DATA_FILE_PATH_ROLE)
         url = "file:///" + data_file
         # noinspection PyTypeChecker, PyCallByClass, PyArgumentList
         res = open_url(url)
@@ -438,11 +567,15 @@ class DataConnection(ProjectItem):
         msg = "The following files will be removed permanently from the project\n\n" f"{files}\n\n" "Are you sure?"
         title = f"Remove {len(file_list)} File(s)"
         message_box = QMessageBox(
-            QMessageBox.Question, title, msg, QMessageBox.Ok | QMessageBox.Cancel, parent=self._toolbox
+            QMessageBox.Icon.Question,
+            title,
+            msg,
+            QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel,
+            parent=self._toolbox,
         )
-        message_box.button(QMessageBox.Ok).setText("Remove Files")
-        answer = message_box.exec_()
-        if answer == QMessageBox.Cancel:
+        message_box.button(QMessageBox.StandardButton.Ok).setText("Remove Files")
+        answer = message_box.exec()
+        if answer == QMessageBox.StandardButton.Cancel:
             return
         self.delete_files_from_project(file_list)
 
@@ -464,25 +597,38 @@ class DataConnection(ProjectItem):
         """List file references in QTreeView."""
         self.reference_model.clear()
         self.reference_model.setHorizontalHeaderItem(0, QStandardItem("References"))  # Add header
-        self._file_ref_root = QStandardItem("File paths")
-        self._db_ref_root = QStandardItem("URLs")
-        self._file_ref_root.setFlags(self._file_ref_root.flags() & ~Qt.ItemIsEditable)
-        self._db_ref_root.setFlags(self._file_ref_root.flags() & ~Qt.ItemIsEditable)
+        self._file_ref_root.removeRows(0, self._file_ref_root.rowCount())
         self.reference_model.appendRow(self._file_ref_root)
+        self._db_ref_root.removeRows(0, self._db_ref_root.rowCount())
         self.reference_model.appendRow(self._db_ref_root)
         self._append_file_references_to_model(*self.file_references)
         self._append_db_references_to_model(*self.db_references)
 
     def _append_file_references_to_model(self, *paths):
+        non_existent_paths = []
         for path in paths:
             item = QStandardItem(path)
             item.setFlags(~Qt.ItemIsEditable)
             if not os.path.exists(path):
-                self._logger.msg_error.emit(f"<b>{self.name}:</b> Could not find file reference {path}.")
-                tooltip = "The file is missing."
-                item.setData(tooltip, Qt.ToolTipRole)
-                item.setData(QBrush(Qt.red), Qt.ForegroundRole)
+                non_existent_paths.append(path)
+                self._mark_as_missing(item)
             self._file_ref_root.appendRow(item)
+        if non_existent_paths:
+            msg = f"<b>{self.name}:</b> Could not find file references:"
+            for path in non_existent_paths:
+                msg += f"<br><b>{os.path.basename(path)}</b>"
+            self._logger.msg_error.emit(msg)
+
+    @staticmethod
+    def _mark_as_missing(item):
+        """Modifies given model item to appear as missing reference.
+
+        Args:
+            item (QStandardItem): item to modify
+        """
+        item.setData("The file is missing.", Qt.ItemDataRole.ToolTipRole)
+        item.setData(_MISSING_ITEM_FOREGROUND, Qt.ForegroundRole)
+        item.setData(True, _MISSING_ROLE)
 
     def _append_db_references_to_model(self, *urls):
         for url in urls:
@@ -501,8 +647,8 @@ class DataConnection(ProjectItem):
             item = QStandardItem(os.path.basename(path))
             item.setFlags(~Qt.ItemIsEditable)
             icon = QFileIconProvider().icon(QFileInfo(path))
-            item.setData(icon, Qt.DecorationRole)
-            item.setData(path, Qt.UserRole)
+            item.setData(icon, Qt.ItemDataRole.DecorationRole)
+            item.setData(path, _DATA_FILE_PATH_ROLE)
             self.data_model.appendRow(item)
 
     def resources_for_direct_successors(self):
@@ -562,6 +708,7 @@ class DataConnection(ProjectItem):
         """Tears down this item. Called by toolbox just before closing."""
         super().tear_down()
         self.file_system_watcher.tear_down()
+        self._database_validator.wait_for_finish()
 
     def notify_destination(self, source_item):
         """See base class."""
@@ -575,7 +722,3 @@ class DataConnection(ProjectItem):
             self._logger.msg.emit("Link established")
         else:
             super().notify_destination(source_item)
-
-
-def _samepath(path1, path2):
-    return os.path.normcase(path1) == os.path.normcase(path2)
