@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -18,19 +18,22 @@ Module for data store class.
 
 import os
 from shutil import copyfile
-from PySide2.QtCore import Slot
-from PySide2.QtWidgets import QAction, QFileDialog, QApplication, QMenu
+from PySide6.QtCore import Slot
+from PySide6.QtWidgets import QFileDialog, QApplication, QMenu
+from PySide6.QtGui import QAction
 from spinedb_api.helpers import vacuum
 from spine_engine.project_item.project_item_resource import database_resource
 from spinetoolbox.project_item.project_item import ProjectItem
 from spinetoolbox.helpers import create_dir
 from spinetoolbox.spine_db_editor.widgets.multi_spine_db_editor import MultiSpineDBEditor
 from spine_engine.utils.serialization import serialize_path, deserialize_path
+from spinetoolbox.widgets.custom_qwidgets import SelectDatabaseItemsDialog
 from .commands import UpdateDSURLCommand
 from .executable_item import ExecutableItem
 from .item_info import ItemInfo
 from .output_resources import scan_for_resources
-from ..utils import database_label, convert_to_sqlalchemy_url
+from ..database_validation import DatabaseConnectionValidator
+from ..utils import database_label, convert_to_sqlalchemy_url, check_database_url
 
 
 class DataStore(ProjectItem):
@@ -44,7 +47,7 @@ class DataStore(ProjectItem):
             y (float): Initial Y coordinate of item icon
             toolbox (ToolboxUI): QMainWindow instance
             project (SpineToolboxProject): the project this item belongs to
-            url (str or dict, optional): SQLAlchemy url
+            url (dict, optional): SQLAlchemy url
         """
         super().__init__(name, description, x, y, project)
         self._toolbox = toolbox
@@ -56,11 +59,16 @@ class DataStore(ProjectItem):
         if url is None:
             url = dict()
         self._url = self.parse_url(url)
+        self._url_validated = False
+        self._resource_to_replace = None
         self._multi_db_editors_open = {}
         self._open_url_menu = QMenu("Open URL in Spine DB editor", self._toolbox)
         self._open_url_action = QAction("Open URL in Spine DB editor")
         self._open_url_menu.triggered.connect(self._handle_open_url_menu_triggered)
         self._open_url_action.triggered.connect(self.open_url_in_spine_db_editor)
+        self._purge_settings = None
+        self._purge_dialog = None
+        self._database_validator = DatabaseConnectionValidator(self)
 
     @staticmethod
     def item_type():
@@ -98,18 +106,12 @@ class DataStore(ProjectItem):
         """Returns a dictionary of all shared signals and their handlers.
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
+        s[self._properties_ui.url_selector_widget.url_changed] = self._update_url_from_properties
         s[self._properties_ui.pushButton_ds_open_editor.clicked] = self.open_url_in_spine_db_editor
         s[self._properties_ui.pushButton_create_new_spine_db.clicked] = self.create_new_spine_database
         s[self._properties_ui.toolButton_copy_url.clicked] = self.copy_url
         s[self._properties_ui.toolButton_vacuum.clicked] = self.vacuum
-        s[self._properties_ui.comboBox_dialect.activated[str]] = self.refresh_dialect
-        s[self._properties_ui.toolButton_select_sqlite_file.clicked] = self.select_sqlite_file
-        s[self._properties_ui.lineEdit_database.file_dropped] = self.set_path_to_sqlite_file
-        s[self._properties_ui.lineEdit_username.editingFinished] = self.refresh_username
-        s[self._properties_ui.lineEdit_password.editingFinished] = self.refresh_password
-        s[self._properties_ui.lineEdit_host.editingFinished] = self.refresh_host
-        s[self._properties_ui.lineEdit_port.editingFinished] = self.refresh_port
-        s[self._properties_ui.lineEdit_database.editingFinished] = self.refresh_database
+        s[self._properties_ui.purge_button.clicked] = self._show_purge_dialog
         return s
 
     def restore_selections(self):
@@ -128,133 +130,124 @@ class DataStore(ProjectItem):
         """Returns current project or None if no project open."""
         return self._project
 
-    @Slot(str)
-    def set_path_to_sqlite_file(self, file_path):
-        """Set path to SQLite file."""
-        abs_path = os.path.abspath(file_path)
-        self.update_url(dialect="sqlite", database=abs_path)
-
-    @Slot(bool)
-    def select_sqlite_file(self, checked=False):
-        """Open file browser where user can select the path to an SQLite
-        file that they want to use."""
+    def select_sqlite_file(self):
+        """Open file browser where user can select the path to an SQLite file that they want to use."""
         candidate_path = os.path.abspath(self._url["database"]) if self._url["database"] else self.data_dir
         answer = QFileDialog.getOpenFileName(self._toolbox, "Select SQLite file", candidate_path)
         file_path = answer[0]
-        if not file_path:  # Cancel button clicked
-            return False
-        self.set_path_to_sqlite_file(file_path)
-        return True
+        return file_path if file_path else None
 
-    def _new_sqlite_file(self, checked=False):
+    def _new_sqlite_file(self):
+        """Shows a file dialog and creates a new sqlite file at the chosen path.
+
+        Returns:
+            bool: True if the file was created successfully, False otherwise
+        """
         candidate_path = os.path.abspath(os.path.join(self.data_dir, self.name + ".sqlite"))
         answer = QFileDialog.getSaveFileName(self._toolbox, "Create SQLite file", candidate_path)
         file_path = answer[0]
         if not file_path:  # Cancel button clicked
             return False
-        self.set_path_to_sqlite_file(file_path)
+        abs_path = os.path.abspath(file_path)
+        url = dict(self._url)
+        url["database"] = abs_path
+        sa_url = convert_to_sqlalchemy_url(url, self.name)
+        self._toolbox.db_mngr.create_new_spine_database(sa_url, self._logger)
+        self.update_url(dialect="sqlite", database=abs_path)
         return True
 
     def load_url_into_selections(self, url):
-        """Load given url attribute into shared widget selections."""
+        """Load given url attribute into shared widget selections.
+
+        Args:
+            url (dict): URL dict
+        """
         if not self._active:
             return
         if not url:
             return
-        dialect = url.get("dialect")
-        host = url.get("host")
-        port = url.get("port")
-        database = url.get("database")
-        username = url.get("username")
-        password = url.get("password")
-        if dialect is not None:
-            self._properties_ui.controller.enable_dialect(dialect)
-            if dialect == "":
-                self._properties_ui.comboBox_dialect.setCurrentIndex(-1)
-            else:
-                self._properties_ui.comboBox_dialect.setCurrentText(dialect)
-        if host is not None:
-            self._properties_ui.lineEdit_host.setText(host)
-        if port is not None:
-            self._properties_ui.lineEdit_port.setText(str(port))
-        if database is not None:
-            if dialect == "sqlite":
-                database = os.path.abspath(database)
-            self._properties_ui.lineEdit_database.setText(database)
-        if username is not None:
-            self._properties_ui.lineEdit_username.setText(username)
-        if password is not None:
-            self._properties_ui.lineEdit_password.setText(password)
+        self._properties_ui.url_selector_widget.set_url(url)
         self._update_actions_enabled()
 
+    @Slot()
+    def _update_url_from_properties(self):
+        """Gets new URL from Properties tab."""
+        url = self._properties_ui.url_selector_widget.url_dict()
+        self.update_url(**url)
+
     def update_url(self, **kwargs):
-        """Set url key to value."""
+        """Sets new URL.
+
+        Args:
+            **kwargs: URL keys and values
+        """
+        invalidating_url = self._url_validated
+        self._url_validated = False
         kwargs = {k: v for k, v in kwargs.items() if v != self._url[k]}
         if not kwargs:
             return False
-        self._toolbox.undo_stack.push(UpdateDSURLCommand(self, **kwargs))
+        self._toolbox.undo_stack.push(UpdateDSURLCommand(self, invalidating_url, **kwargs))
         return True
 
     def do_update_url(self, **kwargs):
         old_url = convert_to_sqlalchemy_url(self._url, self.name)
+        new_dialect = kwargs.get("dialect")
+        if new_dialect == "sqlite":
+            kwargs.update({"username": "", "password": "", "host": "", "port": ""})
         self._url.update(kwargs)
         new_url = convert_to_sqlalchemy_url(self._url, self.name)
-        self.load_url_into_selections(kwargs)
+        self.load_url_into_selections(self._url)
         if old_url and new_url:
-            old = database_resource(self.name, str(old_url), label=database_label(self.name))
-            new = database_resource(self.name, str(new_url), label=database_label(self.name))
-            self._resources_to_predecessors_replaced([old], [new])
-            self._resources_to_successors_replaced([old], [new])
+            self._resource_to_replace = database_resource(
+                self.name, str(old_url), label=database_label(self.name), filterable=True
+            )
         else:
             self._resources_to_predecessors_changed()
             self._resources_to_successors_changed()
         self._check_notifications()
 
     def _update_actions_enabled(self):
-        open_editor_enabled = convert_to_sqlalchemy_url(self._url, self.name) is not None
-        self._open_url_action.setEnabled(open_editor_enabled)
-        self._open_url_menu.setEnabled(open_editor_enabled)
+        url_exists = convert_to_sqlalchemy_url(self._url, self.name) is not None
+        url_valid = url_exists and self._url_validated
+        self._open_url_action.setEnabled(url_valid)
+        self._open_url_menu.setEnabled(url_valid)
         if not self._active:
             return
         is_sqlite = self._url["dialect"].lower() == "sqlite"
-        self._properties_ui.pushButton_ds_open_editor.setEnabled(open_editor_enabled)
-        self._properties_ui.pushButton_create_new_spine_db.setEnabled(is_sqlite or open_editor_enabled)
-        self._properties_ui.toolButton_copy_url.setEnabled(open_editor_enabled)
-        self._properties_ui.toolButton_vacuum.setEnabled(is_sqlite and open_editor_enabled)
+        self._properties_ui.pushButton_ds_open_editor.setEnabled(url_valid)
+        self._properties_ui.pushButton_create_new_spine_db.setEnabled(is_sqlite or url_valid)
+        self._properties_ui.purge_button.setEnabled(url_valid)
+        self._properties_ui.toolButton_copy_url.setEnabled(url_exists)
+        self._properties_ui.toolButton_vacuum.setEnabled(is_sqlite and url_valid)
+
+    @Slot(bool)
+    def _show_purge_dialog(self, _=False):
+        """Shows the Purge dialog."""
+        if self._purge_dialog is not None:
+            self._purge_dialog.raise_()
+            return
+        self._purge_dialog = SelectDatabaseItemsDialog(self._purge_settings, "Purge", self._toolbox)
+        self._purge_dialog.setWindowTitle(f"Purge items from {self.name}")
+        self._purge_dialog.accepted.connect(self._purge)
+        self._purge_dialog.destroyed.connect(self._clean_up_purge_dialog)
+        self._purge_dialog.show()
 
     @Slot()
-    def refresh_host(self):
-        """Refresh host from selections."""
-        host = self._properties_ui.lineEdit_host.text()
-        self.update_url(host=host)
+    def _purge(self):
+        """Purges the database."""
+        self._purge_settings = self._purge_dialog.get_checked_states()
+        db_map = self._toolbox.db_mngr.get_db_map(self.sql_alchemy_url(), self._logger, self.name)
+        if db_map is None:
+            return
+        db_map_purge_data = {db_map: {item_type for item_type, checked in self._purge_settings.items() if checked}}
+        self._toolbox.db_mngr.purge_items(db_map_purge_data)
+        self._toolbox.db_mngr.commit_session("Purge the database.", db_map)
+        self._logger.msg_success.emit(f"<b>{self.name}</b>: Database purged.")
 
     @Slot()
-    def refresh_port(self):
-        """Refresh port from selections."""
-        port = self._properties_ui.lineEdit_port.text()
-        self.update_url(port=port)
-
-    @Slot()
-    def refresh_database(self):
-        """Refresh database from selections."""
-        database = self._properties_ui.lineEdit_database.text()
-        self.update_url(database=database)
-
-    @Slot()
-    def refresh_username(self):
-        """Refresh username from selections."""
-        username = self._properties_ui.lineEdit_username.text()
-        self.update_url(username=username)
-
-    @Slot()
-    def refresh_password(self):
-        """Refresh password from selections."""
-        password = self._properties_ui.lineEdit_password.text()
-        self.update_url(password=password)
-
-    @Slot(str)
-    def refresh_dialect(self, dialect):
-        self.update_url(dialect=dialect)
+    def _clean_up_purge_dialog(self):
+        """Cleans up stuff after Purge dialog has been closed."""
+        self._purge_dialog = None
 
     def actions(self):
         self._multi_db_editors_open = {x.name(): x for x in self._toolbox.db_mngr.get_all_multi_spine_db_editors()}
@@ -279,6 +272,11 @@ class DataStore(ProjectItem):
     @Slot(bool)
     def open_url_in_spine_db_editor(self, checked=False):
         """Opens current url in the Spine database editor."""
+        if not self._url_validated:
+            self._logger.msg_error.emit(
+                f"<b>{self.name}</b> is still validating the database URL or the URL is invalid."
+            )
+            return
         sa_url = self.sql_alchemy_url()
         if sa_url is not None:
             db_url_codenames = {sa_url: self.name}
@@ -286,11 +284,19 @@ class DataStore(ProjectItem):
         self._check_notifications()
 
     def _open_url_in_new_db_editor(self, checked=False):
+        if not self._url_validated:
+            self._logger.msg_error.emit(
+                f"<b>{self.name}</b> is still validating the database URL or the URL is invalid."
+            )
+            return
         sa_url = self.sql_alchemy_url()
         if sa_url is not None:
             MultiSpineDBEditor(self._toolbox.db_mngr, {sa_url: self.name}).show()
 
     def _open_url_in_existing_db_editor(self, db_editor):
+        if not self._url_validated:
+            db_editor.add_message(f"<b>{self.name}</b> is still validating the database URL or the URL is invalid.")
+            return
         sa_url = self.sql_alchemy_url()
         if sa_url is not None:
             db_editor.add_new_tab({sa_url: self.name})
@@ -316,22 +322,68 @@ class DataStore(ProjectItem):
     @Slot(bool)
     def create_new_spine_database(self, checked=False):
         """Create new (empty) Spine database."""
-        # Try to make an url from the current status
-        sa_url = convert_to_sqlalchemy_url(self._url, self.name, self._logger)
-        if not sa_url:
-            if self._url["dialect"] != "sqlite" or not self._new_sqlite_file():
+        if self._url["dialect"] == "sqlite":
+            sqlite_file_creation_successful = self._new_sqlite_file()
+            if not sqlite_file_creation_successful:  # User cancelled
                 return
-            sa_url = convert_to_sqlalchemy_url(self._url, self.name)
-        self._toolbox.db_mngr.create_new_spine_database(sa_url, self._logger)
+        elif self._url["dialect"] == "mysql":
+            sa_url = convert_to_sqlalchemy_url(self._url, self.name, self._logger)
+            if sa_url:
+                self._toolbox.db_mngr.create_new_spine_database(sa_url, self._logger)
+        else:
+            self._logger.msg_error.emit(f"Unknown data store dialect: {self._url['dialect']}.")
         self._check_notifications()
 
     def _check_notifications(self):
         """Updates the SqlAlchemy format URL and checks for notifications"""
-        self.clear_notifications()
-        if convert_to_sqlalchemy_url(self._url, self.name) is None:
+        self._update_actions_enabled()
+        sa_url = convert_to_sqlalchemy_url(self._url, self.name)
+        if sa_url is None:
+            self.clear_notifications()
             self.add_notification(
                 "The URL for this Data Store is not correctly set. Set it in the Data Store Properties panel."
             )
+            return
+        self._database_validator.validate_url(
+            self._url["dialect"], sa_url, self._set_invalid_url_notification, self._accept_url
+        )
+
+    @Slot(str)
+    def _set_invalid_url_notification(self, error_message):
+        """Sets a single notification that warns about broken URL.
+
+        Args:
+            error_message (str): URL failure message
+        """
+        self.clear_notifications()
+        self.add_notification(f"Couldn't connect to the database: {error_message}")
+        if self._resource_to_replace is None:
+            self._resources_to_predecessors_changed()
+            self._resources_to_successors_changed()
+
+    @Slot()
+    def _accept_url(self):
+        """Sets URL as validated and updates advertised resources."""
+        self._url_validated = True
+        self.clear_notifications()
+        if self._resource_to_replace is not None:
+            sa_url = convert_to_sqlalchemy_url(self._url, self.name)
+            new = database_resource(self.name, str(sa_url), label=database_label(self.name), filterable=True)
+            self._resources_to_predecessors_replaced([self._resource_to_replace], [new])
+            self._resources_to_successors_replaced([self._resource_to_replace], [new])
+            self._resource_to_replace = None
+        else:
+            self._resources_to_predecessors_changed()
+            self._resources_to_successors_changed()
+        self._update_actions_enabled()
+
+    def is_url_validated(self):
+        """Tests whether the URL has been validated.
+
+        Returns:
+            bool: True if URL has been validated, False otherwise
+        """
+        return self._url_validated
 
     def item_dict(self):
         """Returns a dictionary corresponding to this item."""
@@ -405,7 +457,7 @@ class DataStore(ProjectItem):
                 f"Link established. Mapped data generated by <b>{source_item.name}</b> will be "
                 f"imported in <b>{self.name}</b> when executing."
             )
-        elif source_item.item_type() in ["Tool", "Gimlet"]:
+        elif source_item.item_type() == "Tool":
             self._logger.msg.emit("Link established")
         elif source_item.item_type() == "Data Transformer":
             self._logger.msg.emit(
@@ -413,7 +465,7 @@ class DataStore(ProjectItem):
                 f"Data transformed by <b>{source_item.name}</b> will be merged into <b>{self.name}</b> upon execution."
             )
         elif source_item.item_type() == "Merger":
-            src_ds_names = ", ".join(x.name for x in source_item.predecesor_data_stores())
+            src_ds_names = ", ".join(x.name for x in source_item.predecessor_data_stores())
             self._logger.msg.emit(
                 "Link established. "
                 f"Data from <b>{src_ds_names}</b> will be merged into <b>{self.name}</b> upon execution."
@@ -423,6 +475,8 @@ class DataStore(ProjectItem):
 
     def resources_for_direct_successors(self):
         """See base class."""
+        if not self._url_validated:
+            return []
         sa_url = convert_to_sqlalchemy_url(self._url, self.name)
         resources = scan_for_resources(self, sa_url)
         return resources
@@ -430,3 +484,8 @@ class DataStore(ProjectItem):
     def resources_for_direct_predecessors(self):
         """See base class."""
         return self.resources_for_direct_successors()
+
+    def tear_down(self):
+        """See base class"""
+        self._database_validator.wait_for_finish()
+        super().tear_down()
