@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -31,4 +31,4 @@ def scan_for_resources(provider, url):
     """
     if not url:
         return list()
-    return [database_resource(provider.name, str(url), label=database_label(provider.name))]
+    return [database_resource(provider.name, str(url), label=database_label(provider.name), filterable=True)]
