@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -28,5 +28,5 @@ def filter_config_path(data_dir):
     Returns:
         str: a path to the config file
     """
-    file_name = "filter_config.json"
+    file_name = ".filter_config.json"
     return str(Path(data_dir, file_name))
