@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -14,20 +14,26 @@ Contains Exporter's undo commands.
 :authors: A. Soininen (VTT)
 :date:    11.12.2020
 """
-from copy import deepcopy
-from PySide2.QtCore import QModelIndex, Qt
-from PySide2.QtWidgets import QUndoCommand
+from copy import copy, deepcopy
+from enum import IntEnum, unique
+from PySide6.QtCore import QModelIndex, Qt
+from PySide6.QtGui import QUndoCommand
 from spinetoolbox.project_commands import SpineToolboxCommand
 from .mvcmodels.mappings_table_model import MappingsTableModel
 
 
+@unique
+class _ID(IntEnum):
+    CHANGE_POSITION = 1
+
+
 class UpdateOutLabel(SpineToolboxCommand):
     """Command to update exporter's output label."""
 
     def __init__(self, exporter, out_label, in_label, previous_label):
         """
         Args:
-            exporter (ExporterBase): exporter
+            exporter (Exporter): exporter
             out_label (str): new output resource label
             in_label (str): associated input resource label
             previous_label (str): previous output resource label
@@ -46,6 +52,31 @@ class UpdateOutLabel(SpineToolboxCommand):
         self._exporter.set_out_label(self._previous_out_label, self._in_label)
 
 
+class UpdateOutUrl(SpineToolboxCommand):
+    """Command to update exporter's output URL."""
+
+    def __init__(self, exporter, in_label, url, previous_url):
+        """
+        Args:
+            exporter (Exporter): exporter
+            in_label (str): input resource label
+            url (dict, optional): new URL dict
+            previous_url (dict, optional): previous URL dict
+        """
+        super().__init__()
+        self._exporter = exporter
+        self._in_label = in_label
+        self._url = copy(url)
+        self._previous_url = copy(previous_url)
+        self.setText(f"change output URL in {exporter.name}")
+
+    def redo(self):
+        self._exporter.set_out_url(self._in_label, copy(self._url))
+
+    def undo(self):
+        self._exporter.set_out_url(self._in_label, copy(self._previous_url))
+
+
 class NewMapping(QUndoCommand):
     def __init__(self, mappings_table_model, mapping_specification, name=""):
         """
@@ -124,7 +155,10 @@ class SetMappingEnabled(QUndoCommand):
         self._mappings_table_model = mappings_table_model
         name = self._mappings_table_model.index(row, 0).data()
         self._row = row
-        self._previously_enabled = self._mappings_table_model.index(self._row, 0).data(Qt.CheckStateRole) == Qt.Checked
+        self._previously_enabled = (
+            self._mappings_table_model.index(self._row, 0).data(Qt.ItemDataRole.CheckStateRole)
+            == Qt.CheckState.Checked.value
+        )
         super().__init__(("disable" if self._previously_enabled else "enable") + f" '{name}'")
 
     def redo(self):
@@ -336,6 +370,25 @@ class SetMappingPositions(QUndoCommand):
         self._positions = positions
         self._previous_positions = previous_positions
 
+    @property
+    def mapping_editor_table_model(self):
+        return self._mapping_editor_table_model
+
+    @property
+    def mapping_name(self):
+        return self._mapping_name
+
+    @property
+    def positions(self):
+        return self._positions
+
+    @property
+    def previous_positions(self):
+        return self._previous_positions
+
+    def id(self):
+        return int(_ID.CHANGE_POSITION)
+
     def redo(self):
         self._mapping_editor_table_model.set_positions(self._positions, self._mapping_name)
 
@@ -343,6 +396,55 @@ class SetMappingPositions(QUndoCommand):
         self._mapping_editor_table_model.set_positions(self._previous_positions, self._mapping_name)
 
 
+class ClearFixedTableName(QUndoCommand):
+    def __init__(self, editor, is_fixed_table_checked, previous_fixed_table_name, mapping_index, mapping_root):
+        """
+        Args:
+            editor (SpecificationEditor): specification editor window
+            is_fixed_table_checked (bool): state of the fixed table name checkbox before clearing
+            previous_fixed_table_name (str): fixed table name before clearing
+            mapping_index (QModelIndex): mapping's row index
+            mapping_root (Mapping): mapping root
+        """
+        super().__init__("change mapping item's position")
+        self._editor = editor
+        self._is_fixed_table_checked = is_fixed_table_checked
+        self._previous_fixed_table_name = previous_fixed_table_name
+        self._mapping_editor_table_model = None
+        self._mapping_name = None
+        self._positions = None
+        self._previous_positions = None
+        self._mapping_index = mapping_index
+        self._mapping_root = mapping_root
+        self._previous_mapping_root = self._mapping_index.data(MappingsTableModel.MAPPING_ROOT_ROLE)
+
+    def id(self):
+        return int(_ID.CHANGE_POSITION)
+
+    def mergeWith(self, other):
+        if self._mapping_editor_table_model is not None:
+            return False
+        self._mapping_editor_table_model = other.mapping_editor_table_model
+        self._mapping_name = other.mapping_name
+        self._positions = other.positions
+        self._previous_positions = other.previous_positions
+        return True
+
+    def redo(self):
+        self._editor.clear_fixed_table_name()
+        self._mapping_index.model().setData(
+            self._mapping_index, deepcopy(self._mapping_root), MappingsTableModel.MAPPING_ROOT_ROLE
+        )
+
+    def undo(self):
+        self._mapping_index.model().setData(
+            self._mapping_index, deepcopy(self._previous_mapping_root), MappingsTableModel.MAPPING_ROOT_ROLE
+        )
+        self._editor.enable_fixed_table_name(self._is_fixed_table_checked, self._previous_fixed_table_name)
+        if self._mapping_editor_table_model is not None:
+            self._mapping_editor_table_model.set_positions(self._previous_positions, self._mapping_name)
+
+
 class SetMappingProperty(QUndoCommand):
     def __init__(self, command_name, setter, mapping_name, row, value, previous_value):
         """
@@ -429,3 +531,24 @@ class SetExportFormat(QUndoCommand):
 
     def undo(self):
         self._specification_editor.set_export_format_silently(self._previous_format)
+
+
+class UpdateOutputTimeStampsFlag(SpineToolboxCommand):
+    """Command to set exporter's output directory time stamps flag."""
+
+    def __init__(self, exporter, value):
+        """
+        Args:
+            exporter (Exporter): exporter item
+            value (bool): flag's new value
+        """
+        super().__init__()
+        self.setText(f"toggle output time stamps setting of {exporter.name}")
+        self._exporter = exporter
+        self._value = value
+
+    def redo(self):
+        self._exporter.set_output_time_stamps_flag(self._value)
+
+    def undo(self):
+        self._exporter.set_output_time_stamps_flag(not self._value)
