@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -17,20 +17,24 @@ Contains the :class:`Exporter` project item.
 """
 from dataclasses import dataclass
 from itertools import combinations, zip_longest
+from operator import itemgetter
 from pathlib import Path
 
-from PySide2.QtCore import Slot, Qt
+from PySide6.QtCore import Slot, Qt
+
 from spinetoolbox.project_item.project_item import ProjectItem
 from spine_engine.utils.serialization import deserialize_path
 from spinedb_api import clear_filter_configs
 from .export_manifest import exported_files_as_resources
-from ..utils import EXPORTER_EXECUTION_MANIFEST_FILE_PREFIX
-from ..item_base import ExporterBase
+from .specification import OutputFormat
+from ..commands import UpdateCancelOnErrorCommand
+from .mvcmodels.full_url_list_model import FullUrlListModel
 from .widgets.export_list_item import ExportListItem
 from .item_info import ItemInfo
 from .executable_item import ExecutableItem
-from .commands import UpdateOutLabel
+from .commands import UpdateOutLabel, UpdateOutputTimeStampsFlag, UpdateOutUrl
 from .output_channel import OutputChannel
+from .utils import EXPORTER_EXECUTION_MANIFEST_FILE_PREFIX, output_database_resources
 
 
 @dataclass
@@ -39,10 +43,11 @@ class _Notifications:
 
     duplicate_out_label: bool = False
     missing_out_label: bool = False
+    unrecognized_extension: bool = False
     missing_specification: bool = False
 
 
-class Exporter(ExporterBase):
+class Exporter(ProjectItem):
     """Exporter project item."""
 
     def __init__(
@@ -71,7 +76,14 @@ class Exporter(ExporterBase):
             output_time_stamps (bool): True to include time stamps to output directory names
             cancel_on_error (bool): True to fail execution in case of non-fatal errors
         """
-        super().__init__(name, description, x, y, toolbox, project, output_time_stamps, cancel_on_error)
+        super().__init__(name, description, x, y, project)
+        self._toolbox = toolbox
+        self._append_output_time_stamps = output_time_stamps
+        self._cancel_on_error = cancel_on_error
+        self._output_filenames = dict()
+        self._export_list_items = dict()
+        self._full_url_model = FullUrlListModel()
+        self._exported_files = None
         self._notifications = _Notifications()
         self._output_channels = []
         if output_channels is None:
@@ -99,24 +111,87 @@ class Exporter(ExporterBase):
     def executable_class(self):
         return ExecutableItem
 
+    def has_out_url(self):
+        """Returns whether any output channel has a URL set.
+
+        Returns:
+            bool: True if any channel has output URL, False otherwise
+        """
+        return any(c.out_url is not None for c in self._output_channels)
+
+    def handle_execution_successful(self, execution_direction, engine_state):
+        """See base class."""
+        if execution_direction != "FORWARD":
+            return
+        self._resources_to_successors_changed()
+
+    def full_url_model(self):
+        """
+        Returns the full URL model held by the exporter.
+
+        Returns:
+            FullUrlListModel: full URL model
+        """
+        return self._full_url_model
+
+    @Slot(int)
+    def _cancel_on_error_option_changed(self, checkbox_state):
+        """Handles changes to the Cancel export on error option."""
+        cancel = checkbox_state == Qt.CheckState.Checked.value
+        if self._cancel_on_error == cancel:
+            return
+        self._toolbox.undo_stack.push(UpdateCancelOnErrorCommand(self, cancel))
+
+    def set_cancel_on_error(self, cancel):
+        """Sets the Cancel export on error option."""
+        self._cancel_on_error = cancel
+        if not self._active:
+            return
+        # This does not trigger the stateChanged signal.
+        self._properties_ui.cancel_on_error_check_box.setCheckState(
+            Qt.CheckState.Checked if cancel else Qt.CheckState.Unchecked
+        )
+
     def _update_properties_tab(self):
         """Updates the labels list in the properties tab."""
+        self._set_properties_message()
         outputs_layout = self._properties_ui.outputs_list_layout
         while not outputs_layout.isEmpty():
             widget_to_remove = outputs_layout.takeAt(0)
             widget_to_remove.widget().deleteLater()
         self._export_list_items.clear()
         for channel in self._output_channels:
-            item = self._export_list_items[channel.in_label] = ExportListItem(channel.in_label, channel.out_label)
+            item = self._export_list_items[channel.in_label] = ExportListItem(
+                channel.in_label, channel.out_label, channel.out_url, self._toolbox.qsettings(), self._toolbox
+            )
             outputs_layout.addWidget(item)
+            output_format = self._specification.output_format if self._specification is not None else None
+            item.set_out_url_enabled(output_format is None or output_format == OutputFormat.SQL)
             item.out_label_changed.connect(self._update_out_label)
+            item.out_url_changed.connect(self._update_out_url)
         self._properties_ui.output_time_stamps_check_box.setCheckState(
-            Qt.Checked if self._append_output_time_stamps else Qt.Unchecked
+            Qt.CheckState.Checked if self._append_output_time_stamps else Qt.CheckState.Unchecked
         )
         self._properties_ui.cancel_on_error_check_box.setCheckState(
-            Qt.Checked if self._cancel_on_error else Qt.Unchecked
+            Qt.CheckState.Checked if self._cancel_on_error else Qt.CheckState.Unchecked
         )
 
+    def _set_properties_message(self):
+        if self._specification is None:
+            message = ""
+        else:
+            if self._specification.is_exporting_multiple_files():
+                message = (
+                    f"Currently exporting multiple files in {self._specification.output_format.value} format."
+                    " The file names are given by the specification."
+                )
+            else:
+                message = (
+                    f"Currently exporting in {self._specification.output_format.value} format."
+                    " The Output labels below are treated as file names."
+                )
+        self._properties_ui.message_label.setText(message)
+
     def upstream_resources_updated(self, resources):
         """See base class."""
         database_resources = [r for r in resources if r.type_ == "database"]
@@ -136,9 +211,9 @@ class Exporter(ExporterBase):
                     # legacy: when in_label was url
                     # we actually have out_label already
                     inactive = inactive_channels.pop(url)
-                    self._output_channels.append(OutputChannel(in_label, inactive.out_label))
+                    self._output_channels.append(OutputChannel(in_label, self.name, inactive.out_label))
                 else:
-                    self._output_channels.append(OutputChannel(in_label))
+                    self._output_channels.append(OutputChannel(in_label, self.name))
         self._inactive_output_channels = list(inactive_channels.values())
         self._full_url_model.set_urls(set(r.url for r in database_resources))
         if self._output_channels != old_output_channels:
@@ -156,12 +231,15 @@ class Exporter(ExporterBase):
                 if channel.in_label == old_resource.label:
                     channel.in_label = new_resource.label
                     break
+            else:
+                raise RuntimeError(f"Logic error: cannot find channel for {old_resource.label}")
             self._full_url_model.update_url(old_resource.url, new_resource.url)
 
     def _check_notifications(self):
         """See base class."""
         self._check_missing_out_labels()
         self._check_duplicate_out_labels()
+        self._check_unrecognized_extension()
         self._check_missing_specification()
         self._report_notifications()
 
@@ -176,17 +254,45 @@ class Exporter(ExporterBase):
             self.add_notification("Output label(s) missing.")
         if self._notifications.missing_specification:
             self.add_notification("Export specification missing.")
+        if self._notifications.unrecognized_extension:
+            self.add_notification("File extensions don't match the output format.")
 
     def _check_missing_out_labels(self):
         """Checks the status of out labels"""
-        self._notifications.missing_output_file_name = not all(bool(c.out_label) for c in self._output_channels)
+        self._notifications.missing_out_label = not all(bool(c.out_label) for c in self._output_channels)
 
     def _check_duplicate_out_labels(self):
         """Checks for duplicate output file names."""
         self._notifications.duplicate_out_label = any(
-            c1.out_label == c2.out_label for c1, c2 in combinations(self._output_channels, 2) if c1 and c2
+            c1.out_label == c2.out_label
+            for c1, c2 in combinations(self._output_channels, 2)
+            if c1.out_label and c2.out_label
         )
 
+    def _check_unrecognized_extension(self):
+        """Checks that file extensions match with selected output format."""
+        if self._notifications.missing_specification or self._notifications.missing_out_label:
+            self._notifications.unrecognized_extension = False
+            return
+        output_formats = set()
+        for channel in self._output_channels:
+            name, separator, extension = channel.out_label.rpartition(".")
+            if not separator:
+                continue
+            output_format = OutputFormat.output_format_from_extension(extension)
+            if output_format is None:
+                self._notifications.unrecognized_extension = True
+                return
+            output_formats.add(output_format)
+        if len(output_formats) == 0:
+            self._notifications.unrecognized_extension = False
+            return
+        if len(output_formats) > 1:
+            self._notifications.unrecognized_extension = True
+            return
+        output_format = next(iter(output_formats))
+        self._notifications.unrecognized_extension = output_format != self._specification.output_format
+
     @Slot(str, str)
     def _update_out_label(self, out_label, in_label):
         """Pushes UpdateOutLabel command to undo stack.
@@ -198,6 +304,39 @@ class Exporter(ExporterBase):
         previous = next(c for c in self._output_channels if c.in_label == in_label)
         self._toolbox.undo_stack.push(UpdateOutLabel(self, out_label, in_label, previous.out_label))
 
+    @Slot(str, dict)
+    def _update_out_url(self, in_label, url_dict):
+        """Pushes a command to change the output URL to undo stack.
+
+        Args:
+            in_label (str): input label
+            url_dict (dict): URL
+        """
+        for channel in self._output_channels:
+            if channel.in_label == in_label:
+                if channel.out_url != url_dict:
+                    self._toolbox.undo_stack.push(UpdateOutUrl(self, in_label, url_dict, channel.out_url))
+                break
+        else:
+            raise RuntimeError(f"Logic error: cannot find channel for input label {in_label}")
+
+    def set_out_url(self, in_label, url_dict):
+        """Sets output URL for an output channel.
+
+        Args:
+            in_label (str): input label
+            url_dict (dict): URL
+        """
+        for channel in self._output_channels:
+            if channel.in_label == in_label:
+                channel.out_url = url_dict
+                if self._active:
+                    export_list_item = self._export_list_items[in_label]
+                    export_list_item.set_out_url(url_dict)
+                break
+        else:
+            raise RuntimeError(f"Logic error: cannot find channel for input label {in_label}")
+
     def set_out_label(self, out_label, in_label):
         """Updates the output label.
 
@@ -214,6 +353,7 @@ class Exporter(ExporterBase):
         channel.out_label = out_label
         self._check_missing_out_labels()
         self._check_duplicate_out_labels()
+        self._check_unrecognized_extension()
         self._report_notifications()
         if self._exported_files is not None:
             exported_files = self._exported_files.pop(old_out_label, None)
@@ -227,30 +367,63 @@ class Exporter(ExporterBase):
             news = [resource for resource in new_resources if resource.label == out_label]
             self._resources_to_successors_replaced(olds, news)
 
+    def get_out_labels(self):
+        """Returns output labels.
+
+        Returns:
+            set: output labels
+        """
+        return {channel.out_label for channel in self._output_channels}
+
     def item_dict(self):
         """See base class."""
         serialized = super().item_dict()
-        serialized["output_labels"] = sorted([c.to_dict() for c in self._output_channels], key=lambda d: d["in_label"])
+        serialized["output_time_stamps"] = self._append_output_time_stamps
+        serialized["cancel_on_error"] = self._cancel_on_error
+        serialized["output_labels"] = sorted(
+            [c.to_dict(self._project.project_dir) for c in self._output_channels], key=itemgetter("in_label")
+        )
+        if self._specification is not None and self._specification.output_format == OutputFormat.SQL:
+            for channel in self._output_channels:
+                if channel.out_url is not None:
+                    serialized.setdefault("db_credentials", {})[channel.in_label] = {
+                        "username": channel.out_url.get("username"),
+                        "password": channel.out_url.get("password"),
+                    }
         serialized["specification"] = self._specification_name
         return serialized
 
+    @staticmethod
+    def item_dict_local_entries():
+        """See base class."""
+        return [("db_credentials",)]
+
     def resources_for_direct_successors(self):
         """See base class."""
+        output_format = self._specification.output_format if self._specification is not None else None
         resources, self._exported_files = exported_files_as_resources(
-            self.name, self._exported_files, self.data_dir, self._output_channels
+            self.name, self._exported_files, self.data_dir, self._output_channels, output_format
         )
+        if output_format == OutputFormat.SQL:
+            resources += output_database_resources(self.name, self._output_channels)
         return resources
 
     @staticmethod
     def from_dict(name, item_dict, toolbox, project):
         """See base class."""
         description, x, y = ProjectItem.parse_item_dict(item_dict)
-        output_channels = [OutputChannel.from_dict(d) for d in item_dict.get("output_labels", [])]
+        output_channels = [
+            OutputChannel.from_dict(d, name, project.project_dir) for d in item_dict.get("output_labels", [])
+        ]
         for db_dict in item_dict.get("databases", []):
             # Legacy item dict.
             out_label = db_dict["output_file_name"]
             url = clear_filter_configs(deserialize_path(db_dict["database_url"], project.project_dir))
-            output_channels.append(OutputChannel(url, out_label))
+            output_channels.append(OutputChannel(url, name, out_label))
+        for in_label, credentials in item_dict.get("db_credentials", {}).items():
+            for channel in output_channels:
+                if channel.in_label == in_label:
+                    channel.out_url.update(credentials)
         output_time_stamps = item_dict.get("output_time_stamps", False)
         cancel_on_error = item_dict.get("cancel_on_error", True)
         specification_name = item_dict.get("specification", "")
@@ -296,10 +469,36 @@ class Exporter(ExporterBase):
         """Returns a dictionary of all shared signals and their handlers.
         This is to enable simpler connecting and disconnecting."""
         s = super().make_signal_handler_dict()
+        s[self._properties_ui.output_time_stamps_check_box.stateChanged] = self._change_output_time_stamps_flag
+        s[self._properties_ui.cancel_on_error_check_box.stateChanged] = self._cancel_on_error_option_changed
         s[self._properties_ui.specification_button.clicked] = self.show_specification_window
         s[self._properties_ui.specification_combo_box.textActivated] = self._change_specification
         return s
 
+    @Slot(int)
+    def _change_output_time_stamps_flag(self, checkbox_state):
+        """
+        Pushes a command that changes the output time stamps flag value.
+
+        Args:
+            checkbox_state (int): setting's checkbox state on properties tab
+        """
+        flag = checkbox_state == Qt.CheckState.Checked.value
+        if flag == self._append_output_time_stamps:
+            return
+        self._toolbox.undo_stack.push(UpdateOutputTimeStampsFlag(self, flag))
+
+    def set_output_time_stamps_flag(self, flag):
+        """
+        Sets the output time stamps flag.
+
+        Args:
+            flag (bool): flag value
+        """
+        self._append_output_time_stamps = flag
+        if self._active:
+            self._properties_ui.output_time_stamps_check_box.setChecked(flag)
+
     def _check_missing_specification(self):
         """Checks specification's status."""
         self._notifications.missing_specification = not self._specification_name
@@ -316,6 +515,11 @@ class Exporter(ExporterBase):
         self._check_notifications()
         if self._active:
             self._properties_ui.specification_combo_box.setCurrentText(self._specification_name)
+            self._set_properties_message()
+            output_format = specification.output_format if specification is not None else None
+            out_url_enabled = output_format is None or output_format == OutputFormat.SQL
+            for item in self._export_list_items.values():
+                item.set_out_url_enabled(out_url_enabled)
         return True
 
     @Slot(bool)
@@ -351,8 +555,12 @@ class Exporter(ExporterBase):
 
     def restore_selections(self):
         """See base class."""
-        super().restore_selections()
+        self._update_properties_tab()
         if self._specification_name:
             self._properties_ui.specification_combo_box.setCurrentText(self._specification_name)
         else:
             self._properties_ui.specification_combo_box.setCurrentIndex(-1)
+
+    def tear_down(self):
+        super().tear_down()
+        self._full_url_model.deleteLater()
