@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -15,8 +15,10 @@ Contains model for export mapping setup table.
 :date:   11.12.2020
 """
 from enum import IntEnum, unique
-from PySide2.QtCore import QAbstractTableModel, QModelIndex, Qt
-from PySide2.QtGui import QFont, QColor
+from operator import itemgetter
+
+from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt
+from PySide6.QtGui import QFont, QColor
 from spinedb_api.mapping import is_pivoted, is_regular, Position, value_index
 from spinedb_api.export_mapping.export_mapping import (
     AlternativeDescriptionMapping,
@@ -78,8 +80,7 @@ class EditorColumn(IntEnum):
 
 
 class MappingEditorTableModel(QAbstractTableModel):
-
-    MAPPING_ITEM_ROLE = Qt.UserRole + 1
+    MAPPING_ITEM_ROLE = Qt.ItemDataRole.UserRole + 1
 
     def __init__(self, mapping_name, root_mapping, undo_stack, mapping_provider, parent=None):
         """
@@ -111,11 +112,20 @@ class MappingEditorTableModel(QAbstractTableModel):
         if self._mappings and isinstance(self._mappings[0], FixedValueMapping):
             # Pop the first element if it's a FixedValueMapping. We don't want to have the fixed table name here.
             self._mappings.pop(0)
-        self._reset_colors()
+        self._reset_colors(emit_data_changed=False)
 
-    def _reset_colors(self):
+    def _reset_colors(self, emit_data_changed=True):
+        """Recalculates position column background colors.
+
+        Args:
+            emit_data_changed (boo): if True emits relevant dataChanged signal
+        """
         positions = [m.position for m in self._mappings if isinstance(m.position, int)]
         self._mapping_colors = {p: color_from_index(i, len(positions)).lighter() for i, p in enumerate(positions)}
+        if emit_data_changed:
+            top_left = self.index(0, EditorColumn.POSITION)
+            bottom_right = self.index(len(self._mappings), EditorColumn.POSITION)
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.BackgroundRole])
 
     def mapping_colors(self):
         return self._mapping_colors
@@ -123,9 +133,9 @@ class MappingEditorTableModel(QAbstractTableModel):
     def columnCount(self, paren=QModelIndex()):
         return len(EditorColumn)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         column = index.column()
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             row = index.row()
             if column == EditorColumn.ROW_LABEL:
                 mapping = self._mappings[row]
@@ -143,21 +153,21 @@ class MappingEditorTableModel(QAbstractTableModel):
                 return self._mappings[row].header
             if column == EditorColumn.FILTER:
                 return self._mappings[row].filter_re
-        elif role == Qt.CheckStateRole:
+        elif role == Qt.ItemDataRole.CheckStateRole:
             if column == EditorColumn.PIVOTED:
                 if is_pivoted(self._mappings[index.row()].position):
-                    return Qt.Checked
-                return Qt.Unchecked
+                    return Qt.CheckState.Checked
+                return Qt.CheckState.Unchecked
             if column == EditorColumn.NULLABLE:
-                return Qt.Checked if self._mappings[index.row()].is_ignorable() else Qt.Unchecked
-        elif role == Qt.FontRole and column == EditorColumn.ROW_LABEL:
+                return Qt.CheckState.Checked if self._mappings[index.row()].is_ignorable() else Qt.CheckState.Unchecked
+        elif role == Qt.ItemDataRole.FontRole and column == EditorColumn.ROW_LABEL:
             font = QFont()
             font.setBold(True)
             return font
-        elif role == Qt.BackgroundRole and column == EditorColumn.ROW_LABEL:
+        elif role == Qt.ItemDataRole.BackgroundRole and column == EditorColumn.ROW_LABEL:
             m = self._mappings[index.row()]
-            return self._mapping_colors.get(m.position, QColor(Qt.gray).lighter())
-        elif role == Qt.ToolTipRole:
+            return self._mapping_colors.get(m.position, QColor(Qt.GlobalColor.gray).lighter())
+        elif role == Qt.ItemDataRole.ToolTipRole:
             if column == EditorColumn.FILTER:
                 return "Regular expression to filter database items."
             elif column == EditorColumn.NULLABLE:
@@ -186,8 +196,8 @@ class MappingEditorTableModel(QAbstractTableModel):
             return super().flags(index) | Qt.ItemIsUserCheckable
         return super().flags(index)
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
             return ("Mapping type", "Map to", "Pivoted", "Nullable", "Header", "Filter")[section]
         return None
 
@@ -201,9 +211,9 @@ class MappingEditorTableModel(QAbstractTableModel):
     def rowCount(self, parent=QModelIndex()):
         return len(self._mappings)
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
         column = index.column()
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             row = index.row()
             mapping = self._mappings[row]
             if column == EditorColumn.POSITION:
@@ -231,7 +241,7 @@ class MappingEditorTableModel(QAbstractTableModel):
                 )
                 self._undo_stack.push(command)
                 return True
-        elif role == Qt.CheckStateRole:
+        elif role == Qt.ItemDataRole.CheckStateRole:
             if column == EditorColumn.PIVOTED:
                 row = index.row()
                 new_positions = _propose_toggled_pivot(self._mappings, row)
@@ -314,9 +324,7 @@ class MappingEditorTableModel(QAbstractTableModel):
         pivot_top = top
         bottom = len(self._mappings)
         pivot_bottom = bottom
-        for row in range(len(self._mappings)):
-            mapping = self._mappings[row]
-            position = positions[row]
+        for row, (mapping, position) in enumerate(zip(self._mappings, positions)):
             if position != mapping.position:
                 top = row
                 bottom = min(bottom, row)
@@ -326,16 +334,16 @@ class MappingEditorTableModel(QAbstractTableModel):
                 mapping.position = position
         top_left = self.index(top, EditorColumn.POSITION)
         bottom_right = self.index(bottom, EditorColumn.POSITION)
-        self.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole])
+        self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
         if pivot_bottom <= pivot_top:
             top_left = self.index(pivot_top, EditorColumn.PIVOTED)
             bottom_right = self.index(pivot_bottom, EditorColumn.PIVOTED)
-            self.dataChanged.emit(top_left, bottom_right, [Qt.CheckStateRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.CheckStateRole])
         non_leaf_pivoted = self._is_non_leaf_pivoted()
         if non_leaf_pivoted != self._non_leaf_mapping_is_pivoted:
             self._non_leaf_mapping_is_pivoted = non_leaf_pivoted
             index = self.index(value_index(self._mappings), EditorColumn.POSITION)
-            self.dataChanged.emit(index, index, [Qt.DisplayRole])
+            self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
         self._reset_colors()
 
     def set_header(self, header, row, mapping_name):
@@ -351,7 +359,7 @@ class MappingEditorTableModel(QAbstractTableModel):
             self._mapping_provider.show_on_table(mapping_name)
         self._mappings[row].header = header
         index = self.index(row, EditorColumn.HEADER)
-        self.dataChanged.emit(index, index, [Qt.DisplayRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
 
     def set_filter_re(self, filter_re, row, mapping_name):
         """
@@ -366,7 +374,7 @@ class MappingEditorTableModel(QAbstractTableModel):
             self._mapping_provider.show_on_table(mapping_name)
         self._mappings[row].filter_re = filter_re
         index = self.index(row, EditorColumn.FILTER)
-        self.dataChanged.emit(index, index, [Qt.DisplayRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
 
     def set_nullable(self, nullable, row, mapping_name):
         """
@@ -381,7 +389,7 @@ class MappingEditorTableModel(QAbstractTableModel):
             self._mapping_provider.show_on_table(mapping_name)
         self._mappings[row].set_ignorable(nullable)
         index = self.index(row, EditorColumn.NULLABLE)
-        self.dataChanged.emit(index, index, [Qt.CheckStateRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.CheckStateRole])
 
     def _is_non_leaf_pivoted(self):
         """Checks if a non-leaf mapping is pivoted.
@@ -414,22 +422,22 @@ class MappingEditorTableModel(QAbstractTableModel):
             pivoted_mappings = [
                 (m.position, m) for row, m in enumerate(self._mappings) if is_pivoted(m.position) and row != value_row
             ]
-            pivoted_mappings.sort(reverse=True, key=lambda x: x[0])
+            pivoted_mappings.sort(reverse=True, key=itemgetter(0))
             for row, item in enumerate(pivoted_mappings):
                 item[1].position = -(row + 1)
             regular_mappings = [
                 (m.position, m) for row, m in enumerate(self._mappings) if is_regular(m.position) and row != value_row
             ]
-            last_column = max(regular_mappings, key=lambda x: x[0], default=-1)[0]
+            last_column = max(regular_mappings, key=itemgetter(0), default=-1)[0]
             regular_mappings.append((last_column + 1, self._mappings[value_row]))
         else:
             regular_mappings = [(m.position, m) for m in self._mappings if is_regular(m.position)]
-        regular_mappings.sort(key=lambda x: x[0])
+        regular_mappings.sort(key=itemgetter(0))
         for column, item in enumerate(regular_mappings):
             item[1].position = column
         top_left = self.index(0, EditorColumn.POSITION)
         bottom_right = self.index(self.rowCount() - 1, EditorColumn.POSITION)
-        self.dataChanged.emit(top_left, bottom_right, Qt.DisplayRole)
+        self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
         self._reset_colors()
 
 
@@ -516,6 +524,13 @@ def _propose_positions(mappings, target_index, new_position):
     if isinstance(new_position, int):
         _insert_into_position(positions, target_index, new_position)
     else:
+        if new_position == Position.table_name:
+            try:
+                other_table_name_index = positions.index(Position.table_name)
+            except ValueError:
+                pass
+            else:
+                positions[other_table_name_index] = positions[target_index]
         positions[target_index] = new_position
     return positions
 
