@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -14,7 +14,7 @@ Contains the :class:`MappingListModel` model.
 :authors: A. Soininen (VTT)
 :date:    30.12.2020
 """
-from PySide2.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal
+from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal
 from spinedb_api.export_mapping.export_mapping import (
     ParameterDefaultValueIndexMapping,
     ParameterValueIndexMapping,
@@ -38,16 +38,16 @@ class MappingsTableModel(QAbstractTableModel):
     write_order_changed = Signal()
     """Emitted after the write order has changed."""
 
-    MAPPING_SPECIFICATION_ROLE = Qt.UserRole + 1
-    MAPPING_TYPE_ROLE = Qt.UserRole + 2
-    MAPPING_ROOT_ROLE = Qt.UserRole + 3
-    ALWAYS_EXPORT_HEADER_ROLE = Qt.UserRole + 4
-    RELATIONSHIP_DIMENSIONS_ROLE = Qt.UserRole + 5
-    USE_FIXED_TABLE_NAME_FLAG_ROLE = Qt.UserRole + 6
-    FIXED_TABLE_NAME_ROLE = Qt.UserRole + 7
-    PARAMETER_DIMENSIONS_ROLE = Qt.UserRole + 8
-    GROUP_FN_ROLE = Qt.UserRole + 9
-    HIGHLIGHT_DIMENSION_ROLE = Qt.UserRole + 10
+    MAPPING_SPECIFICATION_ROLE = Qt.ItemDataRole.UserRole + 1
+    MAPPING_TYPE_ROLE = Qt.ItemDataRole.UserRole + 2
+    MAPPING_ROOT_ROLE = Qt.ItemDataRole.UserRole + 3
+    ALWAYS_EXPORT_HEADER_ROLE = Qt.ItemDataRole.UserRole + 4
+    RELATIONSHIP_DIMENSIONS_ROLE = Qt.ItemDataRole.UserRole + 5
+    USE_FIXED_TABLE_NAME_FLAG_ROLE = Qt.ItemDataRole.UserRole + 6
+    FIXED_TABLE_NAME_ROLE = Qt.ItemDataRole.UserRole + 7
+    PARAMETER_DIMENSIONS_ROLE = Qt.ItemDataRole.UserRole + 8
+    GROUP_FN_ROLE = Qt.ItemDataRole.UserRole + 9
+    HIGHLIGHT_DIMENSION_ROLE = Qt.ItemDataRole.UserRole + 10
 
     def __init__(self, mappings=None, parent=None):
         """
@@ -61,7 +61,7 @@ class MappingsTableModel(QAbstractTableModel):
         self._names = list(mappings)
         self._mappings = mappings
 
-    def columnCount(self, parent):
+    def columnCount(self, parent=QModelIndex()):
         return 2
 
     def extend(self, mapping_specification, name=""):
@@ -86,16 +86,16 @@ class MappingsTableModel(QAbstractTableModel):
         self.endInsertRows()
         return position
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         column = index.column()
         if column == 0:
-            if role == Qt.CheckStateRole:
+            if role == Qt.ItemDataRole.CheckStateRole:
                 spec = self._mappings[self._names[index.row()]]
-                return Qt.Checked if spec.enabled else Qt.Unchecked
-            if role in (Qt.DisplayRole, Qt.EditRole):
+                return Qt.CheckState.Checked if spec.enabled else Qt.CheckState.Unchecked
+            if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
                 return self._names[index.row()]
         if column == 1:
-            if role in (Qt.DisplayRole, Qt.EditRole):
+            if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
                 order = str(index.row() + 1)
                 if order.endswith(("11", "12", "13")):
                     return order + "th"
@@ -108,12 +108,12 @@ class MappingsTableModel(QAbstractTableModel):
                 return order + "th"
             if role == Qt.TextAlignmentRole:
                 return Qt.AlignCenter
-        if role >= Qt.UserRole:
+        if role >= Qt.ItemDataRole.UserRole:
             spec = self._mappings[self._names[index.row()]]
             if role == self.MAPPING_SPECIFICATION_ROLE:
                 return spec
             if role == self.MAPPING_TYPE_ROLE:
-                return spec.type
+                return spec.type.value
             if role == self.MAPPING_ROOT_ROLE:
                 return spec.root
             if role == self.ALWAYS_EXPORT_HEADER_ROLE:
@@ -145,8 +145,8 @@ class MappingsTableModel(QAbstractTableModel):
             return super().flags(index) | Qt.ItemIsUserCheckable | Qt.ItemIsEditable
         return super().flags(index)
 
-    def headerData(self, section, orientation, role):
-        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
             return ("Name", "Write order")[section]
         return None
 
@@ -208,19 +208,19 @@ class MappingsTableModel(QAbstractTableModel):
     def rowCount(self, parent=QModelIndex()):
         return len(self._names)
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
         column = index.column()
         if column == 0:
-            if role == Qt.CheckStateRole:
+            if role == Qt.ItemDataRole.CheckStateRole:
                 row = index.row()
                 self.mapping_enabled_state_change_requested.emit(row)
                 return True
-            if role == Qt.EditRole:
+            if role == Qt.ItemDataRole.EditRole:
                 if not value or value in self._names:
                     return False
                 self.rename_requested.emit(index.row(), value)
                 return True
-        if role > Qt.UserRole:
+        if role > Qt.ItemDataRole.UserRole:
             name = self._names[index.row()]
             spec = self._mappings[name]
             if role == self.MAPPING_ROOT_ROLE:
@@ -291,7 +291,7 @@ class MappingsTableModel(QAbstractTableModel):
         self._names[row] = new_name
         self._mappings[new_name] = self._mappings.pop(previous)
         index = self.index(row, 1)
-        self.dataChanged.emit(index, index, [Qt.DisplayRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
 
     def set_mapping_enabled(self, row, enabled):
         """Enables or disables a mapping.
@@ -304,7 +304,7 @@ class MappingsTableModel(QAbstractTableModel):
         spec = self._mappings[name]
         spec.enabled = enabled
         index = self.index(row, 0)
-        self.dataChanged.emit(index, index, [Qt.CheckStateRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.CheckStateRole])
 
     def set_all_enabled(self, enabled):
         """Enables/disables all mappings.
@@ -324,7 +324,7 @@ class MappingsTableModel(QAbstractTableModel):
         if first is not None:
             top_left = self.index(first, 0)
             bottom_right = self.index(last, 0)
-            self.dataChanged.emit(top_left, bottom_right, [Qt.CheckStateRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.CheckStateRole])
 
     def enable_mapping_rows(self, rows):
         """Enables mappings on given rows and disables the rest.
@@ -345,7 +345,7 @@ class MappingsTableModel(QAbstractTableModel):
         if first is not None:
             top_left = self.index(first, 0)
             bottom_right = self.index(last, 0)
-            self.dataChanged.emit(top_left, bottom_right, [Qt.CheckStateRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.CheckStateRole])
 
     def enabled_mapping_rows(self):
         """Returns enabled mapping rows.
