@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -15,12 +15,13 @@ Contains model for export preview tables.
 :date:   5.1.2021
 """
 from itertools import takewhile
-from PySide2.QtCore import QAbstractItemModel, QModelIndex, Qt
+from operator import methodcaller
+
+from PySide6.QtCore import QAbstractItemModel, QModelIndex, Qt
 
 
 class PreviewTreeModel(QAbstractItemModel):
-
-    TABLE_ROLE = Qt.UserRole + 1
+    TABLE_ROLE = Qt.ItemDataRole.UserRole + 1
 
     def __init__(self):
         super().__init__()
@@ -41,7 +42,7 @@ class PreviewTreeModel(QAbstractItemModel):
         if mapping_name not in self._tables:
             new_names = list(self._mapping_names)
             new_names.append(mapping_name)
-            new_names.sort(key=lambda n: n.lower())
+            new_names.sort(key=methodcaller("lower"))
             row = len(list(takewhile(lambda x: x[0] == x[1], zip(new_names, self._mapping_names))))
             self.beginInsertRows(QModelIndex(), row, row)
             self._tables[mapping_name] = dict()
@@ -67,7 +68,7 @@ class PreviewTreeModel(QAbstractItemModel):
                 self._tables[mapping_name][name] = table
                 new_names = list(self._table_names[mapping_name])
                 new_names.append(name)
-                new_names.sort(key=lambda n: n.lower())
+                new_names.sort(key=methodcaller("lower"))
                 row = len(list(takewhile(lambda x: x[0] == x[1], zip(new_names, self._table_names[mapping_name]))))
                 self.beginInsertRows(parent_index, row, row)
                 self._table_names[mapping_name] = new_names
@@ -76,8 +77,8 @@ class PreviewTreeModel(QAbstractItemModel):
     def columnCount(self, parent=QModelIndex()):
         return 1
 
-    def data(self, index, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole:
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DisplayRole:
             mapping_name_row = index.internalId() - 1
             if mapping_name_row < 0:
                 return self._mapping_names[index.row()]
@@ -137,7 +138,7 @@ class PreviewTreeModel(QAbstractItemModel):
         Returns:
             tuple of str: old name, new name
         """
-        new_names = sorted(new_names, key=lambda n: n.lower())
+        new_names = sorted(new_names, key=methodcaller("lower"))
         olds = set(self._mapping_names)
         news = set(new_names)
         try:
