@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,14 +16,14 @@ Contains :class:`PreviewUpdater`.
 """
 from copy import deepcopy
 from time import monotonic
-from PySide2.QtCore import QItemSelectionModel, QModelIndex, QObject, QRunnable, Qt, QThreadPool, Signal, Slot
-from PySide2.QtWidgets import QFileDialog
+from PySide6.QtCore import QItemSelectionModel, QModelIndex, QObject, QRunnable, Qt, QThreadPool, Signal, Slot
+from PySide6.QtWidgets import QFileDialog
 
 from spinedb_api.export_mapping.group_functions import NoGroup
 from spinedb_api.spine_io.exporters.writer import write
 from spinedb_api import DatabaseMapping, SpineDBVersionError, SpineDBAPIError
 from spinetoolbox.helpers import busy_effect
-from ...models import FullUrlListModel
+from ..mvcmodels.full_url_list_model import FullUrlListModel
 from ..mvcmodels.mappings_table_model import MappingsTableModel
 from ..mvcmodels.preview_tree_model import PreviewTreeModel
 from ..mvcmodels.preview_table_model import PreviewTableModel
@@ -51,8 +51,9 @@ class PreviewUpdater:
         if url_model is None:
             url_model = FullUrlListModel(self._ui.database_url_combo_box)
         self._url_model = url_model
-        self._url_model.rowsInserted.connect(lambda *_: self._enable_controls())
+        self._url_model.rowsInserted.connect(self._enable_controls_after_url_insertion)
         self._url_model.modelReset.connect(self._enable_controls)
+        self._url_model.destroyed.connect(self._forget_url_model)
         self._ui.database_url_combo_box.setModel(self._url_model)
         self._mappings_table_model = mappings_table_model
         self._set_expect_removals_and_inserts(False)
@@ -100,7 +101,7 @@ class PreviewUpdater:
         self._stamps.clear()
         for row in range(self._mappings_table_model.rowCount()):
             index = self._mappings_table_model.index(row, 0)
-            if index.data(Qt.CheckStateRole) == Qt.Checked:
+            if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Checked.value:
                 self._load_preview_data(index.data())
 
     @Slot(QModelIndex, QModelIndex)
@@ -146,7 +147,7 @@ class PreviewUpdater:
             return
         current = self._mappings_proxy_model.mapToSource(current)
         mappings_index = self._mappings_table_model.index(current.row(), 0)
-        if mappings_index.data(Qt.CheckStateRole) == Qt.Unchecked:
+        if mappings_index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Unchecked.value:
             return
         current_preview = self._ui.preview_tree_view.selectionModel().currentIndex()
         if current_preview.parent().row() >= self._preview_tree_model.rowCount():
@@ -259,7 +260,7 @@ class PreviewUpdater:
             return
         for row in range(last, first - 1, -1):
             index = self._mappings_table_model.index(row, 0)
-            if index.data(Qt.CheckStateRole) == Qt.Unchecked:
+            if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Unchecked.value:
                 continue
             removed_name = index.data()
             self._preview_tree_model.remove_mapping(removed_name)
@@ -275,11 +276,15 @@ class PreviewUpdater:
             bottom_right (QModelIndex): bottom right corner of modified mappings' in mapping list model
             roles (list of int): changed data's role
         """
-        if Qt.DisplayRole not in roles or self._current_url is None:
+        if Qt.ItemDataRole.DisplayRole not in roles or self._current_url is None:
             return
         make_index = self._mappings_table_model.index
         indexes = [make_index(row, 0) for row in range(self._mappings_table_model.rowCount())]
-        names = [index.data() for index in indexes if index.data(Qt.CheckStateRole) == Qt.Checked]
+        names = [
+            index.data()
+            for index in indexes
+            if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Checked.value
+        ]
         old_name, new_name = self._preview_tree_model.rename_mappings(names)
         if not old_name:
             return
@@ -296,13 +301,13 @@ class PreviewUpdater:
             bottom_right (QModelIndex): bottom right corner of modified mappings' in mapping list model
             roles (list of int): changed data's role
         """
-        if Qt.CheckStateRole not in roles or self._current_url is None:
+        if Qt.ItemDataRole.CheckStateRole.value not in roles or self._current_url is None:
             return
         first = top_left.row()
         last = bottom_right.row()
         for row in range(first, last + 1):
             index = self._mappings_table_model.index(row, 0)
-            enabled = index.data(Qt.CheckStateRole) == Qt.Checked
+            enabled = index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Checked.value
             name = index.data()
             if not enabled and self._preview_tree_model.has_name(name):
                 self._preview_tree_model.remove_mapping(name)
@@ -328,7 +333,7 @@ class PreviewUpdater:
             return
         row = self._mappings_proxy_model.mapToSource(self._ui.mappings_table.currentIndex()).row()
         index = self._mappings_table_model.index(row, 0)
-        if index.data(Qt.CheckStateRole) == Qt.Unchecked:
+        if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Unchecked.value:
             return
         name = index.data()
         self._load_preview_data(name)
@@ -343,7 +348,7 @@ class PreviewUpdater:
             return
         row = self._mappings_proxy_model.mapToSource(current_index).row()
         index = self._mappings_table_model.index(row, 0)
-        if index.data(Qt.CheckStateRole) == Qt.Unchecked:
+        if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Unchecked.value:
             return
         mapping_name = index.data()
         self._load_preview_data(mapping_name)
@@ -361,7 +366,7 @@ class PreviewUpdater:
             return
         for row in range(first, last + 1):
             index = self._mappings_table_model.index(first, 0)
-            if index.data(Qt.CheckStateRole) == Qt.Unchecked:
+            if index.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Unchecked.value:
                 continue
             mapping_name = index.data()
             self._load_preview_data(mapping_name)
@@ -377,6 +382,11 @@ class PreviewUpdater:
         else:
             self._mapping_editor_table_model.modelReset.connect(self._update_current_mappings_tables)
 
+    @Slot(QModelIndex, int, int)
+    def _enable_controls_after_url_insertion(self, parent, first, last):
+        """Enables controls after new preview database URL has been inserted."""
+        self._enable_controls()
+
     @Slot()
     def _enable_controls(self):
         """Enables and disables widgets as needed."""
@@ -452,12 +462,24 @@ class PreviewUpdater:
         """
         return QFileDialog.getOpenFileName(self._window, "Select database", self._project_dir, "sqlite (*.sqlite)")[0]
 
+    @Slot()
+    def _forget_url_model(self):
+        """Replaces current URL model with a mock one."""
+        self._url_model = FullUrlListModel(self._ui.database_url_combo_box)
+        self._url_model.rowsInserted.connect(self._enable_controls_after_url_insertion)
+        self._url_model.modelReset.connect(self._enable_controls)
+        self._url_model.destroyed.connect(self._forget_url_model)
+        self._ui.database_url_combo_box.setModel(self._url_model)
+        self._reload_preview()
+
     def tear_down(self):
         """Stops all workers."""
         self._stamps.clear()
         self._thread_pool.clear()
         self._thread_pool.deleteLater()
+        self._url_model.rowsInserted.disconnect(self._enable_controls_after_url_insertion)
         self._url_model.modelReset.disconnect(self._enable_controls)
+        self._url_model.destroyed.disconnect(self._forget_url_model)
 
 
 class _Worker(QRunnable):
