@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,9 +16,9 @@ Contains :class:`SpecificationEditorWindow`.
 """
 from copy import deepcopy
 import json
-from PySide2.QtCore import QItemSelectionModel, QMimeData, QModelIndex, QPoint, Qt, Signal, Slot
-from PySide2.QtGui import QKeySequence
-from PySide2.QtWidgets import QAction, QApplication, QHeaderView, QMenu
+from PySide6.QtCore import QItemSelectionModel, QMimeData, QModelIndex, QPoint, Qt, Signal, Slot
+from PySide6.QtGui import QKeySequence, QAction
+from PySide6.QtWidgets import QApplication, QHeaderView, QMenu
 
 from spinedb_api.mapping import unflatten
 from spinedb_api.export_mapping import (
@@ -26,7 +26,6 @@ from spinedb_api.export_mapping import (
     feature_export,
     object_export,
     object_group_export,
-    object_group_parameter_export,
     object_parameter_default_value_export,
     object_parameter_export,
     parameter_value_list_export,
@@ -57,6 +56,7 @@ from spinetoolbox.project_item.specification_editor_window import SpecificationE
 from .preview_updater import PreviewUpdater
 from ..commands import (
     ChangeWriteOrder,
+    ClearFixedTableName,
     CompactMapping,
     DisableAllMappings,
     EnableAllMappings,
@@ -86,7 +86,6 @@ mapping_type_to_combo_box_label = {
     MappingType.features: "Feature",
     MappingType.objects: "Object class",
     MappingType.object_groups: "Object group",
-    MappingType.object_group_parameter_values: "Object group",
     MappingType.object_parameter_default_values: "Object class",
     MappingType.object_parameter_values: "Object class",
     MappingType.parameter_value_lists: "Parameter value list",
@@ -107,7 +106,6 @@ mapping_type_to_parameter_type_label = {
     MappingType.features: "None",
     MappingType.objects: "None",
     MappingType.object_groups: "None",
-    MappingType.object_group_parameter_values: "Value",
     MappingType.object_parameter_default_values: "Default value",
     MappingType.object_parameter_values: "Value",
     MappingType.parameter_value_lists: "None",
@@ -137,29 +135,36 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         """
         Args:
             toolbox (ToolboxUI): Toolbox main window
-            specification (ProjectItemSpecification, optional): exporter specification
+            specification (Specification, optional): exporter specification
             item (ProjectItem, optional): invoking project item, if window was opened from its properties tab
             url_model (QAbstractListModel): model that provides URLs of connected databases
         """
         super().__init__(toolbox, specification, item)
-        self._new_spec = deepcopy(specification) if specification is not None else Specification()
+        if specification is None:
+            output_format = self._sniff_export_format() if item is not None else OutputFormat.default()
+        else:
+            output_format = specification.output_format
+        self._new_spec = (
+            deepcopy(specification) if specification is not None else Specification(output_format=output_format)
+        )
         self._mappings_table_model = MappingsTableModel(self._new_spec.mapping_specifications(), self)
         self._mappings_table_model.dataChanged.connect(self._update_ui_after_mapping_change)
         self._mappings_table_model.rename_requested.connect(self._rename_mapping)
         self._mappings_table_model.mapping_enabled_state_change_requested.connect(self._set_mapping_enabled)
         self._mappings_table_model.set_all_mappings_enabled_requested.connect(self._enable_disable_all_mappings)
         self._mappings_table_model.write_order_about_to_change.connect(
-            lambda: self._expect_current_mapping_change(True)
+            lambda tr=True: self._expect_current_mapping_change(tr)
         )
-        self._mappings_table_model.write_order_changed.connect(lambda: self._expect_current_mapping_change(False))
+        self._mappings_table_model.write_order_changed.connect(lambda tr=False: self._expect_current_mapping_change(tr))
         self._mapping_editor_model = MappingEditorTableModel("", None, self._undo_stack, self, self)
+        self._mapping_editor_model.dataChanged.connect(self._validate_fixed_table_name)
         self._sort_mappings_table_model = MappingsTableProxy(self)
         self._sort_mappings_table_model.setSortCaseSensitivity(Qt.CaseInsensitive)
         self._sort_mappings_table_model.setSourceModel(self._mappings_table_model)
         self._sort_mappings_table_model.rowsInserted.connect(self._select_inserted_row)
         self._sort_mappings_table_model.rowsRemoved.connect(self._check_for_empty_mappings_list)
         self.takeCentralWidget().deleteLater()
-        self._ui.export_format_combo_box.addItems([format.value for format in OutputFormat])
+        self._ui.export_format_combo_box.addItems([output_format.value for output_format in OutputFormat])
         self._ui.export_format_combo_box.setCurrentText(self._new_spec.output_format.value)
         self._ui.export_format_combo_box.currentTextChanged.connect(self._change_format)
         self._add_mapping_action = QAction("Add Mapping", self)
@@ -199,7 +204,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.mappings_table.sortByColumn(0, Qt.AscendingOrder)
         self._ui.mappings_table.customContextMenuRequested.connect(self._show_mappings_table_context_menu)
         self._expect_current_mapping_change(False)
-        self._ui.mappings_table.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
+        self._ui.mappings_table.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
         self._ui.group_fn_combo_box.addItems(GROUP_FUNCTION_DISPLAY_NAMES)
         self._ui.item_type_combo_box.currentTextChanged.connect(self._change_mapping_type)
         self._ui.parameter_type_combo_box.currentTextChanged.connect(self._change_mapping_type)
@@ -219,12 +224,12 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._filter_edit_delegate = FilterEditDelegate(self)
         self._ui.mapping_table_view.setItemDelegateForColumn(EditorColumn.FILTER, self._filter_edit_delegate)
         table_header = self._ui.mapping_table_view.horizontalHeader()
-        table_header.setSectionResizeMode(EditorColumn.ROW_LABEL, QHeaderView.ResizeToContents)
-        table_header.setSectionResizeMode(EditorColumn.POSITION, QHeaderView.ResizeToContents)
-        table_header.setSectionResizeMode(EditorColumn.PIVOTED, QHeaderView.ResizeToContents)
-        table_header.setSectionResizeMode(EditorColumn.NULLABLE, QHeaderView.ResizeToContents)
-        table_header.setSectionResizeMode(EditorColumn.HEADER, QHeaderView.ResizeToContents)
-        table_header.setMinimumSectionSize(position_section_width())
+        table_header.setSectionResizeMode(EditorColumn.ROW_LABEL, QHeaderView.ResizeMode.ResizeToContents)
+        table_header.setSectionResizeMode(EditorColumn.POSITION, QHeaderView.ResizeMode.ResizeToContents)
+        table_header.setSectionResizeMode(EditorColumn.PIVOTED, QHeaderView.ResizeMode.ResizeToContents)
+        table_header.setSectionResizeMode(EditorColumn.NULLABLE, QHeaderView.ResizeMode.ResizeToContents)
+        table_header.setSectionResizeMode(EditorColumn.HEADER, QHeaderView.ResizeMode.ResizeToContents)
+        table_header.setMinimumSectionSize(position_section_width(self))
         self._enable_mapping_specification_editing()
         if specification is None:
             self._mappings_table_model.extend(_new_mapping_specification(MappingType.objects))
@@ -270,30 +275,30 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
                 dock.setFloating(False)
                 self.addDockWidget(area, dock)
         # Left side
-        self.splitDockWidget(self._ui.export_options_dock, self._ui.mappings_dock, Qt.Vertical)
-        self.splitDockWidget(self._ui.mappings_dock, self._ui.mapping_spec_dock, Qt.Vertical)
+        self.splitDockWidget(self._ui.export_options_dock, self._ui.mappings_dock, Qt.Orientation.Vertical)
+        self.splitDockWidget(self._ui.mappings_dock, self._ui.mapping_spec_dock, Qt.Orientation.Vertical)
         some_docks = (self._ui.mappings_dock, self._ui.mapping_options_dock, self._ui.mapping_spec_dock)
         height = sum(d.height() for d in some_docks)
-        self.resizeDocks(some_docks, [height * x for x in (0.1, 0.3, 0.6)], Qt.Vertical)
-        self.splitDockWidget(self._ui.mappings_dock, self._ui.mapping_options_dock, Qt.Horizontal)
+        self.resizeDocks(some_docks, [height * x for x in (0.1, 0.3, 0.6)], Qt.Orientation.Vertical)
+        self.splitDockWidget(self._ui.mappings_dock, self._ui.mapping_options_dock, Qt.Orientation.Horizontal)
         # Right side
         some_docks = (self._ui.preview_tables_dock, self._ui.preview_controls_dock)
-        self.splitDockWidget(*some_docks, Qt.Vertical)
+        self.splitDockWidget(*some_docks, Qt.Orientation.Vertical)
         height = sum(d.height() for d in some_docks)
-        self.resizeDocks(some_docks, [height * x for x in (0.9, 0.1)], Qt.Vertical)
+        self.resizeDocks(some_docks, [height * x for x in (0.9, 0.1)], Qt.Orientation.Vertical)
         some_docks = (self._ui.preview_tables_dock, self._ui.preview_contents_dock)
-        self.splitDockWidget(*some_docks, Qt.Horizontal)
+        self.splitDockWidget(*some_docks, Qt.Orientation.Horizontal)
         width = sum(d.width() for d in some_docks)
-        self.resizeDocks(some_docks, [width * x for x in (0.3, 0.7)], Qt.Horizontal)
+        self.resizeDocks(some_docks, [width * x for x in (0.3, 0.7)], Qt.Orientation.Horizontal)
         qApp.processEvents()  # pylint: disable=undefined-variable
         self.resize(size)
 
     def _make_new_specification(self, spec_name):
         """See base class."""
         description = self._spec_toolbar.description()
-        self._new_spec.name = spec_name
-        self._new_spec.description = description
-        return deepcopy(self._new_spec)
+        mapping_specification = deepcopy(self._new_spec.mapping_specifications())
+        output_format = self._new_spec.output_format
+        return Specification(spec_name, description, mapping_specification, output_format)
 
     @Slot(str)
     def _change_format(self, current):
@@ -319,6 +324,25 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             self._ui.export_format_combo_box.setCurrentText(export_format.value)
             self._ui.export_format_combo_box.currentTextChanged.connect(self._change_format)
 
+    def _sniff_export_format(self):
+        """Tries to guess the export file format from user given export label.
+
+        Returns:
+            OutputFormat: export file format
+        """
+        if self.item.has_out_url():
+            return OutputFormat.SQL
+        out_labels = self.item.get_out_labels()
+        for label in out_labels:
+            name, separator, extension = label.rpartition(".")
+            if not separator:
+                continue
+            output_format = OutputFormat.output_format_from_extension(extension)
+            if output_format is None:
+                return OutputFormat.default()
+            return output_format if output_format is not None else OutputFormat.default()
+        return OutputFormat.default()
+
     def show_on_table(self, mapping_name):
         """
         Changes the current mapping.
@@ -346,7 +370,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             return
         self.current_mapping_about_to_change.emit()
         current = self._sort_mappings_table_model.mapToSource(current)
-        mapping_type = current.data(MappingsTableModel.MAPPING_TYPE_ROLE)
+        mapping_type = MappingType(current.data(MappingsTableModel.MAPPING_TYPE_ROLE))
         self._set_mapping_type_silently(mapping_type_to_combo_box_label[mapping_type])
         self._set_parameter_type_silently(mapping_type_to_parameter_type_label[mapping_type])
         self._set_always_export_header_silently(current.data(MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE))
@@ -381,10 +405,15 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             self._set_parameter_dimensions_silently(current.data(MappingsTableModel.PARAMETER_DIMENSIONS_ROLE))
         else:
             self._set_parameter_dimensions_silently(0)
+        root_mapping = current.data(MappingsTableModel.MAPPING_ROOT_ROLE)
         self._set_use_fixed_table_name_flag_silently(current.data(MappingsTableModel.USE_FIXED_TABLE_NAME_FLAG_ROLE))
         self._set_fixed_table_name_silently(current.data(MappingsTableModel.FIXED_TABLE_NAME_ROLE))
+        if any(
+            m.position == Position.table_name for m in root_mapping.flatten() if not isinstance(m, FixedValueMapping)
+        ):
+            self._ui.fix_table_name_check_box.setEnabled(False)
+            self._ui.fix_table_name_line_edit.setEnabled(False)
         self._set_group_fn_silently(group_function_display_from_name(current.data(MappingsTableModel.GROUP_FN_ROLE)))
-        root_mapping = current.data(MappingsTableModel.MAPPING_ROOT_ROLE)
         mapping_name = self._mappings_table_model.index(current.row(), 0).data()
         self._mapping_editor_model.set_mapping(mapping_name, root_mapping)
         self._enable_relationship_controls()
@@ -392,6 +421,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.mapping_spec_contents.setEnabled(True)
         self.current_mapping_changed.emit()
 
+    @Slot(bool)
     def _expect_current_mapping_change(self, expect_change):
         """Connects and disconnects signals depending of if current mapping is expected to change or not
 
@@ -414,9 +444,9 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             bottom_right (QModelIndex): bottom index of modified mappings
             roles (list of int):
         """
-        if Qt.DisplayRole in roles:
+        if Qt.ItemDataRole.DisplayRole in roles:
             self._sort_mappings_table_model.invalidate()
-        if max(roles) < Qt.UserRole:
+        if max(roles) < Qt.ItemDataRole.UserRole:
             return
         sorted_index = self._sort_mappings_table_model.mapFromSource(top_left)
         if sorted_index != self._ui.mappings_table.currentIndex():
@@ -425,12 +455,12 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         enable_controls = False
         if MappingsTableModel.MAPPING_ROOT_ROLE in roles:
             root_mapping = top_left.data(MappingsTableModel.MAPPING_ROOT_ROLE)
-            self._mapping_editor_model.set_mapping(top_left.data(Qt.DisplayRole), root_mapping)
+            self._mapping_editor_model.set_mapping(top_left.data(Qt.ItemDataRole.DisplayRole), root_mapping)
             self._set_relationship_dimensions_silently(top_left.data(MappingsTableModel.RELATIONSHIP_DIMENSIONS_ROLE))
             self._set_parameter_dimensions_silently(top_left.data(MappingsTableModel.PARAMETER_DIMENSIONS_ROLE))
             enable_controls = True
         if MappingsTableModel.MAPPING_TYPE_ROLE in roles:
-            mapping_type = top_left.data(MappingsTableModel.MAPPING_TYPE_ROLE)
+            mapping_type = MappingType(top_left.data(MappingsTableModel.MAPPING_TYPE_ROLE))
             self._set_mapping_type_silently(mapping_type_to_combo_box_label[mapping_type])
             self._set_parameter_type_silently(mapping_type_to_parameter_type_label[mapping_type])
             enable_controls = True
@@ -459,7 +489,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         Args:
             position (QPoint): requested menu position
         """
-        self._mappings_table_menu.exec_(self._ui.mappings_table.mapToGlobal(position))
+        self._mappings_table_menu.exec(self._ui.mappings_table.mapToGlobal(position))
 
     def _make_mappings_table_menu(self):
         """Creates context menu for Mappings table.
@@ -592,7 +622,10 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
     def _toggle_all_enabled(self):
         """Pushes a command that enables or disables all mappings to undo stack."""
         for row in range(self._mappings_table_model.rowCount()):
-            if self._mappings_table_model.index(row, 0).data(Qt.CheckStateRole) == Qt.Unchecked:
+            if (
+                self._mappings_table_model.index(row, 0).data(Qt.ItemDataRole.CheckStateRole)
+                == Qt.CheckState.Unchecked.value
+            ):
                 self._undo_stack.push(EnableAllMappings(self._mappings_table_model))
                 return
         self._undo_stack.push(DisableAllMappings(self._mappings_table_model))
@@ -693,10 +726,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             else:
                 mapping_type = MappingType.relationship_object_parameter_values
         elif type_label == "Object group":
-            if parameter_type_label == "None":
-                mapping_type = MappingType.object_groups
-            else:
-                mapping_type = MappingType.object_group_parameter_values
+            mapping_type = MappingType.object_groups
         else:
             mapping_type = {
                 "Alternative": MappingType.alternatives,
@@ -754,7 +784,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             checked (int): checked state
         """
         index = self._ui.mappings_table.currentIndex()
-        self._undo_stack.push(SetAlwaysExportHeader(index, checked == Qt.Checked))
+        self._undo_stack.push(SetAlwaysExportHeader(index, checked == Qt.CheckState.Checked.value))
 
     def _set_always_export_header_silently(self, always_export_header):
         """
@@ -766,7 +796,9 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         if always_export_header == self._ui.always_export_header_check_box.isChecked():
             return
         self._ui.always_export_header_check_box.stateChanged.disconnect(self._change_always_export_header)
-        self._ui.always_export_header_check_box.setCheckState(Qt.Checked if always_export_header else Qt.Unchecked)
+        self._ui.always_export_header_check_box.setCheckState(
+            Qt.CheckState.Checked if always_export_header else Qt.CheckState.Unchecked
+        )
         self._ui.always_export_header_check_box.stateChanged.connect(self._change_always_export_header)
 
     @Slot(int)
@@ -781,9 +813,9 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         mapping = self._mappings_table_model.data(index, MappingsTableModel.MAPPING_ROOT_ROLE)
         modified = deepcopy(mapping)
         if self._mappings_table_model.data(index, MappingsTableModel.MAPPING_TYPE_ROLE) in (
-            MappingType.object_parameter_values,
-            MappingType.relationship_parameter_values,
-            MappingType.relationship_object_parameter_values,
+            MappingType.object_parameter_values.value,
+            MappingType.relationship_parameter_values.value,
+            MappingType.relationship_object_parameter_values.value,
         ):
             set_parameter_dimensions(modified, dimensions)
         else:
@@ -814,7 +846,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         index = self._ui.mappings_table.currentIndex()
         if not index.isValid():
             return
-        flag = checked == Qt.Checked
+        flag = checked == Qt.CheckState.Checked.value
         self._undo_stack.beginMacro(("check" if flag else "uncheck") + " fix table name checkbox")
         self._undo_stack.push(SetUseFixedTableNameFlag(index, flag))
         mapping = index.data(MappingsTableModel.MAPPING_ROOT_ROLE)
@@ -847,7 +879,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         if flag == self._ui.fix_table_name_check_box.isChecked():
             return
         self._ui.fix_table_name_check_box.stateChanged.disconnect(self._change_fix_table_name_flag)
-        self._ui.fix_table_name_check_box.setCheckState(Qt.Checked if flag else Qt.Unchecked)
+        self._ui.fix_table_name_check_box.setCheckState(Qt.CheckState.Checked if flag else Qt.CheckState.Unchecked)
         self._ui.fix_table_name_check_box.stateChanged.connect(self._change_fix_table_name_flag)
 
     def _set_fixed_table_name_silently(self, name):
@@ -863,6 +895,71 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.fix_table_name_line_edit.setText(name)
         self._ui.fix_table_name_line_edit.editingFinished.connect(self._change_fix_table_name)
 
+    @Slot(QModelIndex, QModelIndex, list)
+    def _validate_fixed_table_name(self, top_left, bottom_right, roles):
+        """Checks if any mapping position is set to Table name and disables fixed table name accordingly.
+
+        Args:
+            top_left (QModelIndex): top left of changed mapping table model index
+            bottom_right (QModelIndex): bottom right of changed mapping table model index
+            roles (list of int): model roles
+        """
+        if (
+            Qt.ItemDataRole.DisplayRole not in roles
+            or top_left.column() > EditorColumn.POSITION
+            or bottom_right.column() < EditorColumn.POSITION
+        ):
+            return
+        for row in range(top_left.row(), bottom_right.row() + 1):
+            mapping_item = self._mapping_editor_model.index(row, EditorColumn.POSITION).data(
+                MappingEditorTableModel.MAPPING_ITEM_ROLE
+            )
+            if mapping_item.position == Position.table_name:
+                if self._ui.fix_table_name_check_box.isChecked():
+                    index = self._ui.mappings_table.currentIndex()
+                    mapping_root = index.data(MappingsTableModel.MAPPING_ROOT_ROLE)
+                    mapping_root = deepcopy(mapping_root)
+                    mapping_root = _remove_fixed_table_name(mapping_root)
+                    command = ClearFixedTableName(
+                        self,
+                        self._ui.fix_table_name_check_box.isChecked(),
+                        self._ui.fix_table_name_line_edit.text(),
+                        index,
+                        mapping_root,
+                    )
+                    self._undo_stack.push(command)
+                else:
+                    self._ui.fix_table_name_check_box.setEnabled(False)
+                    self._ui.fix_table_name_line_edit.setEnabled(False)
+                return
+        for row in range(top_left.model().rowCount()):
+            mapping_item = self._mapping_editor_model.index(row, EditorColumn.POSITION).data(
+                MappingEditorTableModel.MAPPING_ITEM_ROLE
+            )
+            if mapping_item.position == Position.table_name:
+                self._ui.fix_table_name_check_box.setEnabled(False)
+                self._ui.fix_table_name_line_edit.setEnabled(False)
+                return
+        self._ui.fix_table_name_check_box.setEnabled(True)
+
+    def clear_fixed_table_name(self):
+        """Clears and disable fixed table name widgets."""
+        self._set_fixed_table_name_silently("")
+        self._set_use_fixed_table_name_flag_silently(False)
+        self._ui.fix_table_name_check_box.setEnabled(False)
+
+    def enable_fixed_table_name(self, checked, name):
+        """Enables and sets fixed table name widgets.
+
+        Args:
+            checked (bool): fixed table name checkbox state
+            name (str): fixed table name
+        """
+        self._set_use_fixed_table_name_flag_silently(checked)
+        self._ui.fix_table_name_check_box.setEnabled(True)
+        self._set_fixed_table_name_silently(name)
+        self._ui.fix_table_name_line_edit.setEnabled(True)
+
     @Slot(str)
     def _change_root_mapping_group_fn(self, text):
         """
@@ -963,21 +1060,13 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
     def _enable_parameter_controls(self):
         """Enables and disables controls related to relationship export."""
         mapping_type = self._ui.item_type_combo_box.currentText()
-        types_with_parameters = {
-            "Object class",
-            "Object group",
-            "Relationship class",
-            "Relationship class with object parameter",
-        }
+        types_with_parameters = {"Object class", "Relationship class", "Relationship class with object parameter"}
         if mapping_type in types_with_parameters:
             self._ui.parameter_type_combo_box.setEnabled(True)
             self._ui.parameter_type_combo_box.currentTextChanged.disconnect(self._change_mapping_type)
             model = self._ui.parameter_type_combo_box.model()
             default_value_item = model.item(1)
-            if mapping_type == "Object group":
-                default_value_item.setFlags(default_value_item.flags() & ~Qt.ItemIsEnabled)
-            else:
-                default_value_item.setFlags(default_value_item.flags() | Qt.ItemIsEnabled)
+            default_value_item.setFlags(default_value_item.flags() | Qt.ItemIsEnabled)
             no_value_item = model.item(2)
             if mapping_type == "Relationship class with object parameter":
                 no_value_item.setFlags(default_value_item.flags() & ~Qt.ItemIsEnabled)
@@ -1019,15 +1108,6 @@ def _new_mapping_specification(mapping_type):
         return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, object_export(0, 1))
     if mapping_type == MappingType.object_groups:
         return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, object_group_export(0, 1, 2))
-    if mapping_type == MappingType.object_group_parameter_values:
-        return MappingSpecification(
-            mapping_type,
-            True,
-            True,
-            NoGroup.NAME,
-            False,
-            object_group_parameter_export(0, 1, Position.hidden, 2, 3, 4, Position.hidden, 5, None, None),
-        )
     if mapping_type == MappingType.object_parameter_default_values:
         return MappingSpecification(
             mapping_type,
