@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,8 +16,8 @@ Contains undo and redo commands for Import editor.
 """
 import pickle
 from enum import auto, IntEnum, unique
-from PySide2.QtCore import Qt
-from PySide2.QtWidgets import QUndoCommand
+from PySide6.QtCore import Qt
+from PySide6.QtGui import QUndoCommand
 
 from spinedb_api import import_mapping_from_dict
 from spinedb_api.import_mapping.import_mapping_compat import parse_named_mapping_spec
@@ -735,7 +735,7 @@ class SetColumnOrRowType(QUndoCommand):
             new_type (ConvertSpec): conversion specification for the rows/columns
             previous_type (ConvertSpec): previous conversion specification for the rows/columns
         """
-        text = ("row" if header_widget.orientation() == Qt.Vertical else "column") + " type change"
+        text = ("row" if header_widget.orientation() == Qt.Orientation.Vertical else "column") + " type change"
         super().__init__(text)
         self._source_table_name = source_table_name
         self._header_widget = header_widget
@@ -752,6 +752,30 @@ class SetColumnOrRowType(QUndoCommand):
         self._header_widget.set_data_types(self._source_table_name, self._sections, self._previous_type)
 
 
+class SetColumnDefaultType(QUndoCommand):
+    """Command to change the default type of columns."""
+
+    def __init__(self, import_sources, source_table_name, new_type, previous_type):
+        """
+        Args:
+            import_sources (ImportSources): import sources manager
+            source_table_name (str): name of the source table
+            new_type (str): new column type
+            previous_type (str): previous column type
+        """
+        super().__init__("surplus column type")
+        self._import_sources = import_sources
+        self._table_name = source_table_name
+        self._new_type = new_type
+        self._previous_type = previous_type
+
+    def redo(self):
+        self._import_sources.change_default_column_type(self._table_name, self._new_type)
+
+    def undo(self):
+        self._import_sources.change_default_column_type(self._table_name, self._previous_type)
+
+
 class RestoreMappingsFromDict(QUndoCommand):
     """Restores mappings from a dict."""
 
