@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -568,6 +568,7 @@ class FlattenedMappings:
         last_cls_mapping.child = last_cls_mapping_child
         last_obj_mapping.child = last_obj_mapping_child
         self.set_root_mapping(self._components[0])
+        self._ensure_consistent_import_objects()
 
     def may_import_objects(self):
         """Checks if the mappings can optionally import objects.
@@ -602,13 +603,20 @@ class FlattenedMappings:
         for m in self._import_objects_mappings():
             m.import_objects = import_objects
 
+    def _ensure_consistent_import_objects(self):
+        """If any mapping has the import objects flag set, sets the flag also for all other mappings."""
+        mappings = self._import_objects_mappings()
+        if any(mapping.import_objects for mapping in mappings):
+            for m in mappings:
+                m.import_objects = True
+
     def has_parameters(self):
         """Returns True if the mappings have parameters.
 
         Returns:
             bool: True if mapping has parameter, False otherwise
         """
-        return self._map_type in (MappingType.ObjectClass, MappingType.ObjectGroup, MappingType.RelationshipClass)
+        return self._map_type in (MappingType.ObjectClass, MappingType.RelationshipClass)
 
     def _parameter_definition_component(self):
         """Searches for ParameterDefinitionMapping within the components.
@@ -658,10 +666,10 @@ class FlattenedMappings:
         """
         value_mapping = self.value_mapping()
         if value_mapping is None:
-            return "None"
+            return "<no label>"
         if isinstance(value_mapping, (ParameterValueTypeMapping, ParameterValueMapping)):
-            return "Value"
-        return "Default value"
+            return "Value:"
+        return "Default value:"
 
     def value_mapping(self):
         """Finds first parameter value mapping component from flattened mappings.
