@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -17,22 +17,18 @@ Contains Importer project item class.
 """
 
 import os
-from PySide2.QtCore import QModelIndex, Qt, Slot
+from operator import itemgetter
+from PySide6.QtCore import QModelIndex, Qt, Slot
 from spinetoolbox.helpers import create_dir
-from spinetoolbox.project_item.project_item import ProjectItem
 from spinetoolbox.widgets.custom_menus import ItemSpecificationMenu
-from ..commands import (
-    UpdateCancelOnErrorCommand,
-    ChangeItemSelectionCommand,
-    UpdatePurgeBeforeWritingCommand,
-    UpdateOnConflictCommand,
-)
+from ..db_writer_item_base import DBWriterItemBase
+from ..commands import UpdateCancelOnErrorCommand, ChangeItemSelectionCommand, UpdateOnConflictCommand
 from ..models import CheckableFileListModel
 from .executable_item import ExecutableItem
 from .item_info import ItemInfo
 
 
-class Importer(ProjectItem):
+class Importer(DBWriterItemBase):
     def __init__(
         self,
         name,
@@ -43,7 +39,6 @@ class Importer(ProjectItem):
         project,
         specification_name="",
         cancel_on_error=True,
-        purge_before_writing=False,
         on_conflict="merge",
         file_selection=None,
     ):
@@ -58,8 +53,8 @@ class Importer(ProjectItem):
             project (SpineToolboxProject): the project this item belongs to
             specification_name (str, optional): a spec name
             cancel_on_error (bool): if True the item's execution will stop on import error
-            purge_before_writing (bool): if True the item will purge target dbs before running
-            file_selection (dict): a map from label to a bool indicating if the file item is checked
+            on_conflict (str): how to handle conflicts between parallel importers
+            file_selection (dict, optional): a map from label to a bool indicating if the file item is checked
         """
         super().__init__(name, description, x, y, project)
         # Make logs subdirectory for this item
@@ -76,7 +71,6 @@ class Importer(ProjectItem):
                 f"Importer <b>{self.name}</b> should have a specification <b>{specification_name}</b> but it was not found"
             )
         self.cancel_on_error = cancel_on_error
-        self._purge_before_writing = purge_before_writing
         self.on_conflict = on_conflict
         self._file_model = CheckableFileListModel(header_label="Available resources")
         self._file_model.set_initial_state(file_selection if file_selection is not None else dict())
@@ -96,23 +90,6 @@ class Importer(ProjectItem):
     def executable_class(self):
         return ExecutableItem
 
-    @Slot(object, object)
-    def handle_execution_successful(self, execution_direction, engine_state):
-        """Notifies Toolbox of successful database import."""
-        if execution_direction != "FORWARD":
-            return
-        successors = [self._project.get_item(name) for name in self._project.successor_names(self.name)]
-        committed_db_maps = set()
-        for successor in successors:
-            if successor.item_type() == "Data Store":
-                url = successor.sql_alchemy_url()
-                db_map = self._toolbox.db_mngr.db_map(url)
-                if db_map:
-                    committed_db_maps.add(db_map)
-        if committed_db_maps:
-            cookie = self
-            self._toolbox.db_mngr.session_committed.emit(committed_db_maps, cookie)
-
     def make_signal_handler_dict(self):
         """Returns a dictionary of all shared signals and their handlers.
         This is to enable simpler connecting and disconnecting."""
@@ -124,7 +101,6 @@ class Importer(ProjectItem):
         s[self._properties_ui.radioButton_on_conflict_merge.clicked] = self._update_on_conflict
         s[self._properties_ui.radioButton_on_conflict_keep.clicked] = self._update_on_conflict
         s[self._properties_ui.radioButton_on_conflict_replace.clicked] = self._update_on_conflict
-        s[self._properties_ui.checkBox_purge_before_writing.stateChanged] = self._handle_purge_before_writing_changed
         return s
 
     @Slot(str)
@@ -154,32 +130,11 @@ class Importer(ProjectItem):
         self.cancel_on_error = cancel_on_error
         if not self._active:
             return
-        check_state = Qt.Checked if self.cancel_on_error else Qt.Unchecked
+        check_state = Qt.CheckState.Checked if self.cancel_on_error else Qt.CheckState.Unchecked
         self._properties_ui.cancel_on_error_checkBox.blockSignals(True)
         self._properties_ui.cancel_on_error_checkBox.setCheckState(check_state)
         self._properties_ui.cancel_on_error_checkBox.blockSignals(False)
 
-    @Slot(int)
-    def _handle_purge_before_writing_changed(self, _state):
-        purge_before_writing = self._properties_ui.checkBox_purge_before_writing.isChecked()
-        if self._purge_before_writing == purge_before_writing:
-            return
-        self._toolbox.undo_stack.push(UpdatePurgeBeforeWritingCommand(self, purge_before_writing))
-
-    def set_purge_before_writing(self, purge_before_writing):
-        """Sets purge_before_writing setting.
-
-        Args:
-            purge_before_writing (bool): purge_before_writing flag
-        """
-        self._purge_before_writing = purge_before_writing
-        if not self._active:
-            return
-        check_state = Qt.Checked if self._purge_before_writing else Qt.Unchecked
-        self._properties_ui.checkBox_purge_before_writing.blockSignals(True)
-        self._properties_ui.checkBox_purge_before_writing.setCheckState(check_state)
-        self._properties_ui.checkBox_purge_before_writing.blockSignals(False)
-
     def _on_conflict(self):
         """Reads the on_conflict strategy from UI."""
         strategies = {
@@ -217,9 +172,8 @@ class Importer(ProjectItem):
 
     def restore_selections(self):
         """Restores selections into shared widgets when this project item is selected."""
-        self._properties_ui.cancel_on_error_checkBox.setCheckState(Qt.Checked if self.cancel_on_error else Qt.Unchecked)
-        self._properties_ui.checkBox_purge_before_writing.setCheckState(
-            Qt.Checked if self._purge_before_writing else Qt.Unchecked
+        self._properties_ui.cancel_on_error_checkBox.setCheckState(
+            Qt.CheckState.Checked if self.cancel_on_error else Qt.CheckState.Unchecked
         )
         self._set_on_conflict()
         self._properties_ui.treeView_files.setModel(self._file_model)
@@ -331,41 +285,29 @@ class Importer(ProjectItem):
         else:
             d["specification"] = self.specification().name
         d["cancel_on_error"] = self.cancel_on_error
-        d["purge_before_writing"] = self._purge_before_writing
         d["on_conflict"] = self.on_conflict
         selections = list()
         for row in range(self._file_model.rowCount()):
             label, selected = self._file_model.checked_data(self._file_model.index(row, 0))
             selections.append([label, selected])
-        d["file_selection"] = selections
+        d["file_selection"] = sorted(selections, key=itemgetter(0))
         return d
 
     @staticmethod
     def from_dict(name, item_dict, toolbox, project):
         """See base class."""
-        description, x, y = ProjectItem.parse_item_dict(item_dict)
+        description, x, y = DBWriterItemBase.parse_item_dict(item_dict)
         specification_name = item_dict.get("specification", "")
         cancel_on_error = item_dict.get("cancel_on_error", False)
-        purge_before_writing = item_dict.get("purge_before_writing", False)
         on_conflict = item_dict.get("on_conflict", "merge")
         file_selection = {label: selected for label, selected in item_dict.get("file_selection", list())}
         return Importer(
-            name,
-            description,
-            x,
-            y,
-            toolbox,
-            project,
-            specification_name,
-            cancel_on_error,
-            purge_before_writing,
-            on_conflict,
-            file_selection,
+            name, description, x, y, toolbox, project, specification_name, cancel_on_error, on_conflict, file_selection
         )
 
     def notify_destination(self, source_item):
         """See base class."""
-        if source_item.item_type() in ("Data Connection", "Tool", "Gimlet", "GdxExporter", "Exporter"):
+        if source_item.item_type() in ("Data Connection", "Tool", "Exporter"):
             self._logger.msg.emit(
                 "Link established. You can define mappings on data from "
                 f"<b>{source_item.name}</b> using <b>{self.name}</b>."
