@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -15,11 +15,13 @@ Contains a model to handle source tables and import mapping.
 :author: A. Soininen (VTT)
 :date:   7.10.2021
 """
+from __future__ import annotations
 from dataclasses import dataclass, field
 from enum import IntEnum, unique
 import re
 
-from PySide2.QtCore import QAbstractItemModel, QModelIndex, Qt, Signal
+from PySide6.QtCore import QAbstractItemModel, QModelIndex, Qt, Signal
+from PySide6.QtGui import QColor, QFont
 
 from spinetoolbox.helpers import unique_name
 from spinedb_api.parameter_value import join_value_and_type, split_value_and_type
@@ -49,7 +51,7 @@ from ..mapping_colors import ERROR_COLOR
 from .mappings_model_roles import Role
 
 
-UNNAMED_TABLE_NAME = "<unnamed table>"
+UNNAMED_TABLE_NAME = "<rename this to add table>"
 
 
 @unique
@@ -69,7 +71,10 @@ class SourceTableItem:
     checkable: bool = True
     editable: bool = False
     real: bool = True
-    mapping_list: list = field(init=False, default_factory=list)
+    in_source: bool = False
+    in_specification: bool = False
+    empty: bool = False
+    mapping_list: list[MappingListItem] = field(init=False, default_factory=list)
 
     def append_to_mapping_list(self, list_item):
         """Appends an item to mapping list.
@@ -122,6 +127,8 @@ class MappingsModel(QAbstractItemModel):
     multi_column_type_recommended = Signal(object, object)
     """Emitted when all but given columns should be of given type."""
 
+    _NOT_IN_SOURCE_COLOR = QColor(Qt.GlobalColor.gray)
+
     def __init__(self, undo_stack, parent):
         """
         Args:
@@ -130,6 +137,8 @@ class MappingsModel(QAbstractItemModel):
         super().__init__(parent)
         self._undo_stack = undo_stack
         self._mappings = [self._make_select_all_tables_item()]
+        self._add_table_row_font = QFont()
+        self._add_table_row_font.setItalic(True)
 
     def real_table_names(self):
         """Returns real table names.
@@ -166,12 +175,12 @@ class MappingsModel(QAbstractItemModel):
             return 4
         return 0
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         index_item = index.internalPointer()
         if isinstance(index_item, SourceTableItem):
             return self._table_list_data(index, role)
         if isinstance(index_item, MappingListItem):
-            return self._mapping_list_data(index_item, index, role)
+            return self._mapping_list_data(index_item, role)
         if isinstance(index_item, FlattenedMappings):
             return self._mapping_data(index_item, index, role)
         return None
@@ -186,27 +195,49 @@ class MappingsModel(QAbstractItemModel):
         Returns:
             Any: data
         """
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role == Qt.ItemDataRole.DisplayRole:
+            row = index.row()
+            list_item = self._mappings[row]
+            if row == 0 or list_item.in_specification or list_item.empty:
+                return list_item.name
+            return list_item.name + " (new)"
+        if role == Qt.ItemDataRole.EditRole:
             return self._mappings[index.row()].name
-        if role == Qt.CheckStateRole:
-            return Qt.Checked if self._mappings[index.row()].checked else Qt.Unchecked
+        if role == Qt.ItemDataRole.CheckStateRole:
+            return Qt.CheckState.Checked if self._mappings[index.row()].checked else Qt.CheckState.Unchecked
+        if role == Qt.ItemDataRole.ForegroundRole:
+            row = index.row()
+            item = self._mappings[row]
+            return self._NOT_IN_SOURCE_COLOR if row > 0 and (not item.in_source and not item.empty) else None
+        if role == Qt.ItemDataRole.ToolTipRole:
+            row = index.row()
+            if row == 0:
+                return None
+            list_item = self._mappings[row]
+            if not list_item.empty:
+                if not list_item.in_source:
+                    return "Table isn't in source data."
+                if not list_item.in_specification:
+                    return "Table's mappings haven't been saved with the specification yet."
+            return None
+        if role == Qt.ItemDataRole.FontRole:
+            return self._add_table_row_font if self._mappings[index.row()].empty else None
         if role == Role.ITEM:
             return self._mappings[index.row()]
         return None
 
     @staticmethod
-    def _mapping_list_data(list_item, index, role):
+    def _mapping_list_data(list_item, role):
         """Returns mapping list data for given role.
 
         Args:
             list_item (MappingListItem): list item
-            index (QModelIndex): list index
             role (int): Qt's data role
 
         Returns:
             Any: data
         """
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             return list_item.name
         if role == Role.ITEM:
             return list_item
@@ -235,7 +266,7 @@ class MappingsModel(QAbstractItemModel):
             except ParameterValueFormatError:
                 return None
         column = index.column()
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             if column == FlattenedColumn.NAME:
                 return flattened_mappings.display_names[index.row()]
             if column == FlattenedColumn.POSITION_TYPE:
@@ -248,7 +279,7 @@ class MappingsModel(QAbstractItemModel):
             if column == FlattenedColumn.REGEXP:
                 return flattened_mappings.component_at(index.row()).filter_re
             raise RuntimeError("Column out of bounds.")
-        if role == Qt.BackgroundRole:
+        if role == Qt.ItemDataRole.BackgroundRole:
             if column == FlattenedColumn.NAME:
                 return flattened_mappings.display_colors[index.row()]
             if column == FlattenedColumn.POSITION:
@@ -256,7 +287,7 @@ class MappingsModel(QAbstractItemModel):
                 if issues:
                     return ERROR_COLOR
                 return None
-        if role == Qt.ToolTipRole:
+        if role == Qt.ItemDataRole.ToolTipRole:
             if column == FlattenedColumn.POSITION:
                 issues = flattened_mappings.display_row_issues(index.row())
                 if issues:
@@ -266,8 +297,8 @@ class MappingsModel(QAbstractItemModel):
                 return "Enter regular expression to filter importer data."
         return None
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
             return ("Target", "Source type", "Source ref.", "Filter")[section]
         return None
 
@@ -290,7 +321,7 @@ class MappingsModel(QAbstractItemModel):
         Returns:
             int: flags
         """
-        flags = Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable
+        flags = Qt.ItemIsEnabled | Qt.ItemIsSelectable
         table_item = self._mappings[index.row()]
         if table_item.checkable:
             flags |= Qt.ItemIsUserCheckable
@@ -423,7 +454,7 @@ class MappingsModel(QAbstractItemModel):
         try:
             for table_name, mapping_list_dicts in mappings_dict.get("table_mappings", {}).items():
                 checked = table_name in checked_tables if checked_tables is not None else True
-                table_item = SourceTableItem(table_name, checked)
+                table_item = SourceTableItem(table_name, checked, in_specification=True)
                 for list_dict in mapping_list_dicts:
                     mapping_name, root_mapping = parse_named_mapping_spec(list_dict)
                     flattened_mappings = FlattenedMappings(root_mapping)
@@ -449,7 +480,7 @@ class MappingsModel(QAbstractItemModel):
             return len(parent_item.flattened_mappings.display_names)
         return 0
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
         index_item = index.internalPointer()
         if isinstance(index_item, SourceTableItem):
             return self._set_table_list_data(index_item, index, value, role)
@@ -472,14 +503,14 @@ class MappingsModel(QAbstractItemModel):
             bool: True if data was set successfully, False otherwise
         """
         row = index.row()
-        if role == Qt.CheckStateRole and table_item.checkable:
-            checked = value == Qt.Checked
+        if role == Qt.ItemDataRole.CheckStateRole and table_item.checkable:
+            checked = value == Qt.CheckState.Checked.value
             if row == 0:
                 self._set_multiple_checked_undoable(checked, *range(1, len(self._mappings)))
             else:
                 self._undo_stack.push(SetTableChecked(table_item.name, self, checked, row))
             return True
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             if value == table_item.name:
                 return False
             if value in {table.name for table in self._mappings if table.real}:
@@ -487,7 +518,7 @@ class MappingsModel(QAbstractItemModel):
                 return False
             previous = {"name": table_item.name}
             updates = {"name": value}
-            self._undo_stack.push(UpdateTableItem(self, row, previous, updates, row == len(self._mappings) - 1))
+            self._undo_stack.push(UpdateTableItem(self, row, previous, updates, table_item.empty))
             return True
         return False
 
@@ -503,7 +534,7 @@ class MappingsModel(QAbstractItemModel):
         Returns:
             bool: True if data was set successfully, False otherwise
         """
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             if not value or value in {m.name for m in list_item.source_table_item.mapping_list}:
                 return False
             previous = list_item.name
@@ -520,8 +551,8 @@ class MappingsModel(QAbstractItemModel):
         Args:
             row (int): item row
             data_dict (dict): table item data to update
-            add_empty_row (bool): whether or not to add an empty row
-            remove_empty_row (bool): whether or not to remove the empty row
+            add_empty_row (bool): whether to add an empty row
+            remove_empty_row (bool): whether to remove the empty row
         """
         table_item = self._mappings[row]
         for field_name, value in data_dict.items():
@@ -531,6 +562,8 @@ class MappingsModel(QAbstractItemModel):
             table_item.real = True
             table_item.checkable = True
             table_item.checked = True
+            table_item.empty = False
+            table_item.in_source = True
             default_flattened_mappings = FlattenedMappings(self._create_default_mapping())
             mapping_list_item = MappingListItem(self._unique_mapping_name(table_item))
             mapping_list_item.set_flattened_mappings(default_flattened_mappings)
@@ -542,6 +575,8 @@ class MappingsModel(QAbstractItemModel):
             table_item.real = False
             table_item.checkable = False
             table_item.checked = False
+            table_item.empty = True
+            table_item.in_source = False
             self.beginRemoveRows(index, 0, len(table_item.mapping_list) - 1)
             table_item.mapping_list.clear()
             self.endRemoveRows()
@@ -583,7 +618,7 @@ class MappingsModel(QAbstractItemModel):
         min_row = None
         max_row = None
         for row in rows:
-            if self._mappings[row].checked == checked:
+            if not self._mappings[row].checkable or self._mappings[row].checked == checked:
                 continue
             if min_row is None:
                 min_row = row
@@ -596,7 +631,7 @@ class MappingsModel(QAbstractItemModel):
             return
         top_left = self.index(min_row, 0)
         bottom_right = self.index(max_row, 0)
-        self.dataChanged.emit(top_left, bottom_right, [Qt.CheckStateRole])
+        self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.CheckStateRole])
         self._update_all_checked()
 
     def _update_all_checked(self):
@@ -607,22 +642,22 @@ class MappingsModel(QAbstractItemModel):
         if all_checked_item.checked != all_checked:
             self._mappings[0].checked = all_checked
             index = self.index(0, 0)
-            self.dataChanged.emit(index, index, [Qt.CheckStateRole])
+            self.dataChanged.emit(index, index, [Qt.ItemDataRole.CheckStateRole])
 
     def append_new_table_with_mapping(self, table_name, root_mapping):
         """Appends a new source table and adds given ``root_mapping`` as its first mapping.
 
         Args:
             table_name (str): table name
-            root_mapping (ImportMapping, optional): initial mapping; if None,
-                an object class mapping will be created
+            root_mapping (ImportMapping, optional): initial mapping
         """
-        if root_mapping is None:
+        has_root_mapping = root_mapping is not None
+        if not has_root_mapping:
             root_mapping = self._create_default_mapping()
         flattened_mappings = FlattenedMappings(root_mapping)
         list_item = MappingListItem("Mapping 1")
         list_item.set_flattened_mappings(flattened_mappings)
-        table_item = SourceTableItem(table_name, checked=True)
+        table_item = SourceTableItem(table_name, checked=True, in_source=True, in_specification=has_root_mapping)
         table_item.append_to_mapping_list(list_item)
         self.beginInsertRows(QModelIndex(), len(self._mappings), len(self._mappings))
         self._mappings.append(table_item)
@@ -631,7 +666,7 @@ class MappingsModel(QAbstractItemModel):
     def _remove_last_source_table(self):
         """Removes the last source table."""
         self.beginRemoveRows(QModelIndex(), self.rowCount() - 1, self.rowCount() - 1)
-        self._mappings.pop(-1)
+        del self._mappings[-1]
         self.endRemoveRows()
 
     def rename_mapping(self, table_row, list_row, name):
@@ -655,7 +690,7 @@ class MappingsModel(QAbstractItemModel):
         list_item.name = name
         parent = self.index(table_row, 0)
         index = self.index(list_row, 0, parent)
-        self.dataChanged.emit(index, index, [Qt.DisplayRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
 
     def set_root_mapping(self, table_row, list_row, root_mapping):
         """
@@ -716,7 +751,7 @@ class MappingsModel(QAbstractItemModel):
         Returns:
             bool: True if data was set successfully, False otherwise
         """
-        if role != Qt.EditRole:
+        if role != Qt.ItemDataRole.EditRole:
             return False
         column = index.column()
         if column < FlattenedColumn.POSITION_TYPE:
@@ -958,7 +993,11 @@ class MappingsModel(QAbstractItemModel):
         list_index = self.index(list_row, 0, table_index)
         top_left = self.index(row, 1, list_index)
         bottom_right = self.index(self.rowCount(list_index) - 1, 2, list_index)
-        self.dataChanged.emit(top_left, bottom_right, [Qt.BackgroundRole, Qt.DisplayRole, Qt.ToolTipRole])
+        self.dataChanged.emit(
+            top_left,
+            bottom_right,
+            [Qt.ItemDataRole.BackgroundRole, Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ToolTipRole],
+        )
 
     def set_mapping_position(self, table_row, list_row, row, position_type, position):
         """Modifies mapping component's position
@@ -976,7 +1015,11 @@ class MappingsModel(QAbstractItemModel):
         list_index = self.index(list_row, 0, table_index)
         top_left = self.index(row, 1, list_index)
         bottom_right = self.index(self.rowCount(list_index) - 1, 2, list_index)
-        self.dataChanged.emit(top_left, bottom_right, [Qt.BackgroundRole, Qt.DisplayRole, Qt.ToolTipRole])
+        self.dataChanged.emit(
+            top_left,
+            bottom_right,
+            [Qt.ItemDataRole.BackgroundRole, Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ToolTipRole],
+        )
         self._recommend_source_table_column_or_row_types(flattened_mappings, row)
 
     def set_import_objects(self, table_row, list_row, import_objects):
@@ -1057,7 +1100,7 @@ class MappingsModel(QAbstractItemModel):
         flattened_mappings = self._mappings[table_row].mapping_list[list_row].flattened_mappings
         if not flattened_mappings.is_time_series_value():
             return
-        flattened_mappings.value_mapping.options["repeat"] = repeat
+        flattened_mappings.value_mapping().options["repeat"] = repeat
         table_index = self.index(table_row, 0)
         list_index = self.index(list_row, 0, table_index)
         self.dataChanged.emit(list_index, list_index, [Role.FLATTENED_MAPPINGS])
@@ -1076,7 +1119,7 @@ class MappingsModel(QAbstractItemModel):
         table_index = self.index(table_row, 0)
         list_index = self.index(list_row, 0, table_index)
         index = self.index(row, FlattenedColumn.REGEXP, list_index)
-        self.dataChanged.emit(index, index, [Qt.DisplayRole])
+        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
 
     @staticmethod
     def polish_mapping(list_index, header):
@@ -1160,9 +1203,9 @@ class MappingsModel(QAbstractItemModel):
         if not isinstance(component.position, int):
             return
         if component.position >= 0:
-            self.row_or_column_type_recommended.emit(component.position, convert_spec, Qt.Horizontal)
+            self.row_or_column_type_recommended.emit(component.position, convert_spec, Qt.Orientation.Horizontal)
         else:
-            self.row_or_column_type_recommended.emit(-(component.position + 1), convert_spec, Qt.Vertical)
+            self.row_or_column_type_recommended.emit(-(component.position + 1), convert_spec, Qt.Orientation.Vertical)
 
     def set_tables_editable(self, editable):
         """Enables or disables table name editing.
@@ -1171,14 +1214,16 @@ class MappingsModel(QAbstractItemModel):
             editable (bool): True to enable table name editing, False to disable
         """
         for item in self._mappings[1:]:
-            if item.name == UNNAMED_TABLE_NAME:
+            if item.empty:
                 continue
             item.editable = editable
 
     def add_empty_row(self):
         """Adds the special 'unnamed table' row at the end of table list."""
         last_row = len(self._mappings) - 1
-        empty_item = SourceTableItem(UNNAMED_TABLE_NAME, checked=False, checkable=False, editable=True, real=False)
+        empty_item = SourceTableItem(
+            UNNAMED_TABLE_NAME, checked=False, checkable=False, editable=True, real=False, empty=True
+        )
         self.beginInsertRows(QModelIndex(), last_row, last_row)
         self._mappings.append(empty_item)
         self.endInsertRows()
@@ -1198,12 +1243,12 @@ class MappingsModel(QAbstractItemModel):
         parent_item = parent.internalPointer()
         if parent_item is None:
             self.beginRemoveRows(parent, row, row)
-            self._mappings.pop(row)
+            del self._mappings[row]
             self.endRemoveRows()
             return True
         if isinstance(parent_item, SourceTableItem):
             self.beginRemoveRows(parent, row, row)
-            parent_item.mapping_list.pop(row)
+            del parent_item.mapping_list[row]
             self.endRemoveRows()
             return True
         return False
@@ -1267,3 +1312,45 @@ class MappingsModel(QAbstractItemModel):
             str: name
         """
         return unique_name(prefix, [i.name for i in table_item.mapping_list])
+
+    def set_source_table_items_into_specification(self):
+        """Sets the in_specification flag for all appropriate source table items."""
+        bottom = 0
+        for mapping in self._mappings[1:]:
+            mapping.in_specification = True
+            bottom += 1
+        if bottom > 0:
+            top_left = self.index(1, 0)
+            bottom_right = self.index(bottom, 0)
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
+
+    def remove_tables_not_in_source_and_specification(self):
+        """Removes source table items that are not in source or saved in specification."""
+        start = 1
+        for row, table_item in reversed(list(enumerate(self._mappings[start:], start))):
+            if not table_item.in_source and not table_item.in_specification:
+                self.removeRow(row)
+
+    def cross_check_source_table_names(self, table_names):
+        """Sets the in_source flag for all appropriate source table items.
+
+        Args:
+            table_names (set of str): existing source table names
+        """
+        changed_rows = []
+        start = 1
+        for row, table_item in enumerate(self._mappings[start:], start):
+            in_source = table_item.name in table_names
+            changed = False
+            if table_item.in_source != in_source:
+                table_item.in_source = in_source
+                changed = True
+            if table_item.real != in_source:
+                table_item.real = in_source
+                changed = True
+            if changed:
+                changed_rows.append(row)
+        if changed_rows:
+            top_left = self.index(changed_rows[0], 0)
+            bottom_right = self.index(changed_rows[-1], 0)
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.ForegroundRole])
