@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -14,11 +14,11 @@ Contains user data roles for :class:`MappingsModel`.
 :author: A. Soininen (VTT)
 :date:   15.10.2021
 """
-from PySide2.QtCore import Qt
+from PySide6.QtCore import Qt
 from enum import IntEnum, unique
 
 
 @unique
 class Role(IntEnum):
-    ITEM = Qt.UserRole + 1
-    FLATTENED_MAPPINGS = Qt.UserRole + 2
+    ITEM = Qt.ItemDataRole.UserRole + 1
+    FLATTENED_MAPPINGS = Qt.ItemDataRole.UserRole + 2
