@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,12 +16,11 @@ Classes for context menus used alongside the Importer project item.
 :date:   9.1.2018
 """
 
-from PySide2.QtCore import Qt, QPoint, Slot, Signal
-from PySide2.QtWidgets import QMenu, QWidgetAction
-
+from PySide6.QtCore import QPoint, Slot, Signal
+from PySide6.QtWidgets import QMenu
 from spinetoolbox.widgets.custom_menus import CustomContextMenu, ItemSpecificationMenu, FilterMenuBase
+from spinetoolbox.mvcmodels.filter_checkbox_list_model import DataToValueFilterCheckboxListModel
 from ..mvcmodels.mappings_model_roles import Role
-from .simple_filter_widget import SimpleFilterWidget
 
 
 class FilesContextMenu(CustomContextMenu):
@@ -54,7 +53,6 @@ class SourceListMenu(CustomContextMenu):
     """
 
     def __init__(self, parent, position, can_paste_option, can_paste_mapping):
-
         super().__init__(parent, position)
         self.add_action("Copy options")
         self.add_action("Copy mappings")
@@ -71,7 +69,6 @@ class MappingListMenu(CustomContextMenu):
     """
 
     def __init__(self, parent, position, can_copy, can_paste):
-
         super().__init__(parent, position)
         self.add_action("Copy mapping(s)", enabled=can_copy)
         self.add_action("Paste mapping(s)", enabled=can_paste)
@@ -141,7 +138,6 @@ class SourceDataTableMenu(QMenu):
 
 
 class SimpleFilterMenu(FilterMenuBase):
-
     filterChanged = Signal(set)
 
     def __init__(self, parent, show_empty=True):
@@ -150,11 +146,7 @@ class SimpleFilterMenu(FilterMenuBase):
             parent (SpineDBEditor)
         """
         super().__init__(parent)
-        self._filter = SimpleFilterWidget(self, show_empty=show_empty)
-        self._filter_action = QWidgetAction(parent)
-        self._filter_action.setDefaultWidget(self._filter)
-        self.addAction(self._filter_action)
-        self.connect_signals()
+        self._set_up(DataToValueFilterCheckboxListModel, self, str, show_empty=show_empty)
 
     def emit_filter_changed(self, valid_values):
         self.filterChanged.emit(valid_values)
