@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -19,8 +19,8 @@ Contains ImportPreviewWindow class.
 import os
 import json
 import fnmatch
-from PySide2.QtCore import Qt, Signal, Slot
-from PySide2.QtWidgets import QFileDialog, QDockWidget, QDialog, QVBoxLayout, QListWidget, QDialogButtonBox
+from PySide6.QtCore import Qt, Signal, Slot, QModelIndex, QItemSelectionModel
+from PySide6.QtWidgets import QFileDialog, QDockWidget, QDialog, QVBoxLayout, QListWidget, QDialogButtonBox
 from spinetoolbox.project_item.specification_editor_window import SpecificationEditorWindowBase
 from spinetoolbox.helpers import get_open_file_name_in_last_dir
 from spinetoolbox.config import APPLICATION_PATH
@@ -39,7 +39,7 @@ from .import_mapping_options import ImportMappingOptions
 from .import_mappings import ImportMappings
 from ..importer_specification import ImporterSpecification
 from ..mvcmodels.mappings_model import MappingsModel
-from ...widgets import UrlSelector
+from ...widgets import UrlSelectorDialog
 
 _CONNECTOR_NAME_TO_CLASS = {
     "CSVConnector": CSVConnector,
@@ -88,6 +88,7 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
         self.takeCentralWidget().deleteLater()
         self._filepath = filepath if filepath else self._FILE_LESS
         self._mappings_model = MappingsModel(self._undo_stack, self)
+        self._mappings_model.rowsInserted.connect(self._reselect_source_table)
         self._ui.source_list.setModel(self._mappings_model)
         self._ui.mapping_list.setModel(self._mappings_model)
         self._ui.mapping_list.setRootIndex(self._mappings_model.dummy_parent())
@@ -123,6 +124,11 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
     def settings_group(self):
         return "mappingPreviewWindow"
 
+    def _save(self):
+        """See base class."""
+        if super()._save():
+            self._import_mappings.specification_saved()
+
     @property
     def _duplicate_kwargs(self):
         return dict(filepath=self._filepath)
@@ -140,20 +146,24 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
             dock.setFloating(False)
             self.addDockWidget(Qt.RightDockWidgetArea, dock)
         docks = (self._ui.dockWidget_source_files, self._ui.dockWidget_mappings)
-        self.splitDockWidget(*docks, Qt.Horizontal)
+        self.splitDockWidget(*docks, Qt.Orientation.Horizontal)
         width = sum(d.size().width() for d in docks)
-        self.resizeDocks(docks, [0.9 * width, 0.1 * width], Qt.Horizontal)
+        self.resizeDocks(docks, [0.9 * width, 0.1 * width], Qt.Orientation.Horizontal)
         docks = (self._ui.dockWidget_source_files, self._ui.dockWidget_source_tables, self._ui.dockWidget_source_data)
-        self.splitDockWidget(*docks[:-1], Qt.Vertical)
-        self.splitDockWidget(*docks[1:], Qt.Vertical)
+        self.splitDockWidget(*docks[:-1], Qt.Orientation.Vertical)
+        self.splitDockWidget(*docks[1:], Qt.Orientation.Vertical)
         height = sum(d.size().height() for d in docks)
-        self.resizeDocks(docks, [0.1 * height, 0.2 * height, 0.7 * height], Qt.Vertical)
-        self.splitDockWidget(self._ui.dockWidget_source_tables, self._ui.dockWidget_source_options, Qt.Horizontal)
-        self.splitDockWidget(self._ui.dockWidget_mappings, self._ui.dockWidget_mapping_options, Qt.Vertical)
-        self.splitDockWidget(self._ui.dockWidget_mapping_options, self._ui.dockWidget_mapping_spec, Qt.Vertical)
+        self.resizeDocks(docks, [0.1 * height, 0.2 * height, 0.7 * height], Qt.Orientation.Vertical)
+        self.splitDockWidget(
+            self._ui.dockWidget_source_tables, self._ui.dockWidget_source_options, Qt.Orientation.Horizontal
+        )
+        self.splitDockWidget(self._ui.dockWidget_mappings, self._ui.dockWidget_mapping_options, Qt.Orientation.Vertical)
+        self.splitDockWidget(
+            self._ui.dockWidget_mapping_options, self._ui.dockWidget_mapping_spec, Qt.Orientation.Vertical
+        )
         docks = (self._ui.dockWidget_mapping_options, self._ui.dockWidget_mapping_spec)
         height = sum(d.size().height() for d in docks)
-        self.resizeDocks(docks, [0.1 * height, 0.9 * height], Qt.Vertical)
+        self.resizeDocks(docks, [0.1 * height, 0.9 * height], Qt.Orientation.Vertical)
         qApp.processEvents()  # pylint: disable=undefined-variable
         self.resize(size)
 
@@ -184,8 +194,8 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
         self._ui.comboBox_source_file.setCurrentText(filepath)
 
     def _get_source_url(self):
-        selector = UrlSelector(self._toolbox, parent=self)
-        selector.exec_()
+        selector = UrlSelectorDialog(self._toolbox.qsettigns(), self._toolbox, parent=self)
+        selector.exec()
         return selector.url
 
     def _get_source_file_path(self):
@@ -290,9 +300,9 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
                 row = k
         if row is not None:
             connector_list_wg.setCurrentRow(row)
-        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
-        button_box.button(QDialogButtonBox.Ok).clicked.connect(dialog.accept)
-        button_box.button(QDialogButtonBox.Cancel).clicked.connect(dialog.reject)
+        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
+        button_box.button(QDialogButtonBox.StandardButton.Ok).clicked.connect(dialog.accept)
+        button_box.button(QDialogButtonBox.StandardButton.Cancel).clicked.connect(dialog.reject)
         connector_list_wg.doubleClicked.connect(dialog.accept)
         dialog.layout().addWidget(connector_list_wg)
         dialog.layout().addWidget(button_box)
@@ -300,7 +310,7 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
         if not spec_name:
             spec_name = "unnamed specification"
         dialog.setWindowTitle(f"Select connector for {spec_name}")
-        answer = dialog.exec_()
+        answer = dialog.exec()
         if not answer:
             return None
         row = connector_list_wg.currentIndex().row()
@@ -345,6 +355,23 @@ class ImportEditorWindow(SpecificationEditorWindowBase):
             json.dump(mappings_dict, file_p)
         self._show_status_bar_msg(f"Mapping saved to: {filename[0]}")
 
+    @Slot(QModelIndex, int, int)
+    def _reselect_source_table(self, parent, first, last):
+        """Selects added source table.
+
+        This is a workaround to get the correct source table selected after a new one has been added
+        since the source table view doesn't seem to update the current index correctly in this case.
+
+        Args:
+            parent (QModelIndex): parent index
+            first (int): first new row
+            last (int): last new row
+        """
+        if parent.isValid():
+            return
+        index = self._mappings_model.index(last, 0)
+        self._ui.source_list.selectionModel().setCurrentIndex(index, QItemSelectionModel.ClearAndSelect)
+
     def tear_down(self):
         if not super().tear_down():
             return False
