@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -15,8 +15,7 @@ ImportMappingOptions widget.
 :author: P. Vennström (VTT)
 :date:   12.5.2020
 """
-from PySide2.QtCore import Qt, Slot, QModelIndex
-from PySide2.QtWidgets import QWidget
+from PySide6.QtCore import Qt, Slot, QModelIndex
 from .custom_menus import SimpleFilterMenu
 from ..commands import (
     SetImportObjectsFlag,
@@ -130,8 +129,7 @@ class ImportMappingOptions:
         Updates ui according to the current mapping type.
         """
         has_mapping = self._has_current_mappings()
-        for widget in self._ui.dockWidget_mapping_options.findChildren(QWidget):
-            widget.setEnabled(has_mapping)
+        self._ui.mapping_options_contents.setEnabled(has_mapping)
         if not has_mapping:
             return
 
@@ -158,7 +156,7 @@ class ImportMappingOptions:
         # update item mapping settings
         if flattened_mappings.may_import_objects():
             self._ui.import_objects_check_box.setEnabled(True)
-            check_state = Qt.Checked if flattened_mappings.import_objects() else Qt.Unchecked
+            check_state = Qt.CheckState.Checked if flattened_mappings.import_objects() else Qt.CheckState.Unchecked
             self._ui.import_objects_check_box.setCheckState(check_state)
         else:
             self._ui.import_objects_check_box.setEnabled(False)
@@ -292,13 +290,13 @@ class ImportMappingOptions:
             )
         )
 
-    @Slot(bool)
+    @Slot(int)
     def _change_use_before_alternative(self, state):
         """
         Pushes SetUseBeforeAlternative command to the undo stack.
 
         Args:
-            state (bool): new flag value
+            state (int): New state value
         """
         if self._block_signals or not self._has_current_mappings():
             return
@@ -308,24 +306,27 @@ class ImportMappingOptions:
                 self._list_index.parent().row(),
                 self._list_index.row(),
                 self._mappings_model,
-                state == Qt.Checked,
+                state == Qt.CheckState.Checked.value,
                 previous_mapping,
             )
         )
 
-    @Slot(bool)
+    @Slot(int)
     def _change_import_objects(self, state):
         """
         Pushes SetImportObjectsFlag command to the undo stack.
 
         Args:
-            state (bool): new flag value
+            state (int): New state value
         """
         if self._block_signals or not self._has_current_mappings():
             return
         self._undo_stack.push(
             SetImportObjectsFlag(
-                self._list_index.parent().row(), self._list_index.row(), self._mappings_model, state == Qt.Checked
+                self._list_index.parent().row(),
+                self._list_index.row(),
+                self._mappings_model,
+                state == Qt.CheckState.Checked.value,
             )
         )
 
@@ -368,19 +369,22 @@ class ImportMappingOptions:
             )
         )
 
-    @Slot(bool)
+    @Slot(int)
     def _change_time_series_repeat_flag(self, repeat):
         """
         Pushes :class:`SetTimeSeriesRepeatFlag` to the undo stack.
 
         Args:
-            repeat (bool): True if repeat is enable, False otherwise
+            repeat (int): True if repeat is enabled, False otherwise
         """
         if self._block_signals or not self._has_current_mappings():
             return
         self._undo_stack.push(
             SetTimeSeriesRepeatFlag(
-                self._list_index.parent().row(), self._list_index.row(), self._mappings_model, repeat
+                self._list_index.parent().row(),
+                self._list_index.row(),
+                self._mappings_model,
+                repeat == Qt.CheckState.Checked.value,
             )
         )
 
@@ -405,19 +409,22 @@ class ImportMappingOptions:
             )
         )
 
-    @Slot(bool)
+    @Slot(int)
     def _change_map_compression_flag(self, compress):
         """
         Pushes :class:`SetMapCompressFlag` to the undo stack.
 
         Args:
-            compress (CheckState): if ``Qt.Checked``, Maps will be compressed
+            compress (int): if ``Qt.CheckState.Checked.value``, Maps will be compressed
         """
         if self._block_signals or not self._has_current_mappings():
             return
         self._undo_stack.push(
             SetMapCompressFlag(
-                self._list_index.parent().row(), self._list_index.row(), self._mappings_model, compress == Qt.Checked
+                self._list_index.parent().row(),
+                self._list_index.row(),
+                self._mappings_model,
+                compress == Qt.CheckState.Checked.value,
             )
         )
 
@@ -433,7 +440,7 @@ class ImportMappingOptions:
         is_time_series = flattened_mappings.is_time_series_value()
         self._ui.time_series_repeat_check_box.setEnabled(is_time_series)
         self._ui.time_series_repeat_check_box.setCheckState(
-            Qt.Checked if is_time_series and value_mapping.options.get("repeat") else Qt.Unchecked
+            Qt.CheckState.Checked if is_time_series and value_mapping.options.get("repeat") else Qt.CheckState.Unchecked
         )
 
     def _update_map_options(self):
@@ -453,4 +460,4 @@ class ImportMappingOptions:
         self._ui.map_dimension_spin_box.setEnabled(is_map)
         self._ui.map_dimension_spin_box.setValue(dimension_count)
         self._ui.map_compression_check_box.setEnabled(is_map)
-        self._ui.map_compression_check_box.setChecked(Qt.Checked if is_map and value_mapping.compress else Qt.Unchecked)
+        self._ui.map_compression_check_box.setChecked(True if is_map and value_mapping.compress else False)
