@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,8 +16,8 @@ ImportMappings widget.
 :date:   1.6.2019
 """
 import pickle
-from PySide2.QtCore import QPoint, QItemSelectionModel, Slot, QModelIndex, QMimeData, QItemSelection
-from PySide2.QtWidgets import QStyledItemDelegate, QApplication
+from PySide6.QtCore import QPoint, QItemSelectionModel, Slot, QModelIndex, QMimeData, QItemSelection
+from PySide6.QtWidgets import QStyledItemDelegate, QApplication
 
 from spinedb_api.import_mapping.import_mapping_compat import unparse_named_mapping_spec
 from spinetoolbox.widgets.custom_delegates import ComboBoxDelegate
@@ -71,6 +71,10 @@ class ImportMappings:
         self._ui.duplicate_button.clicked.connect(self._duplicate_selected_mapping)
         self._ui.mapping_list.customContextMenuRequested.connect(self._show_mapping_list_context_menu)
 
+    def specification_saved(self):
+        """Notifies mappings model items that they have been stored into the specification."""
+        self._mappings_model.set_source_table_items_into_specification()
+
     @Slot(QModelIndex, QModelIndex)
     def _change_list(self, current, previous):
         """Loads current source table's mapping list.
