@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,8 +16,8 @@ Contains OptionsWidget class.
 :date:   1.6.2019
 """
 import functools
-from PySide2.QtCore import Qt, Signal, Slot
-from PySide2.QtWidgets import (
+from PySide6.QtCore import Qt, Signal, Slot
+from PySide6.QtWidgets import (
     QLabel,
     QLineEdit,
     QComboBox,
@@ -251,12 +251,13 @@ def _emit_check_box_option_changed(state, option_key, options_widget):
         option_key (str): option's key
         options_widget (OptionsWidget): options widget
     """
+    checked = state == Qt.CheckState.Checked.value
     if options_widget.undo_enabled:
         previous_value = options_widget.connector.get_current_option_value(option_key)
         options_widget.undo_stack.push(
-            SetConnectorOption(options_widget.current_source_table, option_key, options_widget, state, previous_value)
+            SetConnectorOption(options_widget.current_source_table, option_key, options_widget, checked, previous_value)
         )
-    options = {option_key: state == Qt.Checked}
+    options = {option_key: checked}
     options_widget.options_changed.emit(options)
 
 
