@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -10,17 +10,16 @@
 ######################################################################################################################
 
 """
-Classes for handling models in PySide2's model/view framework.
+Classes for handling models in PySide6's model/view framework.
 
 :author: P. Vennström (VTT)
 :date:   11.5.2020
 """
 from collections import namedtuple
 from collections.abc import Iterable
-from PySide2.QtCore import QPoint, Qt, Signal, Slot, QModelIndex
-from PySide2.QtGui import QCursor, QFont, QIcon
-from PySide2.QtWidgets import (
-    QAction,
+from PySide6.QtCore import QPoint, Qt, Signal, Slot, QModelIndex
+from PySide6.QtGui import QCursor, QFont, QIcon, QAction
+from PySide6.QtWidgets import (
     QHeaderView,
     QMenu,
     QTableView,
@@ -37,20 +36,28 @@ from PySide2.QtWidgets import (
 from spinedb_api import DateTime, Duration, ParameterValueFormatError
 from spinetoolbox.helpers import CharIconEngine
 from spinedb_api.spine_io.importers.reader import TYPE_STRING_TO_CLASS
-from spinedb_api.import_mapping.type_conversion import IntegerSequenceDateTimeConvertSpec, value_to_convert_spec
+from spinedb_api.import_mapping.type_conversion import (
+    IntegerSequenceDateTimeConvertSpec,
+    value_to_convert_spec,
+    BooleanConvertSpec,
+    StringConvertSpec,
+    DateTimeConvertSpec,
+    DurationConvertSpec,
+    FloatConvertSpec,
+)
 from ..commands import SetColumnOrRowType
 from ..mvcmodels.source_data_table_model import SourceDataTableModel
 
 _ALLOWED_TYPES = list(sorted(TYPE_STRING_TO_CLASS.keys()))
 _ALLOWED_TYPES.append("integer sequence datetime")
 
-_TYPE_TO_FONT_AWESOME_ICON = {
-    "integer sequence datetime": chr(int('f073', 16)),
-    "boolean": chr(int('f6ad', 16)),
-    "string": chr(int('f031', 16)),
-    "datetime": chr(int('f073', 16)),
-    "duration": chr(int('f017', 16)),
-    "float": chr(int('f534', 16)),
+TYPE_TO_FONT_AWESOME_ICON = {
+    IntegerSequenceDateTimeConvertSpec.DISPLAY_NAME: chr(int('f073', 16)),
+    BooleanConvertSpec.DISPLAY_NAME: chr(int('f6ad', 16)),
+    StringConvertSpec.DISPLAY_NAME: chr(int('f031', 16)),
+    DateTimeConvertSpec.DISPLAY_NAME: chr(int('f073', 16)),
+    DurationConvertSpec.DISPLAY_NAME: chr(int('f017', 16)),
+    FloatConvertSpec.DISPLAY_NAME: chr(int('f534', 16)),
 }
 
 Margin = namedtuple("Margin", ("left", "right", "top", "bottom"))
@@ -64,10 +71,10 @@ class IntegerSequenceDateTimeConvertSpecDialog(QDialog):
             parent (QWidget, optional): parent widget
         """
         super().__init__(parent)
-        self.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
+        self.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self.setWindowTitle("New integer sequence datetime")
 
-        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        self.buttonBox = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
 
@@ -96,9 +103,9 @@ class IntegerSequenceDateTimeConvertSpecDialog(QDialog):
         try:
             Duration(self.duration.text())
         except ParameterValueFormatError:
-            self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+            self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False)
             return
-        self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(True)
 
     def get_spec(self):
         start_datetime = DateTime(self.datetime.dateTime().toString(Qt.ISODate))
@@ -116,8 +123,8 @@ class TableViewWithButtonHeader(QTableView):
             parent (QWidget): a parent widget
         """
         super().__init__(parent)
-        self._horizontal_header = HeaderWithButton(Qt.Horizontal, self)
-        self._vertical_header = HeaderWithButton(Qt.Vertical, self)
+        self._horizontal_header = HeaderWithButton(Qt.Orientation.Horizontal, self)
+        self._vertical_header = HeaderWithButton(Qt.Orientation.Vertical, self)
         self.setHorizontalHeader(self._horizontal_header)
         self._horizontal_header.setContextMenuPolicy(Qt.CustomContextMenu)
         self._horizontal_header.customContextMenuRequested.connect(self._show_horizontal_header_menu)
@@ -158,14 +165,14 @@ class TableViewWithButtonHeader(QTableView):
         """Opens the context menu of the horizontal header."""
         if self._horizontal_header.display_all or self._horizontal_header.sections_with_buttons:
             screen_pos = self._horizontal_header.mapToGlobal(pos)
-            self._horizontal_menu.exec_(screen_pos)
+            self._horizontal_menu.exec(screen_pos)
 
     @Slot(QPoint)
     def _show_vertical_header_menu(self, pos):
         """Opens the context menu of the vertical header."""
         if self._vertical_header.display_all or self._vertical_header.sections_with_buttons:
             screen_pos = self._vertical_header.mapToGlobal(pos)
-            self._vertical_menu.exec_(screen_pos)
+            self._vertical_menu.exec(screen_pos)
 
     @Slot(QAction)
     def _set_all_column_data_types(self, action):
@@ -198,7 +205,7 @@ class TableViewWithButtonHeader(QTableView):
 def _make_type_button(parent, menu, font):
     button = QToolButton(parent=parent)
     button.setMenu(menu)
-    button.setPopupMode(QToolButton.InstantPopup)
+    button.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
     button.setFont(font)
     button.setCursor(Qt.ArrowCursor)
     return button
@@ -213,6 +220,11 @@ class HeaderWithButton(QHeaderView):
     """Emitted when an undo/redo command is going to be executed."""
 
     def __init__(self, orientation, parent=None):
+        """
+        Args:
+            orientation (Orientation): header orientation
+            parent (QWidget, optional): parent widget
+        """
         super().__init__(orientation, parent)
         self._source_table_name = None
         self._undo_stack = None
@@ -281,7 +293,7 @@ class HeaderWithButton(QHeaderView):
                 if isinstance(section_type, IntegerSequenceDateTimeConvertSpec):
                     existing_spec = section_type
             dialog = IntegerSequenceDateTimeConvertSpecDialog(existing_spec, self)
-            if not dialog.exec_():
+            if not dialog.exec():
                 return
             convert_spec = dialog.get_spec()
         else:
@@ -316,7 +328,7 @@ class HeaderWithButton(QHeaderView):
         Returns:
             int: Width of widget
         """
-        if self.orientation() == Qt.Horizontal:
+        if self.orientation() == Qt.Orientation.Horizontal:
             return self.height()
         return self.sectionSize(0)
 
@@ -326,7 +338,7 @@ class HeaderWithButton(QHeaderView):
         Returns:
             int: Height of widget
         """
-        if self.orientation() == Qt.Horizontal:
+        if self.orientation() == Qt.Orientation.Horizontal:
             return self.height()
         return self.sectionSize(0)
 
@@ -343,8 +355,7 @@ class HeaderWithButton(QHeaderView):
             self._button.hide()
 
     def mouseMoveEvent(self, mouse_event):
-        """Moves the button to the correct section so that interacting with the button works.
-        """
+        """Moves the button to the correct section so that interacting with the button works."""
         logical_index = self.logicalIndexAt(mouse_event.x(), mouse_event.y())
         self._hide_or_show_button(logical_index)
         super().mouseMoveEvent(mouse_event)
@@ -369,7 +380,7 @@ class HeaderWithButton(QHeaderView):
             index (int): logical_index to set position and geometry to.
         """
         margin = self._margin
-        if self.orientation() == Qt.Horizontal:
+        if self.orientation() == Qt.Orientation.Horizontal:
             button.setGeometry(
                 self.sectionViewportPosition(index) + margin.left,
                 margin.top,
@@ -409,10 +420,10 @@ class HeaderWithButton(QHeaderView):
         # get the type of the section.
         type_spec = self.model().get_type(logical_index, self.orientation())
         if type_spec is None:
-            type_spec = "string"
+            type_spec = StringConvertSpec.DISPLAY_NAME
         else:
             type_spec = type_spec.DISPLAY_NAME
-        font_str = _TYPE_TO_FONT_AWESOME_ICON[type_spec]
+        font_str = TYPE_TO_FONT_AWESOME_ICON[type_spec]
 
         # set data for both interaction button and render button.
         self._button.setText(font_str)
@@ -421,7 +432,7 @@ class HeaderWithButton(QHeaderView):
 
         # get pixmap from render button and draw into header section.
         rw = self._render_button.grab()
-        if self.orientation() == Qt.Horizontal:
+        if self.orientation() == Qt.Orientation.Horizontal:
             painter.drawPixmap(self.sectionViewportPosition(logical_index), 0, rw)
         else:
             painter.drawPixmap(0, self.sectionViewportPosition(logical_index), rw)
@@ -458,8 +469,7 @@ class HeaderWithButton(QHeaderView):
         self._set_button_geometry(self._button, logical_index)
 
     def fix_widget_positions(self):
-        """Update position of interaction button
-        """
+        """Update position of interaction button"""
         mouse_position = self.mapFromGlobal(QCursor.pos())
         logical_index = self.logicalIndexAt(mouse_position)
         self._set_button_geometry(self._button, logical_index)
@@ -477,7 +487,7 @@ class HeaderWithButton(QHeaderView):
         """
         if isinstance(model, SourceDataTableModel):
             old_model = self.model()
-            if self.orientation() == Qt.Horizontal:
+            if self.orientation() == Qt.Orientation.Horizontal:
                 model.column_types_updated.connect(self.update_buttons)
                 if isinstance(old_model, SourceDataTableModel):
                     old_model.column_types_updated.disconnect(self.update_buttons)
@@ -520,7 +530,7 @@ def _create_allowed_types_menu(parent, trigger_slot):
     """
     menu = QMenu(parent)
     for at in _ALLOWED_TYPES:
-        icon_char = _TYPE_TO_FONT_AWESOME_ICON[at]
+        icon_char = TYPE_TO_FONT_AWESOME_ICON[at]
         engine = CharIconEngine(icon_char, 0)
         icon = QIcon(engine.pixmap())
         menu.addAction(icon, at)
