@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -28,10 +28,9 @@ import time
 import uuid
 from contextlib import ExitStack
 from spine_engine.config import TOOL_OUTPUT_DIR
-from spine_engine.project_item.executable_item_base import ExecutableItemBase
 from spine_engine.spine_engine import ItemExecutionFinishState
 from spine_engine.project_item.project_item_resource import (
-    cmd_line_arg_from_dict,
+    make_cmd_line_arg,
     expand_cmd_line_args,
     labelled_resource_args,
 )
@@ -40,14 +39,26 @@ from .item_info import ItemInfo
 from .utils import file_paths_from_resources, find_file, flatten_file_path_duplicates, is_pattern, make_dir_if_necessary
 from .output_resources import scan_for_resources
 from ..utils import generate_filter_subdirectory_name
+from ..db_writer_executable_item_base import DBWriterExecutableItemBase
 
 
-class ExecutableItem(ExecutableItemBase):
+class ExecutableItem(DBWriterExecutableItemBase):
     """Tool project item's executable parts."""
 
     _MAX_RETRIES = 3
 
-    def __init__(self, name, work_dir, tool_specification, cmd_line_args, options, group_id, project_dir, logger):
+    def __init__(
+        self,
+        name,
+        work_dir,
+        tool_specification,
+        cmd_line_args,
+        options,
+        kill_completed_processes,
+        group_id,
+        project_dir,
+        logger,
+    ):
         """
         Args:
             name (str): item's name
@@ -56,6 +67,7 @@ class ExecutableItem(ExecutableItemBase):
             tool_specification (ToolSpecification): a tool specification
             cmd_line_args (list): a list of command line argument to pass to the tool instance
             options (dict): misc tool options. See ``Tool`` for details.
+            kill_completed_processes (bool): whether to kill completed persistent processes after execution
             group_id (str or None): execution group identifier
             project_dir (str): absolute path to project directory
             logger (LoggerInterface): a logger
@@ -66,6 +78,7 @@ class ExecutableItem(ExecutableItemBase):
         self._tool_specification = tool_specification
         self._cmd_line_args = cmd_line_args
         self._options = options
+        self._kill_completed_processes = kill_completed_processes
         self._tool_instance = None
         self._retry_count = 0
 
@@ -73,6 +86,10 @@ class ExecutableItem(ExecutableItemBase):
     def options(self):
         return self._options
 
+    @property
+    def cmd_line_args(self):
+        return self._cmd_line_args
+
     @staticmethod
     def item_type():
         """Returns the item's type identifier string."""
@@ -86,9 +103,9 @@ class ExecutableItem(ExecutableItemBase):
             self._tool_instance = None
 
     def _copy_input_files(self, paths, execution_dir):
-        """
-        Copies input files from given paths to work or source directory, depending on
-        where the Tool specification requires them to be.
+        """Copies input files from given paths to work or source
+        directory, depending on where the Tool specification
+        requires them to be.
 
         Args:
             paths (dict): key is path to destination file, value is path to source file
@@ -156,9 +173,9 @@ class ExecutableItem(ExecutableItemBase):
         return True
 
     def _copy_optional_input_files(self, paths, execution_dir):
-        """
-        Copies optional input files from given paths to work or source directory, depending on
-        where the Tool specification requires them to be.
+        """Copies optional input files from given paths to work or source
+        directory, depending on where the Tool specification requires
+        them to be.
 
         Args:
             paths (dict): key is the source path, value is the destination path
@@ -372,15 +389,13 @@ class ExecutableItem(ExecutableItemBase):
                 return False
             if use_julia_kernel == "0" and julia_path == "":
                 self._logger.msg_error.emit(
-                    "Julia not found in PATH. Please select the Julia you want to use in Settings->Tools."
+                    "Julia not found in PATH. Please select a Julia executable in Settings->Tools."
                 )
                 return False
         elif self._tool_specification.tooltype.lower() == "gams":
             gams_path = resolve_gams_executable(settings.value("appSettings/gamsPath", defaultValue=""))
             if not gams_path:
-                self._logger.msg_error.emit(
-                    "Gams not found in PATH. Please select the Gams you want to use in Settings->Tools."
-                )
+                self._logger.msg_error.emit("Gams not found in PATH. Please set a path to Gams in Settings->Tools.")
                 return False
         elif self._tool_specification.tooltype.lower() == "executable":
             if not self._tool_specification.main_prgm:
@@ -396,7 +411,7 @@ class ExecutableItem(ExecutableItemBase):
                     return False
         return True
 
-    def execute(self, forward_resources, backward_resources):
+    def execute(self, forward_resources, backward_resources, lock):
         """See base class.
 
         Before launching the tool script in a separate instance,
@@ -404,7 +419,7 @@ class ExecutableItem(ExecutableItemBase):
         and copying input files where needed.
         After execution archives the output files.
         """
-        if not super().execute(forward_resources, backward_resources):
+        if not super().execute(forward_resources, backward_resources, lock):
             return ItemExecutionFinishState.FAILURE
         execution_dir = _execution_directory(self._work_dir, self._tool_specification)
         if execution_dir is None:
@@ -419,7 +434,7 @@ class ExecutableItem(ExecutableItemBase):
             f"*** Executing Tool specification <b>{self._tool_specification.name}</b> in {anchor} ***"
         )
         if work_or_source == "work":
-            self._logger.msg.emit(f"*** Copying program files ***")
+            self._logger.msg.emit("*** Copying program files ***")
             if not self._copy_program_files(execution_dir):
                 self._logger.msg_error.emit("Copying program files failed")
                 return ItemExecutionFinishState.FAILURE
@@ -459,18 +474,27 @@ class ExecutableItem(ExecutableItemBase):
             self._logger.msg_error.emit("Creating output subdirectories failed. Tool execution aborted.")
             return ItemExecutionFinishState.FAILURE
         if not os.path.isdir(execution_dir):
-            self._logger.msg_warning.emit(
-                f"Work directory was not created because Tool Specification "
-                f"<b>{self._tool_specification.name}</b> does not contain any "
-                f"program files nor (optional) input files. Please <b>execute the "
-                f"Tool in source execution mode</b> or include files to the "
-                f"specification."
-            )
-            return ItemExecutionFinishState.FAILURE
-        self._tool_instance = self._tool_specification.create_tool_instance(execution_dir, self._logger, self)
+            if self._tool_specification.tooltype.lower() == "executable":
+                try:
+                    os.makedirs(execution_dir, exist_ok=True)
+                except OSError:
+                    self._logger.msg_error.emit(
+                        f"Creating work directory {execution_dir} failed. Try execution "
+                        f"in source directory instead."
+                    )
+                    return ItemExecutionFinishState.FAILURE
+            else:
+                self._logger.msg_warning.emit(
+                    f"Project is not self-contained. Please make sure all Tool specification "
+                    f"files are in the project directory."
+                )
+                return ItemExecutionFinishState.FAILURE
+        self._tool_instance = self._tool_specification.create_tool_instance(
+            execution_dir, self._kill_completed_processes, self._logger, self
+        )
         resources = forward_resources + backward_resources
         with ExitStack() as stack:
-            labelled_args = labelled_resource_args(resources, stack)
+            labelled_args = labelled_resource_args(resources, stack, db_checkin=True, db_checkout=True)
             expanded_args = expand_cmd_line_args(self._cmd_line_args, labelled_args, self._logger)
             try:
                 self._tool_instance.prepare(expanded_args)
@@ -481,21 +505,28 @@ class ExecutableItem(ExecutableItemBase):
             return_code = self._tool_instance.execute()
             if return_code != 0 and self._tool_instance is not None and self._tool_instance.killed:
                 # NOTE: return_code will be 0 if the instance was killed by e.g. `exit(0)` in julia
-                # In this case we want to consider the tool successfull and not retry it
+                # In this case we want to consider the tool successful and not retry it
                 if self._retry_count < self._MAX_RETRIES:
                     # Try again
                     self._retry_count += 1
                     self._logger.msg_warning.emit("The Tool process was found dead. Retrying...")
-                    return self.execute(forward_resources, backward_resources)
+                    return self.execute(forward_resources, backward_resources, lock)
                 self._logger.msg_warning.emit("Maximum amount of retries reached.")
         self._handle_output_files(return_code, self._filter_id, forward_resources, execution_dir)
         self._tool_instance = None
         # TODO: Check what return code is 'stopped' and return ItemExecutionFinishState.STOPPED in this case
         return ItemExecutionFinishState.SUCCESS if return_code == 0 else ItemExecutionFinishState.FAILURE
 
+    def exclude_execution(self, forward_resources, backward_resources, lock):
+        """See base class."""
+        super().exclude_execution(forward_resources, backward_resources, lock)
+        if self._filter_id:
+            self._output_dir = os.path.join(
+                self._output_dir, generate_filter_subdirectory_name(forward_resources, self.hash_filter_id())
+            )
+
     def _find_input_files(self, resources):
-        """
-        Iterates required input  files in tool specification and looks for them in the given resources.
+        """Iterates required input  files in tool specification and looks for them in the given resources.
 
         Args:
             resources (list): resources available
@@ -513,8 +544,7 @@ class ExecutableItem(ExecutableItemBase):
         return file_paths
 
     def _find_optional_input_files(self, resources):
-        """
-        Tries to find optional input files from previous project items in the DAG.
+        """Tries to find optional input files from previous project items in the DAG.
 
         Args:
             resources (list): resources available
@@ -603,8 +633,7 @@ class ExecutableItem(ExecutableItemBase):
             self._logger.msg_warning.emit(f"\tNo output files defined for this Tool specification. {tip_anchor}")
 
     def _optional_output_destination_paths(self, paths, execution_dir):
-        """
-        Returns a dictionary telling where optional output files should be copied to before execution.
+        """Returns a dictionary telling where optional output files should be copied to before execution.
 
         Args:
             paths (dict): key is the optional file name pattern, value is a list of paths to source files
@@ -663,24 +692,34 @@ class ExecutableItem(ExecutableItemBase):
         else:
             work_dir = None
         specification_name = item_dict["specification"]
-        specification = ExecutableItemBase._get_specification(
+        specification = DBWriterExecutableItemBase._get_specification(
             name, ItemInfo.item_type(), specification_name, specifications, logger
         )
-        cmd_line_args = [cmd_line_arg_from_dict(arg) for arg in item_dict["cmd_line_args"]]
+        cmd_line_args = [make_cmd_line_arg(arg) for arg in item_dict["cmd_line_args"]]
         options = item_dict.get("options", {})
+        kill_completed_processes = item_dict.get("kill_completed_processes", False)
         group_id = item_dict.get("group_id")
-        return cls(name, work_dir, specification, cmd_line_args, options, group_id, project_dir, logger)
+        return cls(
+            name,
+            work_dir,
+            specification,
+            cmd_line_args,
+            options,
+            kill_completed_processes,
+            group_id,
+            project_dir,
+            logger,
+        )
 
 
 def _count_files_and_dirs(paths):
-    """
-    Counts the number of files and directories in given paths.
+    """Counts the number of files and directories in given paths.
 
     Args:
         paths (list): list of paths
 
     Returns:
-        Tuple containing the number of required files and directories.
+        tuple: Tuple containing the number of required files and directories.
     """
     n_dir = 0
     n_file = 0
@@ -710,16 +749,27 @@ def _create_output_dir_timestamp():
 
 
 def _execution_directory(work_dir, tool_specification):
-    """
-    Returns the path to the execution directory, depending on ``execute_in_work``.
+    """Returns the path to the execution directory.
 
-    If ``execute_in_work`` is ``True``, a new unique path will be returned.
-    If a main program file does not exist, Tool spec definition file path is
-    returned. Otherwise, the main program file path from tool specification
-    is returned.
+    The returned path is a new unique path when the user
+    has selected to execute the Tool in work directory.
+
+    The returned path is the directory where the Tool spec's
+    main program file is located when the user has selected
+    to execute the Tool in source directory.
+
+    The returned path is the directory where the Tool spec's
+    definition file is located when the user has selected
+    to execute the Tool in source directory and the Tool
+    spec has no main program file (it is an Executable Tool
+    spec).
+
+    Args:
+        work_dir (str): Path to work dir or None
+        tool_specification (ToolSpecification): Tool spec instance
 
     Returns:
-        str: a full path to next basedir
+        str: Full path to execution dir
     """
     if work_dir is not None:
         basedir = os.path.join(work_dir, _unique_dir_name(tool_specification))
@@ -730,12 +780,12 @@ def _execution_directory(work_dir, tool_specification):
 
 
 def _find_files_in_pattern(pattern, available_file_paths):
-    """
-    Returns a list of files that match the given pattern.
+    """Returns a list of files that match the given pattern.
 
     Args:
-        pattern (str): file pattern
-        available_file_paths (list): list of available file paths from upstream items
+        pattern (str): File pattern
+        available_file_paths (list): List of available file paths from upstream items
+
     Returns:
         list: List of (full) paths
     """
