@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,15 +16,15 @@ Tool class.
 :date:   19.12.2017
 """
 import os
-from PySide2.QtCore import Slot, QItemSelection, Qt
-from PySide2.QtWidgets import QAction
-from spinetoolbox.project_item.project_item import ProjectItem
+from PySide6.QtCore import Slot, QItemSelection, Qt
+from PySide6.QtGui import QAction
 from spinetoolbox.helpers import open_url
 from spinetoolbox.mvcmodels.file_list_models import FileListModel
 from spine_engine.config import TOOL_OUTPUT_DIR
-from spine_engine.project_item.project_item_resource import CmdLineArg, cmd_line_arg_from_dict, LabelArg
+from spine_engine.project_item.project_item_resource import CmdLineArg, make_cmd_line_arg, LabelArg
 from spine_engine.utils.helpers import resolve_python_interpreter
-from .commands import UpdateToolExecuteInWorkCommand, UpdateToolOptionsCommand
+from .commands import UpdateKillCompletedProcesses, UpdateToolExecuteInWorkCommand, UpdateToolOptionsCommand
+from ..db_writer_item_base import DBWriterItemBase
 from ..commands import UpdateCmdLineArgsCommand, UpdateGroupIdCommand
 from .item_info import ItemInfo
 from .widgets.custom_menus import ToolSpecificationMenu
@@ -35,7 +35,7 @@ from ..models import ToolCommandLineArgsModel
 from .output_resources import scan_for_resources
 
 
-class Tool(ProjectItem):
+class Tool(DBWriterItemBase):
     def __init__(
         self,
         name,
@@ -48,6 +48,7 @@ class Tool(ProjectItem):
         execute_in_work=True,
         cmd_line_args=None,
         options=None,
+        kill_completed_processes=False,
         group_id=None,
     ):
         """Tool class.
@@ -63,6 +64,8 @@ class Tool(ProjectItem):
             execute_in_work (bool): Execute associated Tool specification in work (True) or source directory (False)
             cmd_line_args (list, optional): Tool command line arguments
             options (dict, optional): misc tool options. At the moment it just holds the location of the julia sysimage
+            kill_completed_processes (bool): whether to kill completed persistent processes
+            group_id (str, optional): execution group id
         """
         super().__init__(name, description, x, y, project)
         self._toolbox = toolbox
@@ -85,6 +88,7 @@ class Tool(ProjectItem):
         self.output_dir = os.path.join(self.data_dir, TOOL_OUTPUT_DIR)
         self._specification_menu = None
         self._options = options if options is not None else {}
+        self._kill_completed_processes = kill_completed_processes
         self._resources_from_upstream = list()
         self._resources_from_downstream = list()
 
@@ -144,6 +148,7 @@ class Tool(ProjectItem):
             self._properties_ui.treeView_cmdline_args.selectionModel().selectionChanged
         ] = self._update_remove_args_button_enabled
         s[self._properties_ui.lineEdit_group_id.editingFinished] = self._set_group_id
+        s[self._properties_ui.kill_consoles_check_box.clicked] = self._set_kill_completed_processes
         return s
 
     @Slot()
@@ -178,6 +183,11 @@ class Tool(ProjectItem):
         enabled = self._properties_ui.treeView_cmdline_args.selectionModel().hasSelection()
         self._properties_ui.toolButton_remove_arg.setEnabled(enabled)
 
+    def _update_kill_consoles_check_box_enabled(self):
+        """Updates the enabled state of 'Kill consoles after execution' check box."""
+        enabled = self._specification is None or self._specification.tooltype in ("python", "julia")
+        self._properties_ui.kill_consoles_check_box.setEnabled(enabled)
+
     def restore_selections(self):
         """Restore selections into shared widgets when this project item is selected."""
         self._properties_ui.treeView_input_files.setModel(self._input_file_model)
@@ -209,7 +219,7 @@ class Tool(ProjectItem):
         self.update_execute_in_work_button()
 
     def update_execute_in_work_button(self):
-        """Sets the execute in work radio button check state according to
+        """Sets execute in work radio button check state according to
         execute_in_work instance variable."""
         if not self._active:
             return
@@ -270,7 +280,6 @@ class Tool(ProjectItem):
         if self._specification is None:
             return None
         undo_spec = self._specification.clone()
-        undo_spec.execute_in_work = self.execute_in_work
         return undo_spec
 
     def do_set_specification(self, specification):
@@ -280,14 +289,12 @@ class Tool(ProjectItem):
         self._populate_cmdline_args_model()
         if self._active:
             self._update_tool_ui()
-        if specification:
-            self.do_update_execution_mode(specification.execute_in_work)
         self._resources_to_successors_changed()
         self._check_notifications()
         return True
 
     def update_options(self, options):
-        """Pushes a new UpdateToolOptionsCommand to the toolbox stack."""
+        """Pushes a new UpdateToolOptionsCommand to the toolbox undo stack."""
         self._toolbox.undo_stack.push(UpdateToolOptionsCommand(self, options))
 
     def do_set_options(self, options):
@@ -300,6 +307,25 @@ class Tool(ProjectItem):
         if self._active:
             _ = self._get_options_widget()
 
+    @Slot(bool)
+    def _set_kill_completed_processes(self, kill_completed_processes):
+        """Pushes a UpdateKillCompletedProcesses to the undo stack
+
+        Args:
+            kill_completed_processes (bool): whether to kill completed persistent processes after execution
+        """
+        self._toolbox.undo_stack.push(UpdateKillCompletedProcesses(self, kill_completed_processes))
+
+    def do_update_kill_completed_processes(self, kill_completed_processes):
+        """Pushes a UpdateKillCompletedProcesses to the undo stack
+
+        Args:
+            kill_completed_processes (bool): whether to kill completed persistent processes after execution
+        """
+        self._kill_completed_processes = kill_completed_processes
+        if self._active:
+            self._properties_ui.kill_consoles_check_box.setChecked(kill_completed_processes)
+
     def _update_tool_ui(self):
         """Updates Tool properties UI. Used when Tool specification is changed.."""
         options_widget = self._properties_ui.horizontalLayout_options.takeAt(0)
@@ -330,6 +356,8 @@ class Tool(ProjectItem):
             else:
                 env = "" if not env else f"[{env}]"
                 self._properties_ui.label_jupyter.setText(f"[Jupyter Console] {k_spec_name} {env}")
+        self._properties_ui.kill_consoles_check_box.setChecked(self._kill_completed_processes)
+        self._update_kill_consoles_check_box_enabled()
 
     def _update_specification_menu(self):
         spec_model_index = self._toolbox.specification_model.specification_index(self.specification().name)
@@ -410,6 +438,7 @@ class Tool(ProjectItem):
 
     def handle_execution_successful(self, execution_direction, engine_state):
         """See base class."""
+        super().handle_execution_successful(execution_direction, engine_state)
         if execution_direction != "FORWARD":
             return
         self._resources_to_successors_changed()
@@ -486,6 +515,7 @@ class Tool(ProjectItem):
         d["cmd_line_args"] = [arg.to_dict() for arg in self.cmd_line_args]
         if self._options:
             d["options"] = self._options
+        d["kill_completed_processes"] = self._kill_completed_processes
         if self._group_id:
             d["group_id"] = self._group_id
         return d
@@ -493,12 +523,13 @@ class Tool(ProjectItem):
     @staticmethod
     def from_dict(name, item_dict, toolbox, project):
         """See base class."""
-        description, x, y = ProjectItem.parse_item_dict(item_dict)
+        description, x, y = DBWriterItemBase.parse_item_dict(item_dict)
         specification_name = item_dict.get("specification", "")
         execute_in_work = item_dict.get("execute_in_work", True)
         cmd_line_args = item_dict.get("cmd_line_args", [])
-        cmd_line_args = [cmd_line_arg_from_dict(arg) for arg in cmd_line_args]
+        cmd_line_args = [make_cmd_line_arg(arg) for arg in cmd_line_args]
         options = item_dict.get("options", {})
+        kill_completed_processes = item_dict.get("kill_completed_processes", False)
         group_id = item_dict.get("group_id")
         return Tool(
             name,
@@ -511,6 +542,7 @@ class Tool(ProjectItem):
             execute_in_work,
             cmd_line_args,
             options,
+            kill_completed_processes,
             group_id,
         )
 
@@ -533,18 +565,13 @@ class Tool(ProjectItem):
                 f"Link established. Tool <b>{self.name}</b> will look for input "
                 f"files from <b>{source_item.name}</b>'s references and data directory."
             )
-        elif source_item.item_type() in ("GdxExporter", "Exporter"):
+        elif source_item.item_type() == "Exporter":
             self._logger.msg.emit(
                 f"Link established. The file exported by <b>{source_item.name}</b> are now "
                 f"available in <b>{self.name}</b>."
             )
         elif source_item.item_type() in ["Data Transformer", "Tool"]:
             self._logger.msg.emit("Link established")
-        elif source_item.item_type() == "Gimlet":
-            self._logger.msg.emit(
-                f"Link established. Tool <b>{self.name}</b> will look for input "
-                f"files from <b>{source_item.name}</b>."
-            )
         else:
             super().notify_destination(source_item)
 
