@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -20,7 +20,6 @@ from collections import OrderedDict
 import copy
 import logging
 import os.path
-import json
 from spine_engine.project_item.project_item_specification import ProjectItemSpecification
 from spine_engine.utils.command_line_arguments import split_cmdline_args
 from .item_info import ItemInfo
@@ -39,7 +38,6 @@ OPTIONAL_KEYS = [
     "inputfiles_opt",
     "outputfiles",
     "cmdline_args",
-    "execute_in_work",
     "execution_settings",
 ]
 LIST_REQUIRED_KEYS = ["includes", "inputfiles", "inputfiles_opt", "outputfiles"]  # These should be lists
@@ -105,7 +103,6 @@ class ToolSpecification(ProjectItemSpecification):
         inputfiles_opt=None,
         outputfiles=None,
         cmdline_args=None,
-        execute_in_work=True,
     ):
         """
 
@@ -121,7 +118,6 @@ class ToolSpecification(ProjectItemSpecification):
             inputfiles_opt (list, optional): List of optional data files (wildcards may be used)
             outputfiles (list, optional): List of output files (wildcards may be used)
             cmdline_args (str, optional): Tool command line arguments (read from tool definition file)
-            execute_in_work (bool): Execute in work folder
         """
         super().__init__(name, description, item_type=ItemInfo.item_type(), item_category=ItemInfo.item_category())
         self._settings = settings
@@ -141,7 +137,6 @@ class ToolSpecification(ProjectItemSpecification):
         self.inputfiles_opt = set(inputfiles_opt) if inputfiles_opt else set()
         self.outputfiles = set(outputfiles) if outputfiles else set()
         self.return_codes = {}
-        self.execute_in_work = execute_in_work
 
     def _includes_main_path_relative(self):
         return os.path.relpath(self.path, os.path.dirname(self.definition_file_path)).replace(os.path.sep, "/")
@@ -157,11 +152,10 @@ class ToolSpecification(ProjectItemSpecification):
             "tooltype": self.tooltype,
             "includes": [src_file.replace(os.path.sep, "/") for src_file in self.includes],
             "description": self.description,
-            "inputfiles": list(self.inputfiles),
-            "inputfiles_opt": list(self.inputfiles_opt),
-            "outputfiles": list(self.outputfiles),
+            "inputfiles": sorted(self.inputfiles),
+            "inputfiles_opt": sorted(self.inputfiles_opt),
+            "outputfiles": sorted(self.outputfiles),
             "cmdline_args": self.cmdline_args,
-            "execute_in_work": self.execute_in_work,
             "includes_main_path": self._includes_main_path_relative(),
         }
 
@@ -223,11 +217,12 @@ class ToolSpecification(ProjectItemSpecification):
                     pass
         return kwargs
 
-    def create_tool_instance(self, basedir, logger, owner):
+    def create_tool_instance(self, basedir, kill_completed_processes, logger, owner):
         """Returns an instance of this tool specification that is configured to run in the given directory.
 
         Args:
             basedir (str): Path to the directory where the instance will run
+            kill_completed_processes (bool): whether to kill completed persistent processes
             logger (LoggerInterface): Logger
             owner (ExecutableItemBase): Project item that owns the instance
         """
@@ -250,7 +245,6 @@ class GAMSTool(ToolSpecification):
         inputfiles_opt=None,
         outputfiles=None,
         cmdline_args=None,
-        execute_in_work=True,
     ):
         """
 
@@ -280,7 +274,6 @@ class GAMSTool(ToolSpecification):
             inputfiles_opt,
             outputfiles,
             cmdline_args,
-            execute_in_work,
         )
         main_file = includes[0]
         # Add .lst file to list of output files
@@ -390,7 +383,7 @@ class GAMSTool(ToolSpecification):
             return GAMSTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    def create_tool_instance(self, basedir, logger, owner):
+    def create_tool_instance(self, basedir, kill_completed_processes, logger, owner):
         """See base class."""
         return GAMSToolInstance(self, basedir, self._settings, logger, owner)
 
@@ -411,7 +404,6 @@ class JuliaTool(ToolSpecification):
         inputfiles_opt=None,
         outputfiles=None,
         cmdline_args=None,
-        execute_in_work=True,
     ):
         """
         Args:
@@ -440,7 +432,6 @@ class JuliaTool(ToolSpecification):
             inputfiles_opt,
             outputfiles,
             cmdline_args,
-            execute_in_work,
         )
         self.main_prgm = includes[0]
         self.julia_options = OrderedDict()
@@ -465,9 +456,9 @@ class JuliaTool(ToolSpecification):
             return JuliaTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    def create_tool_instance(self, basedir, logger, owner):
+    def create_tool_instance(self, basedir, kill_completed_processes, logger, owner):
         """See base class."""
-        return JuliaToolInstance(self, basedir, self._settings, logger, owner)
+        return JuliaToolInstance(self, basedir, self._settings, kill_completed_processes, logger, owner)
 
 
 class PythonTool(ToolSpecification):
@@ -486,7 +477,6 @@ class PythonTool(ToolSpecification):
         inputfiles_opt=None,
         outputfiles=None,
         cmdline_args=None,
-        execute_in_work=True,
         execution_settings=None,
     ):
         """
@@ -518,7 +508,6 @@ class PythonTool(ToolSpecification):
             inputfiles_opt,
             outputfiles,
             cmdline_args,
-            execute_in_work,
         )
         self.main_prgm = includes[0]
         self.python_options = OrderedDict()
@@ -583,9 +572,9 @@ class PythonTool(ToolSpecification):
             return PythonTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    def create_tool_instance(self, basedir, logger, owner):
+    def create_tool_instance(self, basedir, kill_completed_processes, logger, owner):
         """See base class."""
-        return PythonToolInstance(self, basedir, self._settings, logger, owner)
+        return PythonToolInstance(self, basedir, self._settings, kill_completed_processes, logger, owner)
 
 
 class ExecutableTool(ToolSpecification):
@@ -604,7 +593,6 @@ class ExecutableTool(ToolSpecification):
         inputfiles_opt=None,
         outputfiles=None,
         cmdline_args=None,
-        execute_in_work=True,
         execution_settings=None,
         definition_file_path=None,
     ):
@@ -639,7 +627,6 @@ class ExecutableTool(ToolSpecification):
             inputfiles_opt,
             outputfiles,
             cmdline_args,
-            execute_in_work,
         )
         try:
             self.main_prgm = includes[0]
@@ -664,8 +651,7 @@ class ExecutableTool(ToolSpecification):
     def _includes_main_path_relative(self):
         if not self.path:
             return None
-        else:
-            return super()._includes_main_path_relative()
+        return super()._includes_main_path_relative()
 
     def set_execution_settings(self):
         """Updates old Executable Tool specifications by adding the
@@ -711,6 +697,6 @@ class ExecutableTool(ToolSpecification):
             return ExecutableTool(path=path, settings=settings, logger=logger, **kwargs)
         return None
 
-    def create_tool_instance(self, basedir, logger, owner):
+    def create_tool_instance(self, basedir, kill_completed_processes, logger, owner):
         """See base class."""
         return ExecutableToolInstance(self, basedir, self._settings, logger, owner)
