@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -20,10 +20,11 @@ is filled with all the information from the specification being edited.
 import logging
 import os
 from copy import deepcopy
-from PySide2.QtGui import QStandardItemModel, QStandardItem, QTextDocument, QFont
-from PySide2.QtWidgets import QInputDialog, QFileDialog, QFileIconProvider, QMessageBox, QLabel
-from PySide2.QtCore import Slot, Qt, QFileInfo, QTimer, QItemSelection, QModelIndex
-from spinetoolbox.helpers import busy_effect, open_url
+from operator import methodcaller
+from PySide6.QtGui import QStandardItemModel, QStandardItem, QTextDocument, QFont
+from PySide6.QtWidgets import QInputDialog, QFileDialog, QFileIconProvider, QMessageBox, QLabel
+from PySide6.QtCore import Slot, Qt, QFileInfo, QTimer, QItemSelection, QModelIndex, QItemSelectionModel
+from spinetoolbox.helpers import busy_effect, open_url, same_path
 from spinetoolbox.widgets.custom_qwidgets import ToolBarWidget
 from spinetoolbox.config import STATUSBAR_SS
 from spinetoolbox.project_item.specification_editor_window import (
@@ -77,7 +78,6 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.comboBox_tooltype.setCurrentIndex(-1)
         # if a specification is given, fill the form with data from it
         if specification is not None:
-            self._ui.checkBox_execute_in_work.setChecked(specification.execute_in_work)
             self._ui.lineEdit_args.setText(" ".join(specification.cmdline_args))
             tooltype = specification.tooltype.lower()
             index = next(iter(k for k, t in enumerate(TOOL_TYPES) if t.lower() == tooltype), -1)
@@ -87,12 +87,12 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             specification.set_execution_settings()  # Set default execution settings
             self._init_optional_widget(specification)
         # Init lists
-        programfiles = list(specification.includes) if specification else list()
+        programfiles = list(specification.includes) if specification else []
         # Get first item from programfiles list as the main program file
         main_program_file = programfiles.pop(0) if programfiles else None
-        inputfiles = list(specification.inputfiles) if specification else list()
-        inputfiles_opt = list(specification.inputfiles_opt) if specification else list()
-        outputfiles = list(specification.outputfiles) if specification else list()
+        inputfiles = list(specification.inputfiles) if specification else []
+        inputfiles_opt = list(specification.inputfiles_opt) if specification else []
+        outputfiles = list(specification.outputfiles) if specification else []
         self.includes_main_path = specification.path if specification else None
         self.spec_dict = deepcopy(specification.to_dict()) if specification else dict(item_type=ItemInfo.item_type())
         # Populate lists (this will also create headers)
@@ -111,7 +111,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         parent = self.programfiles_model.index(0, 0)
         index = self.programfiles_model.index(0, 0, parent)
         selection_model = self._ui.treeView_programfiles.selectionModel()
-        selection_model.setCurrentIndex(index, selection_model.Select)
+        selection_model.setCurrentIndex(index, QItemSelectionModel.Select)
 
     def _make_ui(self):
         from ..ui.tool_specification_form import Ui_MainWindow  # pylint: disable=import-outside-toplevel
@@ -128,18 +128,17 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
 
     def _restore_dock_widgets(self):
         docks = (self._ui.dockWidget_program_files, self._ui.dockWidget_program)
-        self.splitDockWidget(*docks, Qt.Horizontal)
+        self.splitDockWidget(*docks, Qt.Orientation.Horizontal)
         width = sum(d.width() for d in docks)
-        self.resizeDocks(docks, [width * x for x in (0.3, 0.7)], Qt.Horizontal)
+        self.resizeDocks(docks, [width * x for x in (0.3, 0.7)], Qt.Orientation.Horizontal)
         docks = (self._ui.dockWidget_program_files, self._ui.dockWidget_io_files)
-        self.splitDockWidget(*docks, Qt.Vertical)
+        self.splitDockWidget(*docks, Qt.Orientation.Vertical)
         height = sum(d.height() for d in docks)
-        self.resizeDocks(docks, [height * x for x in (0.6, 0.4)], Qt.Vertical)
+        self.resizeDocks(docks, [height * x for x in (0.6, 0.4)], Qt.Orientation.Vertical)
 
     @Slot(bool)
     def _update_window_modified(self, _clean):
-        clean = self._undo_stack.isClean()
-        clean &= not any([doc.isModified() for doc in self._programfile_documents.values()])
+        clean = self._undo_stack.isClean() and not any(doc.isModified() for doc in self._programfile_documents.values())
         super()._update_window_modified(clean)
 
     def _make_optional_widget(self, toolspectype):
@@ -160,9 +159,8 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         return None
 
     def _get_optional_widget(self, toolspectype):
-        """Returns the current optional widget for given tool
-        spec type or None if unavailable."""
-        if toolspectype.lower() == "python" or toolspectype.lower() == "executable":
+        """Returns the current optional widget for given tool spec type or None if unavailable."""
+        if toolspectype.lower() in ("python", "executable"):
             optional_widget = self._ui.horizontalLayout_options_placeholder.itemAt(0)
             return optional_widget.widget()
         return None
@@ -191,9 +189,8 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         """
         toolspectype = specification.tooltype.lower()
         opt_widget = self._get_optional_widget(toolspectype)
-        if not opt_widget:
-            return
-        opt_widget.init_widget(specification)
+        if opt_widget:
+            opt_widget.init_widget(specification)
 
     def _make_new_specification(self, spec_name):
         """See base class."""
@@ -207,18 +204,14 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         if main_program_file is None and toolspectype != "executable":
             self.show_error("Please add a main program file")
             return None
-        new_spec_dict = dict()
-        new_spec_dict["name"] = spec_name
-        new_spec_dict["description"] = self._spec_toolbar.description()
-        new_spec_dict["tooltype"] = toolspectype
+        new_spec_dict = {"name": spec_name, "description": self._spec_toolbar.description(), "tooltype": toolspectype}
         main_prgm_dir, main_prgm_file_name = os.path.split(main_program_file) if main_program_file else ("", "")
         if not main_prgm_dir:
             self.includes_main_path = None
         else:
             self.includes_main_path = os.path.abspath(main_prgm_dir)
         self._label_main_path.setText(self.includes_main_path)
-        new_spec_dict["execute_in_work"] = self._ui.checkBox_execute_in_work.isChecked()
-        new_spec_dict["includes"] = [main_prgm_file_name] if main_prgm_file_name else list()
+        new_spec_dict["includes"] = [main_prgm_file_name] if main_prgm_file_name else []
         new_spec_dict["includes"] += self._additional_program_file_list()
         new_spec_dict["inputfiles"] = self._input_file_list()
         new_spec_dict["inputfiles_opt"] = self._opt_input_file_list()
@@ -321,7 +314,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             '<p><span style=" font-weight:600;">Tip</span>: '
             'You can Drag &amp; Drop files and/or directories here from your computer.</p>'
         )
-        self.programfiles_model.setData(index, tool_tip, role=Qt.ToolTipRole)
+        self.programfiles_model.setData(index, tool_tip, role=Qt.ItemDataRole.ToolTipRole)
 
     def _current_main_program_file(self):
         """Returns the full path to the current main program file or None if there's no main program."""
@@ -329,7 +322,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         root_item = self.programfiles_model.itemFromIndex(index)
         if not root_item.hasChildren():
             return None
-        return root_item.child(0).data(Qt.UserRole)
+        return root_item.child(0).data(Qt.ItemDataRole.UserRole)
 
     def _additional_program_file_list(self):
         return self.spec_dict.get("includes", [])[1:]
@@ -346,8 +339,8 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         if file_path:  # Do only if the spec has a main program (Executable Tool specs may not have one)
             item = QStandardItem(os.path.basename(file_path))
             item.setFlags(item.flags() & ~Qt.ItemIsEditable)
-            item.setData(QFileIconProvider().icon(QFileInfo(file_path)), Qt.DecorationRole)
-            item.setData(file_path, Qt.UserRole)
+            item.setData(QFileIconProvider().icon(QFileInfo(file_path)), Qt.ItemDataRole.DecorationRole)
+            item.setData(file_path, Qt.ItemDataRole.UserRole)
             root_item.appendRow(item)
             QTimer.singleShot(0, self._push_change_main_program_file_command)
 
@@ -400,10 +393,10 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             index (QModelIndex): index in program files model
             dirty (bool)
         """
-        basename = os.path.basename(index.data(Qt.UserRole))
+        basename = os.path.basename(index.data(Qt.ItemDataRole.UserRole))
         if dirty:
             basename += "*"
-        self.programfiles_model.setData(index, basename, role=Qt.DisplayRole)
+        self.programfiles_model.setData(index, basename, role=Qt.ItemDataRole.DisplayRole)
 
     def init_io_file_list(self):
         for name in ("Input files", "Optional input files", "Output files"):
@@ -443,7 +436,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         root_item.removeRows(0, root_item.rowCount())
         for name in names:
             item = QStandardItem(name)
-            item.setData(QFileIconProvider().icon(QFileInfo(name)), Qt.DecorationRole)
+            item.setData(QFileIconProvider().icon(QFileInfo(name)), Qt.ItemDataRole.DecorationRole)
             root_item.appendRow(item)
         self._ui.treeView_io_files.expand(index)
 
@@ -488,7 +481,6 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         # Push undo commands
         self._ui.comboBox_tooltype.currentIndexChanged.connect(self._push_change_tooltype_command)
         self._ui.comboBox_tooltype.currentIndexChanged.connect(self._show_optional_widget)
-        self._ui.checkBox_execute_in_work.toggled.connect(self._push_change_execute_in_work_command)
         self._ui.lineEdit_args.editingFinished.connect(self._push_change_args_command)
         self.io_files_model.dataChanged.connect(self._push_io_file_renamed_command)
         # Selection changed
@@ -519,23 +511,19 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         Sets the default execution settings into self.spec_dict when a Python
         Tool spec type is selected. Removes execution settings from self.spec_dict
         when any other Tool spec type is selected."""
-        if "execution_settings" in self.spec_dict.keys():
+        if "execution_settings" in self.spec_dict:
             self.spec_dict.pop("execution_settings")  # Clear execution settings
         if spec_type == "python":
-            self.spec_dict["execution_settings"] = dict()
-            default_use_jupyter_console = bool(
-                int(self._toolbox.qsettings().value("appSettings/usePythonKernel", defaultValue="0"))
-            )
-            default_k_spec = self._toolbox.qsettings().value("appSettings/pythonKernel", defaultValue="")
-            self.spec_dict["execution_settings"]["use_jupyter_console"] = default_use_jupyter_console
-            self.spec_dict["execution_settings"]["kernel_spec_name"] = default_k_spec
-            self.spec_dict["execution_settings"]["env"] = ""
-            default_python_exe = self._toolbox.qsettings().value("appSettings/pythonPath", defaultValue="")
-            self.spec_dict["execution_settings"]["executable"] = default_python_exe
+            self.spec_dict["execution_settings"] = {
+                "use_jupyter_console": bool(
+                    int(self._toolbox.qsettings().value("appSettings/usePythonKernel", defaultValue="0"))
+                ),
+                "kernel_spec_name": self._toolbox.qsettings().value("appSettings/pythonKernel", defaultValue=""),
+                "env": "",
+                "executable": self._toolbox.qsettings().value("appSettings/pythonPath", defaultValue=""),
+            }
         elif spec_type == "executable":
-            self.spec_dict["execution_settings"] = dict()
-            self.spec_dict["execution_settings"]["cmd"] = ""
-            self.spec_dict["execution_settings"]["shell"] = ""
+            self.spec_dict["execution_settings"] = {"cmd": "", "shell": ""}
 
     @Slot(int)
     def _push_change_kernel_spec_command(self, index):
@@ -617,19 +605,6 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         index = next(iter(k for k, t in enumerate(opt_widget.shells) if t.lower() == value), 0)
         opt_widget.ui.comboBox_shell.setCurrentIndex(index)
 
-    @Slot(bool)
-    def _push_change_execute_in_work_command(self, new_value):
-        old_value = self.spec_dict.get("execute_in_work", True)
-        if new_value == old_value:
-            return
-        self._undo_stack.push(
-            ChangeSpecPropertyCommand(self._set_execute_in_work, new_value, old_value, "change execute in work")
-        )
-
-    def _set_execute_in_work(self, value):
-        self.spec_dict["execute_in_work"] = value
-        self._ui.checkBox_execute_in_work.setChecked(value)
-
     @Slot()
     def _push_change_args_command(self):
         old_value = self.spec_dict.get("cmdline_args", [])
@@ -666,7 +641,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             old_program_files = [os.path.join(self.includes_main_path, f) for f in old_program_files]
         old_value = old_program_files[0] if old_program_files else ""
         if new_value is not None:  # Enables removing main program
-            if os.path.normcase(old_value) == os.path.normcase(new_value):
+            if same_path(old_value, new_value):
                 return
             new_program_files = old_program_files.copy()
             if new_program_files:
@@ -674,7 +649,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             elif new_value:
                 new_program_files.append(new_value)
         else:
-            new_program_files = list()
+            new_program_files = []
         self._undo_stack.push(
             ChangeSpecPropertyCommand(
                 self._set_program_files, new_program_files, old_program_files, "change main program file"
@@ -856,7 +831,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
                 continue
             if self.includes_main_path is not None:
                 common_prefix = os.path.commonprefix([os.path.abspath(self.includes_main_path), os.path.abspath(file)])
-                if os.path.normcase(common_prefix) != os.path.normcase(self.includes_main_path):
+                if not same_path(common_prefix, self.includes_main_path):
                     invalid.append(os.path.basename(file))
                     continue
             valid_files.append(file)
@@ -911,7 +886,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             self._show_status_bar_msg("No program files to remove")
             return
         old_program_files = self.spec_dict.get("includes", [])
-        new_program_files = list()
+        new_program_files = []
         if self.includes_main_path is not None:
             old_program_files = [os.path.join(self.includes_main_path, f) for f in old_program_files]
         self._undo_stack.push(
@@ -1041,7 +1016,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
             bottom_right (QModelIndex): bottom right index to renamed files
             roles (list of int):
         """
-        if Qt.DisplayRole not in roles:
+        if Qt.ItemDataRole.DisplayRole not in roles:
             return
         parent_index = top_left.parent()
         if parent_index.row() == 0:
@@ -1086,7 +1061,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         if not indexes:  # Nothing selected
             self._show_status_bar_msg("Please select the input files to remove")
             return
-        removed_files = {ind.data(Qt.DisplayRole) for ind in indexes}
+        removed_files = {ind.data(Qt.ItemDataRole.DisplayRole) for ind in indexes}
         old_files = self.spec_dict.get("inputfiles", [])
         new_files = [f for f in old_files if f not in removed_files]
         self._undo_stack.push(
@@ -1101,7 +1076,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         if not indexes:  # Nothing selected
             self._show_status_bar_msg("Please select the optional input files to remove")
             return
-        removed_files = {ind.data(Qt.DisplayRole) for ind in indexes}
+        removed_files = {ind.data(Qt.ItemDataRole.DisplayRole) for ind in indexes}
         old_files = self.spec_dict.get("inputfiles_opt", [])
         new_files = [f for f in old_files if f not in removed_files]
         self._undo_stack.push(
@@ -1118,7 +1093,7 @@ class ToolSpecificationEditorWindow(SpecificationEditorWindowBase):
         if not indexes:  # Nothing selected
             self._show_status_bar_msg("Please select the output files to remove")
             return
-        removed_files = {ind.data(Qt.DisplayRole) for ind in indexes}
+        removed_files = {ind.data(Qt.ItemDataRole.DisplayRole) for ind in indexes}
         old_files = self.spec_dict.get("outputfiles", [])
         new_files = [f for f in old_files if f not in removed_files]
         self._undo_stack.push(
@@ -1173,24 +1148,24 @@ def _build_tree(root, components):
     leafs = []
     for parent, children in components.items():
         item = QStandardItem(parent)
-        item.setData(parent, Qt.UserRole)
+        item.setData(parent, Qt.ItemDataRole.UserRole)
         item.setFlags(item.flags() & ~Qt.ItemIsEditable)
         _build_tree(item, children)
         if item.hasChildren():
             nodes.append(item)
-            item.setData(QFileIconProvider().icon(QFileIconProvider.Folder), Qt.DecorationRole)
+            item.setData(QFileIconProvider().icon(QFileIconProvider.IconType.Folder), Qt.ItemDataRole.DecorationRole)
         else:
-            item.setData(QFileIconProvider().icon(QFileInfo(parent)), Qt.DecorationRole)
+            item.setData(QFileIconProvider().icon(QFileInfo(parent)), Qt.ItemDataRole.DecorationRole)
             leafs.append(item)
-    for item in sorted(nodes, key=lambda x: x.text()):
+    for item in sorted(nodes, key=methodcaller("text")):
         root.appendRow(item)
-    for item in sorted(leafs, key=lambda x: x.text()):
+    for item in sorted(leafs, key=methodcaller("text")):
         root.appendRow(item)
 
 
 def _path_components_from_index(index):
     components = []
     while index.parent().isValid():
-        components.insert(0, index.data(Qt.UserRole))
+        components.insert(0, index.data(Qt.ItemDataRole.UserRole))
         index = index.parent()
     return components
