@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -20,12 +20,4 @@ from spinetoolbox.project_item_icon import ProjectItemIcon
 
 
 class ViewIcon(ProjectItemIcon):
-    def __init__(self, toolbox, icon, icon_color):
-        """View icon for the Design View.
-
-        Args:
-            toolbox (ToolBoxUI): QMainWindow instance
-            icon (str): icon resource path
-            icon_color (QColor): Icon's color
-        """
-        super().__init__(toolbox, icon, icon_color)
+    """View icon for the Design View."""
