@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Items.
 # Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -16,20 +16,19 @@ View properties widget.
 :date:   12.9.2019
 """
 
-from PySide2.QtCore import Slot, QPoint
+from PySide6.QtCore import Slot, QPoint
 from spinetoolbox.widgets.properties_widget import PropertiesWidgetBase
 from .custom_menus import ViewRefsContextMenu, ViewSelectionsContextMenu
 
 
 class ViewPropertiesWidget(PropertiesWidgetBase):
-    """Widget for the View Item Properties.
-
-    Args:
-        toolbox (ToolboxUI): The toolbox instance where this widget should be embedded
-    """
+    """Widget for the View Item Properties."""
 
     def __init__(self, toolbox):
-        """Init class."""
+        """
+        Args:
+            toolbox (ToolboxUI): The toolbox instance where this widget should be embedded
+        """
         from ..ui.view_properties import Ui_Form  # pylint: disable=import-outside-toplevel
 
         super().__init__(toolbox)
